
/*
 *     Example of a connection-oriented server using TCP protocol.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define SERVER_TCP_PORT     7000
#define MAX_MSG_SIZE        2048

main()
{
     int sockfd, newsockfd, clilen, childpid, len;
     struct sockaddr_in  serv_addr, client_addr;
     char message[MAX_MSG_SIZE];

     /*
      * Open a TCP socket (an Internet stream socket).
      */

     if ( ( sockfd = socket(AF_INET, SOCK_STREAM, 0) ) < 0 ) {
        fprintf(stderr, "server: can't open stream socket.");
        exit(1);
     }

     /*
      * Bind the local address of server.
      */

     bzero((char *) &serv_addr, sizeof(serv_addr));      /* Initialize      */
     serv_addr.sin_family      = AF_INET;                /* Internet family */
     serv_addr.sin_port        = htons(SERVER_TCP_PORT); /* Port number     */
     serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);      /* Netid / Hostid  */

     if ( bind(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0 ) {
        fprintf(stderr, "server: can't bind local address.");
        exit(1);
     }

     /*
      * Ready to receive connections. 
      * SOMAXCONN : Maximum queue length of pending requests for connections.
      */

     listen(sockfd, SOMAXCONN);

     /*
      * Wait for a connection from a client process.
      * This is an example of concurrent server.
      */

     while ( 1 ) {
        clilen = sizeof(client_addr);
        newsockfd = accept(sockfd, (struct sockaddr *) &client_addr, &clilen);
        if ( newsockfd < 0 ) {
           fprintf(stderr, "server: can't accept a connection.");
           exit(1);
        }
        if ( (childpid = fork()) < 0 ) {
           fprintf(stderr, "server: can't fork child process.");
           exit(1);
        }
        else if ( childpid == 0 ) { /* This is child process    */
           close(sockfd);           /* Close original socket    */

           /*
            * Do a specific task for the server.
            */

           if ( ( len = read(newsockfd, message, MAX_MSG_SIZE) ) < 0 ) {
              fprintf(stderr, "server: can't read from the socket.\n");
              exit(1);
           }
           strcpy(&message[6], "Client !\n");
           if ( write(newsockfd, message, len) != len ) {
              fprintf(stderr, "server: can't write into the socket.");
              exit(1);
           }
           exit(0);
        }
        close(newsockfd);           /* This is parent process   */
     }
}

