/*************************************************************************

	File Name	: CFD.java

	This is CFD(Computational Fluid Dynamincs) application.

	This application contains 9 files

		CFD.java : The main applet
		PlotCanvas.java : Graph Part
		IsoCurve.java
		OutlinePanel.java : Making outline
		Status.java : Showing the position and vector of mouse
			      point on the graph
		NonUniformFlow.java : Non uniform data
		UniformFlow.java : Uniform data
		PlotType.java : plot type
		About.java : about this applet

**************************************************************************/

import java.awt.*;
import java.util.*;
import java.lang.*;
import java.awt.event.*;
import java.applet.Applet;

import webwisdom.tango.*;
import netscape.javascript.JSObject;

public class CFD extends Applet implements TDataListener, TControlListener {

	// TANGO variables
	TAgent agent = null;
	boolean	isMaster = true;
	String	MyName = null;

	private int channel = 20;
	private boolean javaOnly = true;
	boolean inTango = true;
	boolean fromScript = false;

	Status status;
	PlotCanvas plotCanvas;
	NonUniformFlow non_uniform_flow;
	UniformFlow uniform_flow;
	PlotType plot_type;
	About about;

	// message variables
	String messageSpace	= " ";

	String messageReceive	= null;
	String messageTotalData	= null;

	final String messageInitial	= "Initial";

	final int messageAll		= 0;
	final int messageDummy		= 0;

	final int messageNonUniform	= 1;
	final int messageAdd		= 11;
	final int messageDelete		= 12;
	final int messageClear		= 13;
	final int messageHistory	= 14;

	final int messageUniform	= 2;
	final int messageApply		= 21;
	final int messageDisable	= 22;

	final int messagePlotType	= 3;
	final int messageType		= 31;

	final int messageMouse		= 4;
	final int messageDOWN		= 41;
	final int messageUP		= 42;
	final int messageDRAG		= 43;

	final String messageEnd		= "kimd";

	// data for this applet
	int	numOfSource = 0;
	int []sourceIndex = new int[20];
	double []sourceStrength = new double[20];
	double []SX = new double[20];
	double []SY = new double[20];
	boolean	uniformflow_main
	double	direction_main;
	double	velocity_main;
	int	plotType_main;

	boolean debugApplet = false;

	public void printLine(String printData) {
		if ( debugApplet ) {
			System.out.println(printData);
		}
	}

       /*****************************************************************
	*
	*	method	: init()
	*
	*	Function : init of this applet
	*
	*****************************************************************/
	public void init() {
		initThis();
		initTango();
		initApplet();
	}

	public void initTango() {

		String arg = null;
		try {
			arg = getParameter("interface");
			inTango = true;
		}
		catch(Exception e) {
			inTango = false;
		}

		if (arg == null) {
			inTango = false;
		}
		printLine("<CFD> arg : " + arg);
		printLine("<CFD> inTango : " + inTango);

		if (inTango) {
			setTango(arg);
		}
	}

	public void initThis() {

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);

		setBackground(Color.lightGray);

		c.fill = GridBagConstraints.CENTER;

		// plot canvas
		c.weightx = 0;
		c.weighty = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		Dimension d = new Dimension(320, 240);
		plotCanvas = new PlotCanvas(this, d);
		gridbag.setConstraints(plotCanvas, c);
		add(plotCanvas);

		c.fill = GridBagConstraints.BOTH;

		//graph status
		c.weightx = 1;
		c.weighty = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		OutlinePanel out_status = new OutlinePanel(this, "VALUES UNDER MOUSE", 10, 5, 5, 25);
		out_status.add("Center", status = new Status(this));
		gridbag.setConstraints(out_status, c);
		add(out_status);

		//Non Uniform Flow
		c.weightx = 1;
		c.weighty = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		OutlinePanel out_nonuniform = new OutlinePanel(this, "NON UNIFORM FLOW", 10, 5, 5, 25);
		out_nonuniform.add("Center", non_uniform_flow = new NonUniformFlow(this));
		gridbag.setConstraints(out_nonuniform, c);
		add(out_nonuniform);

		// Uniform Flow
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		OutlinePanel out_uniform = new OutlinePanel(this, "UNIFORM FLOW", 10, 5, 5, 25);
		out_uniform.add("Center", uniform_flow = new UniformFlow(this));
		gridbag.setConstraints(out_uniform, c);
		add(out_uniform);

		// Plot Type
		c.weightx = 1;
		c.weighty = 1;
		c.gridwidth = GridBagConstraints.RELATIVE;
		OutlinePanel out_plot_type = new OutlinePanel(this, "PLOT TYPE", 10, 5, 5, 25);
		out_plot_type.add("Center", plot_type = new PlotType(this));
		gridbag.setConstraints(out_plot_type, c);
		add(out_plot_type);


		// About
		c.weightx = 1;
		c.weighty = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		OutlinePanel out_about = new OutlinePanel(this, "ABOUT", 10, 5, 5, 25);
		out_about.add("Center", about = new About(this));
		gridbag.setConstraints(out_about, c);
		add(out_about);

		NonUniform_Clear();
	}

	public void initApplet() {

		if ( inTango ) {
			// slave sends the initial request
			if ( !isMaster) {
				String message =  messageInitial + messageSpace
						+ Integer.toString(numOfSource) + messageSpace
						+ messageEnd;
				send2Master(message);
			}

			ButtonSetEnabled(isMaster);
		}
		else {
			isMaster = true;
			ButtonSetEnabled(isMaster);
		}
	}

	public void startThis() {
		printLine("<CFD> startThis()");
	}

	public void commJava() {

		try {
			agent = new TAgent(new TLAgentApplet(this));
		}
		catch(TangoException e) {
			e.printStackTrace();
			System.out.println("<CFD> agent is not proprely assigned");
		}

	}

	public void commJavaScript() {

		JSObject jsWin = JSObject.getWindow(this);
		Object[] args = new Object[1];
		args[0] = this;
		jsWin.call("appletReadyJS", args);
	}

	public void setTango(String arg) {
                if ( arg.equals("javaApplet") ) {
			fromScript = false;
			commJava();
		}
		else if ( arg.equals("javaScript") ) {
			fromScript = true;
			commJavaScript();
		}

		while ( agent == null ) {
			printLine("<CFD> waiting for the agent to be assigned");
		}
		printLine("<CFD> agent assigned");

		//dataListener = agent.addTDataListener(channel, this);
		//controlListener = agent.addTControlListener(this);
		agent.addTDataListener(channel, this);
		agent.addTControlListener(this);
		isMaster = agent.isMaster();
	}

	public void setTango(String args[]) {

		try {
			agent = new TAgent(new TLAgentApplication(args));
		}
		catch(TangoException e) {
			e.printStackTrace();
			System.out.println("<CFD> agent is not proprely assigned");
		}

		printLine("<CFD> Application2 agent assigned");

		agent.addTDataListener(channel, this);
		printLine("<CFD> Data added");

		agent.addTControlListener(this);
		printLine("<CFD> Control added");

//		isMaster = agent.isMaster();
		printLine("<CFD> Data Control added");
	}

	public void initTangoJS(TAgent a, Number ch, Number ma) {

		channel = ((Number) ch).intValue();

		if ( ((Number) ma).intValue() ==  0 ) {
			isMaster = true;
		}
		else {
			isMaster = false;
		}

		agent = a;
	}

	public void channelReceive(int ch, byte b[]) {
		if ( channel != ch ) {
			return;
		}

		receive(b);
	}

	public void masterChangedJS(Number ma, String masterName) {
		if ( ((Number) ma).intValue() ==  0 ) {
			isMaster = true;
		}
		else {
			isMaster = false;
		}

		ButtonSetEnabled(isMaster);
	}

       /*****************************************************************
	*
	*	method	 : masterchanged()
	*	Input    : boolean master, String masterName
	*	Output   : none
	*	Function : Built-in function for TControlListener
	*
	*****************************************************************/
	public void masterChanged(boolean master, String masterName) {

		isMaster = master;

		ButtonSetEnabled(isMaster);

	}

       /*****************************************************************
	*
	*	method	 : participantJoined()
	*	Input    : String participantName
	*	Output   : none
	*	Function : Built-in function for TControlListener
	*
	*****************************************************************/
	public void participantJoined(String participantName) {
	}

       /*****************************************************************
	*
	*	method	 : participantLeft()
	*	Input    : String participantName
	*	Output   : none
	*	Function : Built-in function for TControlListener
	*
	*****************************************************************/
	public void participantLeft(String participantName) {
	}

       /*****************************************************************
	*
	*	method	 : audioRequested()
	*	Input    : boolean isREquested
	*	Output   : none
	*	Function : Built-in function for TControlListener
	*
	*****************************************************************/
	public void audioRequested(boolean isREquested) {
	}

       /*****************************************************************
	*
	*	method	 : stringToByte()
	*	Input    : String s
	*	Output   : byte[]
	*	Function : convert String to Byte
	*
	*****************************************************************/
	private byte[] stringToBytes(String s) {
		int l = s.length();
		byte[] t = new byte[l];
		s.getBytes(0, l, t, 0);
		return t;
	}

       /*****************************************************************
	*
	*	method	 : bytesToString()
	*	Input    : byte t[]
	*	Output   : String
	*	Function : convert byte to String
	*
	*****************************************************************/
	private String bytesToString(byte t[]) {
		return new String(t, 0);
	}

       /*****************************************************************
	*
	*	method	 : destroy()
	*	Input    : none
	*	Output   : none
	*	Function : 
	*
	*****************************************************************/
	public void destroy() {
		printLine("<CFD> inside destroy()");

		if ( fromScript ) {
			agent.removeTControlListener(this);
			agent.removeTDataListener(this);
			printLine("<CFD> data and control removed");
			return;
		}

		if (agent != null) {
			agent.exit();
		}
		super.destroy();
	}

       /*****************************************************************
	*
	*	method	 : ButtonSetEnabled()
	*	Input    : boolean setButtons
	*	Output   : none
	*	Function : set the buttons of this applet regarding
	*		   master or slave.
	*
	*****************************************************************/
	void ButtonSetEnabled(boolean setButtons) {

		non_uniform_flow.SourceType.setEnabled(setButtons);
		non_uniform_flow.XPos.setEnabled(setButtons);
		non_uniform_flow.YPos.setEnabled(setButtons);
		non_uniform_flow.Strength.setEnabled(setButtons);
		non_uniform_flow.addButton.setEnabled(setButtons);
		non_uniform_flow.deleteButton.setEnabled(setButtons);
		non_uniform_flow.clearButton.setEnabled(setButtons);

		uniform_flow.enableButton.setEnabled(setButtons);
		uniform_flow.disableButton.setEnabled(setButtons);

		if ( isMaster ) { 
			if ( uniformflow_main ) {
				uniform_flow.applyButton.setEnabled(true);
				uniform_flow.Direction.setEnabled(true);
				uniform_flow.Velocity.setEnabled(true);
			}
			else {
				uniform_flow.applyButton.setEnabled(false);
				uniform_flow.Direction.setEnabled(false);
				uniform_flow.Velocity.setEnabled(false);
			}

		}
		else {
			uniform_flow.applyButton.setEnabled(setButtons);
			uniform_flow.Direction.setEnabled(setButtons);
			uniform_flow.Velocity.setEnabled(setButtons);
		}

		plot_type.plotType.setEnabled(setButtons);

	}

       /*****************************************************************
	*
	*	method	 : send2Master()
	*	Input    : String messageToBeSent
	*	Output   : none
	*	Function : send a message to the master
	*
	*****************************************************************/
	void send2Master(String messageToBeSent) {

		// send the message
		if ( agent != null ) {
			send_message(messageToBeSent);
		}
	}

       /*****************************************************************
	*
	*	method	 : sendHistory()
	*	Input    : none
	*	Output   : none
	*	Function : send the all data
	*
	*****************************************************************/
	void sendHistory() {

		int i;
		String InitialData = null;

		InitialData	= Integer.toString(messageAll) + messageSpace
				+ Integer.toString(numOfSource) + messageSpace
				+ Integer.toString(messageDummy) + messageSpace;


		for ( i = 0 ; i < numOfSource ; i++ ) {
			InitialData	= InitialData + messageSpace
					+ Integer.toString(sourceIndex[i]) + messageSpace
					+ Double.toString(SX[i]) + messageSpace
					+ Double.toString(SY[i]) + messageSpace
					+ Double.toString(sourceStrength[i]) + messageSpace;
		}

		int tmp;
		if ( uniformflow_main ) {
			tmp = 1;
		}
		else {
			tmp = 0;
		}

		InitialData	= InitialData
				+ Integer.toString(tmp) + messageSpace
				+ Double.toString(direction_main) + messageSpace
				+ Double.toString(velocity_main) + messageSpace
				+ Integer.toString(plotType_main) + messageSpace;

		InitialData	= InitialData + messageEnd;

		if ( agent != null ) {
			send_message(InitialData);
		}
	}

       /*****************************************************************
	*
	*	method	 : send_message()
	*	Input    : String messageToBeSent
	*	Output   : none
	*	Function : send the message and draw it on the Graph
	*
	*****************************************************************/
	void send_message(String messageToBeSent) {

		if (inTango) {
			printLine("<CFD> send : " + messageToBeSent);
			agent.send(channel, stringToBytes(messageToBeSent));
		}

		// draw the graph
		drawGraph(messageToBeSent);
	}

       /*****************************************************************
	*
	*	method	 : receive()
	*	Input    : byte b[]
	*	Output   : none
	*	Function : built-in function of TDataListener
	*
	*****************************************************************/
	public void receive(byte b[]) {

		String recvMessage = bytesToString(b);

		printLine("<CFD> received : " + recvMessage);

		if ( isMaster ) {

			StringTokenizer selectedItem = null;
			selectedItem = new StringTokenizer(recvMessage);
			String head = selectedItem.nextToken();

			if ( head.equals(messageInitial) ) {
				// sends all the history data
				sendHistory();
			}
		}
		else { // slave
			// draw the graph
			drawGraph(recvMessage);
		}
	}

       /*****************************************************************
	*
	*	method	 : drawGraph()
	*	Input    : String messageToBeDrwan
	*	Output   : none
	*	Function : draw the graph or update the data
	*
	*****************************************************************/
	void drawGraph(String messageToBeDrawn) {

		int i;
		StringTokenizer selectedItem = null;
		String	firstToken;
		int	Head;
		int	totalData;
		int	Function;

		String	End = null;

		// for add function
		int	sourceType;
		double	Xcoordinate;
		double	Ycoordinate;
		double	strength;

		// for delete function
		int	itemIndex;

		// for clear function
		// none

		// for history function
		// same as delete function

		// for uniform flow apply function
		double	direction;
		double	velocity;

		// for uniform flow disable function
		// none

		// for plot type changing function
		int	plotType;

		selectedItem = new StringTokenizer(messageToBeDrawn);

		// normal data or initial request
		firstToken = selectedItem.nextToken();

		// initial request : do nothing
		if ( firstToken.equals(messageInitial) ) {
			return;
		}

		Head = (Integer.valueOf(firstToken)).intValue();
		totalData = (Integer.valueOf(selectedItem.nextToken())).intValue();
		Function = (Integer.valueOf(selectedItem.nextToken())).intValue();

		// error check
		if ( (numOfSource != totalData) && (Head != messageAll) ) {
			return;
		}

		if ( (Head == messageAll) && (numOfSource > 0) ) {
			return;
		}

		switch(Head) {

		case messageAll:

			numOfSource = totalData;
			for ( i = 0 ; i < numOfSource ; i++ ) {
				sourceIndex[i] = (Integer.valueOf(selectedItem.nextToken())).intValue();
				SX[i] = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				SY[i] = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				sourceStrength[i] = (Double.valueOf(selectedItem.nextToken())).doubleValue();
			}

			int tmp = (Integer.valueOf(selectedItem.nextToken())).intValue();
			if ( tmp == 1 ) {
				uniformflow_main = true;
			}
			else {
				uniformflow_main = false;
			}
			direction_main = (Double.valueOf(selectedItem.nextToken())).doubleValue();
			velocity_main = (Double.valueOf(selectedItem.nextToken())).doubleValue();
			plotType_main = (Integer.valueOf(selectedItem.nextToken())).intValue();

			End = selectedItem.nextToken();

			// error check
			if ( !End.equals(messageEnd) ) {
				break;
			}

			All_Add();

			break;

		case messageNonUniform:

			switch(Function) {
			case messageAdd:

				sourceType = (Integer.valueOf(selectedItem.nextToken())).intValue();
				Xcoordinate = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				Ycoordinate = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				strength = (Double.valueOf(selectedItem.nextToken())).doubleValue();

				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				NonUniform_Add(sourceType, Xcoordinate, Ycoordinate, strength); 

				break;

			case messageDelete:
				itemIndex = (Integer.valueOf(selectedItem.nextToken())).intValue();
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				NonUniform_Delelte(itemIndex);

				break;

			case messageClear:
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				NonUniform_Clear();

				break;

			case messageHistory:
				itemIndex = (Integer.valueOf(selectedItem.nextToken())).intValue();
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				NonUniform_History(itemIndex);

				break;
			}
			break;

		case messageUniform:

			switch(Function) {
			case messageApply:
				direction = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				velocity = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				Uniform_Apply(direction, velocity);

				break;

			case messageDisable:
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				Uniform_Disable();

				break;
			}
			break;

		case messagePlotType:

			switch(Function) {
			case messageType:
				plotType = (Integer.valueOf(selectedItem.nextToken())).intValue(); 
				End = selectedItem.nextToken();

				// error check
				if ( !End.equals(messageEnd) ) {
					break;
				}

				PlotType_changed(plotType);

				break;
			}
			break;

		case messageMouse:

			switch(Function) {
			// same action when mouse down and mouse drag
			case messageDOWN: 

			case messageDRAG:
				double temp1, temp2, temp3, temp4;

				temp1 = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				temp2 = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				temp3 = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				temp4 = (Double.valueOf(selectedItem.nextToken())).doubleValue();
				Graphics g = plotCanvas.getGraphics();
				plotCanvas.drawVector(g, temp1, temp2, temp3, temp4);
				break;

			case messageUP:
				break;

			}
			break;

		}
	}

       /*****************************************************************
	*
	*	method	 : All_Add()
	*	Input    : none
	*	Output   : none
	*	Function : Action when the slave receives 
	*		   the initial data from the master
	*
	*****************************************************************/
	void All_Add() {

		int i;
		String history = null;

		for ( i = 0 ; i < numOfSource ; i++ ) {
			history = null;
			history = non_uniform_flow.SourceType.getItem(sourceIndex[i]) 
				+ ", Pos("
				+ SX[i] + ", "
				+ SY[i] + ") "
				+ "Strength("
				+ sourceStrength[i] + ")";

			non_uniform_flow.ItemList.addItem(history);
		}

		if ( uniformflow_main ) {
			uniform_flow.enableButton.setState(true);
			uniform_flow.disableButton.setState(false);
			uniform_flow.Direction.setText(Double.toString(direction_main));
			uniform_flow.Velocity.setText(Double.toString(velocity_main));
		}
		else {
			uniform_flow.enableButton.setState(false);
			uniform_flow.disableButton.setState(true);
		}

		plot_type.plotType.select(plotType_main);

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : NonUniform_Add()
	*	Input    : int sourceType, double Xcoordinate,
	*		   double Ycoordinate, double strength
	*	Output   : none
	*	Function : Action when the ADD buttone is clicked
	*
	*****************************************************************/
	void NonUniform_Add(
			int sourceType, double Xcoordinate, 
			double Ycoordinate, double strength) {

		String history = null;

		history = non_uniform_flow.SourceType.getItem(sourceType) 
				+ ", Pos("
				+ Double.toString(Xcoordinate) + ", "
				+ Double.toString(Ycoordinate) + ") "
				+ "Strength("
				+ Double.toString(strength) + ")";

		sourceIndex[numOfSource] = sourceType;
		sourceStrength[numOfSource] = strength;
		SX[numOfSource] = Xcoordinate;
		SY[numOfSource] = Ycoordinate;

		numOfSource++;

		non_uniform_flow.ItemList.addItem(history);

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : NonUniform_Delete()
	*	Input    : int itemIndex
	*	Output   : none
	*	Function : Action when the DELETE buttone is clicked
	*
	*****************************************************************/
	void NonUniform_Delelte(int itemIndex) {

		int i;

		non_uniform_flow.ItemList.delItem(itemIndex);

		for ( i = itemIndex ; i < numOfSource - 1 ; i++ ) {
			sourceIndex[i] = sourceIndex[i+1];
			sourceStrength[i] = sourceStrength[i+1];
			SX[i] = SX[i+1];
			SY[i] = SY[i+1];
		}

		numOfSource--;

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : NonUniform_Clear()
	*	Input    : none
	*	Output   : none
	*	Function : Action when the CLEAR buttone is clicked
	*
	*****************************************************************/
	void NonUniform_Clear() {

		String zero = "0";
		String one = "1";

		non_uniform_flow.ItemList.clear();
		non_uniform_flow.XPos.setText(zero);
		non_uniform_flow.YPos.setText(zero);
		non_uniform_flow.Strength.setText(one);

		numOfSource = 0;

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : NonUniform_History()
	*	Input    : int itemIndex
	*	Output   : none
	*	Function : Action when one of the data in the List is clicked
	*
	*****************************************************************/
	void NonUniform_History(int itemIndex) {

		String	itemString;
		StringTokenizer selectedItem = null;

		int sourceType;
		String type = null;
		String xPos = null;
		String yPos = null;
		String strength = null;

		itemString = non_uniform_flow.ItemList.getItem(itemIndex);

		non_uniform_flow.ItemList.select(itemIndex);

		String newItemString = itemString.replace('(', ' ').replace(')', ' ').replace(',', ' ');

		selectedItem = new StringTokenizer(newItemString);

		type		= selectedItem.nextToken();
		// string "Pos"
		String dummy	= selectedItem.nextToken();
		xPos		= selectedItem.nextToken();
		yPos		= selectedItem.nextToken();
		// string "Stength"
		dummy		= selectedItem.nextToken();
		strength	= selectedItem.nextToken();

		non_uniform_flow.SourceType.select(type);
		non_uniform_flow.XPos.setText(xPos);
		non_uniform_flow.YPos.setText(yPos);
		non_uniform_flow.Strength.setText(strength);

	}

       /*****************************************************************
	*
	*	method	 : Uniform_Apply()
	*	Input    : double direction, double velocity
	*	Output   : none
	*	Function : Action when uniform flow APPLY button is clicked
	*
	*****************************************************************/
	void Uniform_Apply(double direction, double velocity) {

		uniformflow_main = true;

		direction_main = direction;
		velocity_main = velocity;

		uniform_flow.enableButton.setState(true);
		uniform_flow.disableButton.setState(false);
		uniform_flow.Direction.setText(Double.toString(direction));
		uniform_flow.Velocity.setText(Double.toString(velocity));

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : Uniform_Disable()
	*	Input    : none
	*	Output   : none
	*	Function : Action when uniform flow DISABLE button is clicked
	*
	*****************************************************************/
	void Uniform_Disable() {

		uniformflow_main = false;	

		uniform_flow.enableButton.setState(false);
		uniform_flow.disableButton.setState(true);

		plotCanvas.calVelocity();
		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : PlotType_changed()
	*	Input    : int plotType
	*	Output   : none
	*	Function : Action when the plot type is changed
	*
	*****************************************************************/
	void PlotType_changed(int plotType) {

		plot_type.plotType.select(plotType);

		plotType_main = plotType;

		plotCanvas.invalidate();
		plotCanvas.repaint();
	}

       /*****************************************************************
	*
	*	method	 : main()
	*
	*	Function : main function of this applet
	*
	*****************************************************************/
	public static void main(String args[]) {
		Frame f = new Frame("CFD Applet");
		CFD cfd = new CFD();

		cfd.initThis();

		try {
			System.out.println("<CFD> arg[0] : " + args[0]);
			cfd.inTango = true;
		}
		catch (Exception e) {
			cfd.inTango = false;
		}

		System.out.println("<CFD> inTango : " + cfd.inTango);

		if (cfd.inTango) {
			cfd.setTango(args);
		}

		cfd.initApplet();

		f.add("Center", cfd);
		f.setSize(320, 500);
		f.show();
	}

}
