import java.applet.Applet;
import java.awt.*;
import java.util.*;

class PlotCanvas extends Canvas {

	CFD smain;
	Dimension dim;
	Dimension areaSize;
	int NUMCONTOURLEVEL = 50;
	int GRIDSTEP = 1;
	int numCurves;
	boolean mousePosition;
	boolean MD = false;
	boolean ContourOK = false;
	int ccx, ccy, cax, cay, ch, cw;
	double uup, vvp, xxp, yyp, sip;
	int gridx, gridy;
	double scale, Xrange, Yrange;
	double arrowL, arrowW, velocityScale;
	int []xPoint = new int[3];
	int []yPoint = new int[3];
	double [][] u;
	double [][] v;
	double [] si;
	double [][] xx;
	double [][] yy;
	int numXVec = 20;
	int numYVec = 20;
	boolean uniformOn;

	double	[]	curves[];
	int	[]	numPts;

	String messageSpace	= " ";

	String messageToBeSent	= null;
	String messageHead	= "4";
	String messageTotalData	= null;
	String messageDOWN	= "41";
	String messageUP	= "42";
	String messageDRAG	= "43";
	String messageBody	= null;
	String messageEnd	= "kimd";

	PlotCanvas(CFD smain, Dimension d) {

		this.smain = smain;
		dim = d;
		cw = dim.width;
		ch = dim.height;
		ccx = cw/2;
		ccy = ch/2;
		gridx = cw/GRIDSTEP;
		gridy = ch/GRIDSTEP;
		u = new double[gridy][gridx];
		v = new double[gridy][gridx];
		si = new double [gridy*gridx];
		xx = new double[gridy][gridx];
		yy = new double[gridy][gridx];
		curves = new double[NUMCONTOURLEVEL*20][];
		numPts = new int[NUMCONTOURLEVEL*20];
		Xrange = 10.0;
		scale = (double)(dim.width)/Xrange;
		Yrange = Xrange*(double)(ch)/(double)(cw);
		uniformOn = true;
		velocityScale = 10.0;
		arrowL = 3.0;
		arrowW = 3.0;
		MD = false;
		resize(dim.width, dim.height);
		setBackground(Color.black);
	}

	public void paint(Graphics g) {

		g.setColor(Color.blue);
		g.drawLine(ccx, 0, ccx, ch);
		g.drawLine(0, ccy, cw, ccy);
		g.drawRect(0, 0, cw, ch);
		g.setColor(Color.blue);
		g.drawRect(0, 0, size().width-1, size().height-1);
		g.setColor(Color.yellow);
		plotVector(g);
	}

	public Dimension minimumSize() {
		return new Dimension(320,240);
	}

	public Dimension preferredSize() {
		return dim;
	}

	public boolean mouseMove(Event e, int x, int y) {

		y = ch-1-y;
		int xp = x/GRIDSTEP;
		int yp = y/GRIDSTEP;
		double temp1, temp2, temp3, temp4;
		if((xp*GRIDSTEP == x) && (yp*GRIDSTEP == y)) {
			temp1 = (int)(xx[yp][xp]*100.0)/100.0;
			temp2 = (int)(yy[yp][xp]*100.0)/100.0;
			temp3 = (int)(u[yp][xp]*100.0)/100.0;
			temp4 = (int)(v[yp][xp]*100.0)/100.0;
		}
		else {
			calVelocity(x, y);
			temp1 = (int)(xxp*100.0)/100.0;
			temp2 = (int)(yyp*100.0)/100.0;
			temp3 = (int)(uup*100.0)/100.0;
			temp4 = (int)(vvp*100.0)/100.0;
		}

		smain.status.Pos.setText("("+ Double.toString(temp1)
			+ ", " + Double.toString(temp2) + ")");
		if ( smain.numOfSource > 0 || smain.uniformflow_main ) {
			smain.status.Velocity.setText("(" + Double.toString(temp3)
				+ ", " + Double.toString(temp4) + ")");
		}
		else {
			smain.status.Velocity.setText("(0.0, 0.0)");
		}

		return true;
	}

	public boolean mouseDrag(Event e, int x, int y) {

		// only Master can do
		if ( !smain.isMaster ) {
			return true;
		}

		if ( smain.numOfSource <= 0 ) {
			if ( !smain.uniformflow_main ) {
				return true;
			}
		}

		y = ch-1-y;
		int xp = x/GRIDSTEP;
		int yp = y/GRIDSTEP;
		double temp1, temp2, temp3, temp4;
		if((xp*GRIDSTEP == x) && (yp*GRIDSTEP == y)) {
			temp1 = (int)(xx[yp][xp]*100.0)/100.0;
			temp2 = (int)(yy[yp][xp]*100.0)/100.0;
			temp3 = (int)(u[yp][xp]*100.0)/100.0;
			temp4 = (int)(v[yp][xp]*100.0)/100.0;
		}
		else {
			calVelocity(x, y);
			temp1 = (int)(xxp*100.0)/100.0;
			temp2 = (int)(yyp*100.0)/100.0;
			temp3 = (int)(uup*100.0)/100.0;
			temp4 = (int)(vvp*100.0)/100.0;
		}

		smain.status.Pos.setText("("+ Double.toString(temp1)
			+ ", " + Double.toString(temp2) + ")");
		smain.status.Velocity.setText("(" + Double.toString(temp3)
			+ ", " + Double.toString(temp4) + ")");

		if (MD == true) {
			Graphics g = this.getGraphics();
			drawVector(g, temp1, temp2, temp3, temp4);
		}

		messageTotalData = Integer.toString(smain.numOfSource);

		messageToBeSent = messageHead + messageSpace
				+ messageTotalData + messageSpace
				+ messageDRAG + messageSpace
				+ Double.toString(temp1) + messageSpace
				+ Double.toString(temp2) + messageSpace
				+ Double.toString(temp3) + messageSpace
				+ Double.toString(temp4) + messageSpace
				+ messageEnd;

		smain.send_message(messageToBeSent);

		return true;
	}

	public boolean mouseDown(Event e, int x, int y) {

		// only Master can do
		if ( !smain.isMaster  || smain.numOfSource <= 0 ) {
			return true;
		}

		MD = true;
		y = ch-1-y;
		int index = smain.plot_type.plotType.getSelectedIndex();
		double temp1 = 0, temp2 = 0, temp3 = 0, temp4 = 0;
		if (index != 2) {
			int xp = x/GRIDSTEP;
			int yp = y/GRIDSTEP;

			if ((xp*GRIDSTEP == x) && (yp*GRIDSTEP == y)) {
				temp1 = (int)(xx[yp][xp]*100.0)/100.0;
				temp2 = (int)(yy[yp][xp]*100.0)/100.0;
				temp3 = (int)(u[yp][xp]*100.0)/100.0;
				temp4 = (int)(v[yp][xp]*100.0)/100.0;
			}
			else {
				calVelocity(x, y);
				temp1 = (int)(xxp*100.0)/100.0;
				temp2 = (int)(yyp*100.0)/100.0;
				temp3 = (int)(uup*100.0)/100.0;
				temp4 = (int)(vvp*100.0)/100.0;
			}

			Graphics g = this.getGraphics();
			drawVector(g, temp1, temp2, temp3, temp4);
		}

		messageTotalData = Integer.toString(smain.numOfSource);

		messageToBeSent = messageHead + messageSpace
				+ messageTotalData + messageSpace
				+ messageDRAG + messageSpace
				+ Double.toString(temp1) + messageSpace
				+ Double.toString(temp2) + messageSpace
				+ Double.toString(temp3) + messageSpace
				+ Double.toString(temp4) + messageSpace
				+ messageEnd;

		smain.send_message(messageToBeSent);

		return true;
	}

	public void drawVector(Graphics g, double x, double y, double u, double v) {

		double length, temp, tempS;
		int x0 = (int)(x*scale+ccx);
		int y0 = (int)(ccy-y*scale);
		length = Math.sqrt(u*u+v*v);
		temp = length*velocityScale;
		if(temp<1.0) {
			g.setColor(Color.red);
			g.drawLine(x0, y0, x0, y0);
			return;
		}
		double tsin = v/length;
		double tcos = u/length;

		if(temp>1.5*velocityScale) {
			tempS=1.5*velocityScale/temp;
			xPoint[0] = x0 + (int)(u*velocityScale*tempS);
			yPoint[0] = y0 - (int)(v*velocityScale*tempS);
			g.setColor(Color.yellow);
		}
		else {
			xPoint[0] = x0 + (int)(u*velocityScale);
			yPoint[0] = y0 - (int)(v*velocityScale);
			g.setColor(Color.yellow);
		}

		g.drawLine(x0, y0, xPoint[0], yPoint[0]);

		if(velocityScale > 9.0) {
			xPoint[1] = xPoint[0]-(int)(arrowL*tcos+arrowW*tsin);
			yPoint[1] = yPoint[0]+(int)(arrowL*tsin-arrowW*tcos);
			xPoint[2] = xPoint[0]-(int)(arrowL*tcos-arrowW*tsin);
			yPoint[2] = yPoint[0]+(int)(arrowL*tsin+arrowW*tcos);
			g.drawLine(xPoint[0], yPoint[0], xPoint[1], yPoint[1]);
			g.drawLine(xPoint[0], yPoint[0], xPoint[2], yPoint[2]);
		}
	}

	public void plotVector(Graphics g) {
		if( (smain.numOfSource <= 0) && (!smain.uniformflow_main) ) {
			return;
		}

		int index = smain.plotType_main;
		int i, j;
		int stepx;
		int stepy;

        switch(index) {
            case 0:
	            numXVec = 20;
	            numYVec = 20;
                velocityScale = 10.0;
        	    stepx = gridx/numXVec;
	            stepy = gridy/numYVec;
            	for(i=0; i<gridy; i+=stepy) {
	    	        for(j=0; j<gridx; j+=stepx) {
		    	        drawVector(g, xx[i][j], yy[i][j], u[i][j], v[i][j]);
		            }
	            }
                break;
            case 1:
	            numXVec = 20;
	            numYVec = 20;
                velocityScale = 8.0;
        	    stepx = gridx/numXVec;
	            stepy = gridy/numYVec;
            	for(i=0; i<gridy; i+=stepy) {
	    	        for(j=0; j<gridx; j+=stepx) {
		    	        drawVector(g, xx[i][j], yy[i][j], u[i][j], v[i][j]);
		            }
	            }
                break;
			case 2:
				if(!ContourOK){
					calContour();
				}
					int x0, y0, x1, y1;
			    	g.setColor(Color.green);
					for(i=0; i<numCurves; i++) {
						x0 = (int)curves[i][0];
						y0 = ch-(int)curves[i][1];
						for(j=2; j<numPts[i]; j+=2) {
							x1 = (int)curves[i][j];
							y1 = ch-(int)curves[i][j+1];
			    			g.drawLine(x0, y0, x1, y1);
							x0 = x1;
							y0 = y1;
						}
					}
				break;
	        default:
    	        break;
    	}
    }

	public void calContour() {
		int i, j;
		IsoCurve isocurve;
		isocurve = new IsoCurve(si, gridx, gridy);
		double smax = si[0];
		for(i=1; i<gridx*gridy; i++) {
			if(si[i]>smax)
				smax = si[i];
		}
		if(smax > 10.0)
			smax = 10.0;
		double zrange = 2*smax;
		double smin = -smax;
		double step = zrange/(NUMCONTOURLEVEL-1);
		numCurves = 0;
		for(i=0; i<=NUMCONTOURLEVEL; i++) {
			isocurve.setValue((double)i*step+smin);
			while( (curves[numCurves] = isocurve.getCurve()) != null ) {
				numPts[numCurves] = curves[numCurves].length;
				for(j=0; j<numPts[numCurves]; j++)
					curves[numCurves][j] = (curves[numCurves][j])*GRIDSTEP;
				numCurves ++;
				if(numCurves >= NUMCONTOURLEVEL*20)
					break;
			}
		}
		ContourOK = true;
	}

	public void calPlotContour(Graphics g) {
		int i, j;
		IsoCurve isocurve;
		isocurve = new IsoCurve(si, gridx, gridy);
		double smax = si[0];
		for(i=1; i<gridx*gridy; i++) {
			if(si[i]>smax)
				smax = si[i];
		}
		if(smax > 5.0)
			smax = 5.0;
		double zrange = 2*smax;
		double smin = -smax;
		double step = zrange/(NUMCONTOURLEVEL-1);
		g.setColor(Color.green);
		int x0, y0, x1, y1;
		numCurves = 0;
		for(i=0; i<=NUMCONTOURLEVEL; i++) {
			isocurve.setValue((double)i*step+smin);
			while( (curves[numCurves] = isocurve.getCurve()) != null ) {
				numPts[numCurves] = curves[numCurves].length;
				for(j=0; j<numPts[numCurves]; j++)
					curves[numCurves][j] = (curves[numCurves][j])*GRIDSTEP;
				x0 = (int)curves[numCurves][0];
				y0 = ch-(int)curves[numCurves][1];
				for(j=2; j<numPts[numCurves]; j+=2) {
					x1 = (int)curves[numCurves][j];
					y1 = ch-(int)curves[numCurves][j+1];
	    			g.drawLine(x0, y0, x1, y1);
					x0 = x1;
					y0 = y1;
				}
				numCurves ++;
				if(numCurves >= NUMCONTOURLEVEL*20)
					break;
			}
		}
		ContourOK = true;
	}

	public void calVelocity() {

		Graphics g = this.getGraphics();
		g.setColor(Color.red);
		g.drawString("Caculating...., Please wait a moment!", cw/4, ch/2);
		int i, j, ii;
		double pi = Math.atan(1.0)*4.0;
		double alpha;
		double sina;
		double cosa;

		if(smain.uniformflow_main) {

			double U0 = smain.velocity_main;
			alpha = smain.direction_main / 180.0 * pi;
			sina = Math.sin(alpha);
			cosa = Math.cos(alpha);
			double uu = U0 * cosa;
			double vv = U0 * sina;

			for(i=0; i<gridy; i++) {
				ii = i*gridx;

				for(j=0; j<gridx; j++) {
					u[i][j] = uu;
					v[i][j] = vv;
					xx[i][j] = (double)(j*GRIDSTEP-ccx)/scale;
					yy[i][j] = (double)(i*GRIDSTEP-ccy)/scale;
					si[ii+j] = uu*yy[i][j] - vv*xx[i][j];
				}
			}
		}
		else {
			alpha = 0.0;
			sina = 0.0;
			cosa = 1.0;

			for(i=0; i<gridy; i++) {
				ii = i*gridx;

				for(j=0; j<gridx; j++) {
					u[i][j] = 0.0;
					v[i][j] = 0.0;
					xx[i][j] = (double)(j*GRIDSTEP-ccx)/scale;
					yy[i][j] = (double)(i*GRIDSTEP-ccy)/scale;
					si[ii+j] = 0.0;
				}
			}
		}

		int k;
		double tempU, tempV, tempR, tempVr, temp1, temp2, tempSI;

		for(k=0; k<smain.numOfSource; k++) {
			switch (smain.sourceIndex[k]) {
			case 0:
			    for(i=0; i<gridy; i++) {
			        ii = i*gridx;
				    for(j=0; j<gridx; j++) {
	    				temp1 = xx[i][j]-smain.SX[k];
		    			temp2 = yy[i][j]-smain.SY[k];
			    		tempR = Math.sqrt(temp1*temp1 +temp2*temp2);

		    			if(tempR*scale<1.0)
			    		{
				    	    u[i][j] = v[i][j] = 0.0;
					    	tempU = tempV = tempSI = 0.0;
						    si[ii+j] = 0.0;
	    				}
		    			else {
			    			tempVr = smain.sourceStrength[k]/tempR;
				    		tempU = tempVr * temp1/tempR;
					    	tempV = tempVr * temp2/tempR;
							double aa = Math.asin(temp2/tempR);
							if(temp1<0)
							{
								if(temp2>=0)
									aa = pi-aa;
								else if(temp2 < 0)
									aa = -aa-pi;
							}
							aa -=alpha;
							if(aa > pi)
								aa -= pi*2.0;
							else if(aa <= -pi)
								aa += pi*2.0;

					    	tempSI = smain.sourceStrength[k]*aa;
				    	}
					    u[i][j] += tempU;
					    v[i][j] += tempV;
					    si[ii+j] += tempSI;
				    }
			    }
        		break;
		    case 1:
			    for(i=0; i<gridy; i++) {
			        ii = i*gridx;
				    for(j=0; j<gridx; j++) {
	    				temp1 = xx[i][j]-smain.SX[k];
		    			temp2 = yy[i][j]-smain.SY[k];
			    		tempR = Math.sqrt(temp1*temp1 +temp2*temp2);

				    	if(tempR*scale<1.0)
					    {
					        u[i][j] = v[i][j] = 0.0;
	    					tempU = tempV = tempSI = 0.0;
		    				si[ii+j] = 0.0;
			    		}
			    		else {
				    		tempVr = smain.sourceStrength[k]/tempR;
					    	tempU = tempVr * temp1/tempR;
						    tempV = tempVr * temp2/tempR;
							double aa = Math.asin(temp2/tempR);
							if(temp1<0)
							{
								if(temp2>=0)
									aa = pi-aa;
								else if(temp2 < 0)
									aa = -aa-pi;
							}
							aa -=alpha;
							if(aa > pi)
								aa -= pi*2.0;
							else if(aa <= -pi)
								aa += pi*2.0;
					    	tempSI = smain.sourceStrength[k]*aa;
	    				}
		    			u[i][j] -= tempU;
			    		v[i][j] -= tempV;
				    	si[ii+j] -= tempSI;
		    		}
		    	}
        		break;
	    	case 2:
			    for(i=0; i<gridy; i++) {
			        ii = i*gridx;
				    for(j=0; j<gridx; j++) {
				    	temp1 = xx[i][j]-smain.SX[k];
				    	temp2 = yy[i][j]-smain.SY[k];
				    	tempR = Math.sqrt(temp1*temp1 +temp2*temp2);

    					if(tempR*scale<1.0)
	    				{
		    			    u[i][j] = v[i][j] = 0.0;
			    			tempU = tempV = tempSI = 0.0;
				    		si[ii+j] = 0.0;
					    }
	    				else {
		    				tempVr = smain.sourceStrength[k]/tempR;
			    			tempU = tempVr * temp2/tempR;
				    		tempV = tempVr * temp1/tempR;
					    	tempSI = smain.sourceStrength[k]*Math.log(tempR);
			    		}
				    	u[i][j] -= tempU;
					    v[i][j] += tempV;
			    		si[ii+j] -= tempSI;
			    	}
    			}
	        	break;
    		case 3:
				double xt, yt, ut, vt;
			    for(i=0; i<gridy; i++) {
			        ii = i*gridx;
				    for(j=0; j<gridx; j++) {
				    	temp1 = xx[i][j]-smain.SX[k];
					    temp2 = yy[i][j]-smain.SY[k];
						xt = temp1*cosa + temp2*sina;
						yt = temp2*cosa - temp1*sina;
						temp1 = xt;
						temp2 = yt;
		    			tempR = (temp1*temp1 +temp2*temp2);

		    			if(Math.sqrt(tempR)*scale<1.0)
		    			{
			    		    u[i][j] = v[i][j] = 0.0;
				    		tempU = tempV = tempSI = 0.0;
	    					si[ii+j] = 0.0;
		    			}
			    		else {
				    		tempVr = smain.sourceStrength[k]/(tempR*tempR);
					    	tempU = tempVr * (temp1*temp1-temp2*temp2);
						    tempV = tempVr * temp2*temp1*2.0;
							ut = tempU*cosa - tempV*sina;
							vt = tempV*cosa + tempU*sina;
							tempU = ut;
							tempV = vt;
    						tempSI = smain.sourceStrength[k]*temp2/tempR;
	    				}
		    			u[i][j] -= tempU;
			    		v[i][j] -= tempV;
				    	si[ii+j] -= tempSI;
    				}
	    		}
        		break;
    		default:
	        	break;
    		}
        }
		ContourOK = false;
	}

    public void calVelocity(int xp, int yp) {
		if( (smain.numOfSource <= 0) && (!smain.uniformflow_main) ) {
			return;
		}

		double pi = Math.atan(1.0)*4.0;
	    double alpha;
	    double sina;
	    double cosa;
	    if(smain.uniformflow_main) {
			double U0 = smain.velocity_main;
			alpha = smain.direction_main / 180.0 * pi;

		    sina = Math.sin(alpha);
		    cosa = Math.cos(alpha);
		    double uu = U0 * cosa;
		    double vv = U0 * sina;
		    uup = uu;
		    vvp = vv;
		    xxp = (double)(xp-ccx)/scale;
		    yyp = (double)(yp-ccy)/scale;
		    sip = uu*yyp - vv*xxp;

	    } else {
			alpha = 0.0;
		    sina = 0.0;
		    cosa = 1.0;
		    uup = 0.0;
		    vvp = 0.0;
		    xxp = (double)(xp-ccx)/scale;
		    yyp = (double)(yp-ccy)/scale;
		    sip = 0.0;
	    }

	    int k;
	    double tempU, tempV, tempR, tempVr, temp1, temp2, tempSI;
	    for(k=0; k<smain.numOfSource; k++) {
    		switch (smain.sourceIndex[k]) {
	    	case 0:
   				temp1 = xxp-smain.SX[k];
    			temp2 = yyp-smain.SY[k];
	    		tempR = Math.sqrt(temp1*temp1 +temp2*temp2);

    			if(tempR*scale<1.0)
	    		{
		    	    uup = vvp = 0.0;
			    	tempU = tempV = tempSI = 0.0;
					sip = 0.0;
   				}
    			else {
	    			tempVr = smain.sourceStrength[k]/tempR;
		    		tempU = tempVr * temp1/tempR;
			    	tempV = tempVr * temp2/tempR;
					double aa = Math.asin(temp2/tempR);
					if(temp1<0)
					{
						if(temp2>=0)
							aa = pi-aa;
						else if(temp2 < 0)
							aa = -aa-pi;
					}
					aa -=alpha;
					if(aa > pi)
						aa -= pi*2.0;
					else if(aa <= -pi)
						aa += pi*2.0;

			    	tempSI = smain.sourceStrength[k]*aa;
		    	}
			    uup += tempU;
			    vvp += tempV;
			    sip += tempSI;
        		break;
		    case 1:
   				temp1 = xxp-smain.SX[k];
    			temp2 = yyp-smain.SY[k];
	    		tempR = Math.sqrt(temp1*temp1 +temp2*temp2);

		    	if(tempR*scale<1.0)
			    {
			        uup = vvp = 0.0;
   					tempU = tempV = tempSI = 0.0;
    				sip = 0.0;
	    		}
	    		else {
		    		tempVr = smain.sourceStrength[k]/tempR;
			    	tempU = tempVr * temp1/tempR;
				    tempV = tempVr * temp2/tempR;
					double aa = Math.asin(temp2/tempR);
					if(temp1<0)
					{
						if(temp2>=0)
							aa = pi-aa;
						else if(temp2 < 0)
							aa = -aa-pi;
					}
					aa -=alpha;
					if(aa > pi)
						aa -= pi*2.0;
					else if(aa <= -pi)
						aa += pi*2.0;
			    	tempSI = smain.sourceStrength[k]*aa;
   				}
    			uup -= tempU;
	    		vvp -= tempV;
		    	sip -= tempSI;
        		break;
	    	case 2:
		    	temp1 = xxp-smain.SX[k];
		    	temp2 = yyp-smain.SY[k];
		    	tempR = Math.sqrt(temp1*temp1 +temp2*temp2);
				if(tempR*scale<1.0)
   				{
    			    uup = vvp = 0.0;
	    			tempU = tempV = tempSI = 0.0;
		    		sip = 0.0;
			    }
   				else {
    				tempVr = smain.sourceStrength[k]/tempR;
	    			tempU = tempVr * temp2/tempR;
		    		tempV = tempVr * temp1/tempR;
			    	tempSI = smain.sourceStrength[k]*Math.log(tempR);
	    		}
		    	uup -= tempU;
			    vvp += tempV;
	    		sip -= tempSI;
	        	break;
    		case 3:
				double xt, yt, ut, vt;
		    	temp1 = xxp-smain.SX[k];
				temp2 = yyp-smain.SY[k];
				xt = temp1*cosa + temp2*sina;
				yt = temp2*cosa - temp1*sina;
				temp1 = xt;
				temp2 = yt;
    			tempR = (temp1*temp1 +temp2*temp2);

    			if(Math.sqrt(tempR)*scale<1.0)
    			{
	    		    uup = vvp = 0.0;
		    		tempU = tempV = tempSI = 0.0;
   					sip = 0.0;
    			}
	    		else {
		    		tempVr = smain.sourceStrength[k]/(tempR*tempR);
			    	tempU = tempVr * (temp1*temp1-temp2*temp2);
				    tempV = tempVr * temp2*temp1*2.0;
					ut = tempU*cosa - tempV*sina;
					vt = tempV*cosa + tempU*sina;
					tempU = ut;
					tempV = vt;
					tempSI = smain.sourceStrength[k]*temp2/tempR;
				}
    			uup -= tempU;
	    		vvp -= tempV;
		    	sip -= tempSI;
        		break;
    		default:
	        	break;
    		}
        }
	}
}
