package sv.config;

import sv.engine.SV2Engine;
import java.io.*;
import java.util.*;

public class ClientConfig {
  private Config1D config1d;
  private Config2D config2d;
  private Config3D config3d;

  // global variables
  private String filter_menu = null;
  private String sv2_config = null;
  private String browser_name = null;
  private String filter_home = null;
  private boolean globalCheck = false;

  /**
   * constructor
   */
  public ClientConfig(SV2Engine eng, String dir) {
    config1d = new Config1D(eng);
    config2d = new Config2D(eng);
    config3d = new Config3D(eng);

    if (dir != null) {
       if (!dir.endsWith(File.separator))
         dir = dir + File.separator;
 
       filter_menu = dir + "filter_menu";
       sv2_config = dir + "sv2.cfg";
       parseConfigs(sv2_config);
    }
  }

  /**
   * return menu file name
   */
  public String getMenuFile() {
    return filter_menu;
  }

  /*
   * return setting file name
   */
  public String getConfigFile() {
    return sv2_config;
  }
  
  /**
   * return browser name
   */
  public String getBrowserName() {
    return browser_name;
  }

  
  /**
   * return filer home directory
   */
  public String getFilterHome() {
    return filter_home;
  }

  /**
   * return global check flag
   */
  public boolean isGlobalMode() {
    return globalCheck;
  }

  /**
   * reconfiguration 
   */ 
  public void reconfigure() {
    parseConfigs(sv2_config);
  }  
  
  /**
   * return 1d frame configuration
   */
  public Config1D getConfig1D() {
    return config1d;
  }

  /**
   * return 2d frame configuration
   */
  public Config2D getConfig2D() {
    return config2d;
  }

  /**
   * return 3d frame configuration
   */
  public Config3D getConfig3D() {
    return config3d;
  }

  // parse configuration file
  private void parseConfigs(String fname) {

     String s;

     try {
       FileInputStream propFile = new FileInputStream(fname);
       Properties p = new Properties(System.getProperties());
       p.load(propFile);

       browser_name = p.getProperty("brwoser_name");
       filter_home = p.getProperty("filter_home_directory");

       /*
       s = p.getProperty("command_broadcast");
       if (s != null && s.equals("yes"))  globalCheck = true;
       else                  globalCheck = false;
       */
       
       s = p.getProperty("colormap");
       config1d.setColormap(s);
       config2d.setColormap(s);
       config3d.setColormap(s);

       //////////////////////////////////////////////////////////
       // 1d
       //////////////////////////////////////////////////////////

       //////////////////////////////////////////////////////////
       // 2d
       //////////////////////////////////////////////////////////

       //////////////////////////////////////////////////////////
       // 3d
       //////////////////////////////////////////////////////////


     } catch (Exception e) {
       System.err.println("I/O failed while reading the configuration file...");
     }
  }


} // end of class




