package sv.config;

import java.awt.*;
import sv.engine.SV2Engine;

/** 
 * define configurations
 */

public abstract class Configs {

  // 2d and 3d plot mode
  public static int WIREFRAME      = 0;
  public static int COLORWIREFRAME = 1;
  public static int SHADING        = 2;
  public static int POINTPLOT      = 3;

  /**
   * friend class SV2Engine
   */
  protected SV2Engine engine;

  /**
   * colormap file name
   */
  protected String colormap = null;

  /**
   * if setted, use global min and max
   */
  protected boolean global_mode = true;    

  /**
   * filter overlay mode
   */
  protected boolean overlay_mode = false;

  /**
   * after filtering, discard or not the original data
   */
  protected boolean discard_mode = false;

  /**
   * should display time value on the upper-left corner
   */
  protected boolean display_time = true;
 
  /**
   * what is the background color
   */
  protected Color bgColor = Color.white;

  /**
   * what is the foreround color
   */
  protected Color fgColor = Color.black;

  /**
   * legend draw
   */
  protected boolean legendDraw = false;

  /**
   * tick mark
   */
  protected boolean tickMarkDraw = false;

  /**
   * grid
   */
  protected boolean gridDraw = false;
  
  /**
   * label
   */
  protected boolean labelDraw = false;

  /**
   * buffer size of the data set
   */
  protected int bufferSize = Integer.MAX_VALUE;

  /**
   * constructor
   */
  public Configs(SV2Engine eng) {
    engine = eng;
  }

  /**
   * return engine
   */
  public SV2Engine getEngine() {
    return engine;
  }

  /**
   * return colomap
   */
  public String getColormap() {
    return colormap;
  }

  /**
   * return colomap
   */
  public void setColormap(String c) {
    colormap = c;
  }

  /**
   * return the value of global mode
   */
  public boolean isGlobalMode() {
    return global_mode;
  }

  /**
   * return the value of overlay mode
   */
  public boolean isOverlayMode() {
    return overlay_mode;
  }

  /**
   * set the value of overlay mode
   */
  public void setOverlayMode(boolean b) {
    overlay_mode = b;
  }


  /**
   * return the value of discard mode
   */
  public boolean isDiscardMode() {
    return discard_mode;
  }

  /**
   * set the value of discard mode
   */
  public void setDiscardMode(boolean b) {
    discard_mode = b;
  }

  /**
   * return the value of display time mode
   */
  public boolean isDisplayTimeMode() {
    return display_time;
  }

  /**
   * set the value of display time mode
   */
  public void setDisplayTimeMode(boolean b) {
    display_time = b;
  }
 
  /**
   * return the background color
   */
  public Color getBGColor() {
    return bgColor;
  }

  /**
   * set the background color
   */
  public void setBGColor(Color c) {
    bgColor = c;
  }

  /**
   * return the foreground color
   */
  public Color getFGColor() {
    return fgColor;
  }

  /**
   * set the foreground color
   */
  public void setFGColor(Color c) {
    fgColor = c;
  }

  /**
   * return the value of legend draw
   */
  public boolean isLegendDraw() {
    return legendDraw;
  }

  /**
   * set legend drawing mode
   */
  public void setLegendDraw(boolean b) {
    legendDraw = b;
  }

  /**
   * return the value of tick mark
   */
  public boolean isTickMarkDraw() {
    return tickMarkDraw;
  }

  /**
   * set tick mark drawing mode
   */
  public void setTickMarkDraw(boolean b) {
    tickMarkDraw = b;
  }

  /**
   * return if the viewer should draw the grid
   */
  public boolean isGridDraw() {
    return gridDraw;
  }

  /**
   * set grid drawing mode
   */
  public void setGridDraw(boolean b) {
    gridDraw = b;
  }
  
  /**
   * return the value of label draw
   */
  public boolean isLabelDraw() {
    return labelDraw;
  }

  /**
   * set label drawing mode
   */
  public void setLabelDraw(boolean b) {
    labelDraw = b;
  }
  

  /**
   * return buffer size of the data set
   */
  public int getBufferSize() {
    return bufferSize;
  }

  /**
   * return  setting values
   */
  public abstract String[] getConfigs();

  /**
   * set setting values from the arguments
   */
  public abstract void setConfigs(String[] args);

} // end of class







