package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;
import sv.engine.*;

/** 
 * File reading utility
 */

public class DataReader {

  /**
   * read the data file
   */
  public static void readDataFile(SV2Engine engine, DataInputStream in) 
                                                         throws IOException {
    DataSetBrief brief = null;

    int noOfDataSet = in.readInt();   
    for (int i=0; i<noOfDataSet; i++) {
      int sz = in.readInt();
      byte[] byteArr = new byte[sz];
      in.read(byteArr, 0, sz);
      String title = new String(byteArr);

      sz = in.readInt();
      TimeData[] tds = new TimeData[sz];
      int rank = 0;

      for (int j=0; j<sz; j++) {
        // read magine number
        int magic = in.readInt();
        switch (magic) {
          case 101:
            rank = TimeData.ONED;
            tds[j] = parse101(in);
            break;
          case 102:
            rank = TimeData.ONED;
            tds[j] = parse102(in);
            break;
          case 103:
            rank = TimeData.ONED;
            tds[j] = parse103(in);
            break;
          case 104:
            rank = TimeData.ONED;
            tds[j] = parse104(in);
            break;
          case 105:
            rank = TimeData.ONED;
            tds[j] = parse105(in);
            break;
          case 201:
            rank = TimeData.TWOD;
            tds[j] = parse201(in);
            break;
          default:
            System.out.println("This is not SV2 file.");
            return;
        }
      }
      
      DataSet dataSet = new DataSet(tds);
      dataSet.setDataName(title);
      
      if (brief  == null)
        brief = new DataSetBrief(title, tds.length, rank, dataSet.getDataSize());
 
      brief.addDataSet(dataSet);
    }

    in.readByte();               // NOT USED YET

    brief.setID(System.currentTimeMillis());
    brief.setPrivate(true);
    brief.updateInfo();
    engine.receivedBriefInfo(brief); 

    long fid = engine.vmanager.register(brief, -1, null);
    brief.setViewerID(fid); 
  }

  /**
   * parse 1d point data
   */
  public static TDPoint parse101(DataInputStream in) throws IOException {
    boolean COLOR_CODE = false;
    boolean ERRORS = false;
    boolean BB = false;

    float[] xArr = null, yArr = null, cArr = null, eArr= null;
    float xmin = 0f, xmax = 0f;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int noOfPoints = in.readInt();

    // received x and y arrays
    if (in.readByte() == 1) {
      BB = true;
      yArr = new float[noOfPoints];
   
      xmin = in.readFloat();
      xmax = in.readFloat();

      for (int i=0; i<noOfPoints; i++) 
        yArr[i] = in.readFloat();

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
      }
    }

    // received cArr
    if (in.readByte() == 1) {
      COLOR_CODE = true;
      cArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        cArr[i] = in.readFloat();
    } 

    // received error array
    if (in.readByte() == 1) {
      ERRORS = true;
      eArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        eArr[i] = in.readFloat();
    } 

    // receive reserved 1 item and flush out
    in.readInt();
  
    // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
    if (BB) {
      if (COLOR_CODE && ERRORS) 
        return null;   // not implemented yet
      else if (COLOR_CODE && !ERRORS)
        return null;   // not implemented yet
      else if (!COLOR_CODE && ERRORS)
        return new TDPoint(time, noOfPoints, xmin, xmax, yArr, eArr);
      else  
        return new TDPoint(time, noOfPoints, xmin, xmax, yArr);

    } else {

      if (COLOR_CODE && ERRORS) 
        return new TDPoint(time, noOfPoints, xArr, yArr, eArr, cArr); 
      else if (COLOR_CODE && !ERRORS)
        return new TDPoint(time, noOfPoints, xArr, yArr, cArr); 
      else if (!COLOR_CODE && ERRORS)
        return new TDPoint(time, noOfPoints, xArr, yArr, eArr, true);
      else 
        return new TDPoint(time, noOfPoints, xArr, yArr);
    }
  }

  /**
   * parse 1d vector data
   */
  public static TDVector parse102(DataInputStream in) throws IOException {
    boolean COLOR_CODE = false;
    boolean BB = false;

    float[] xArr = null, yArr = null, cArr = null, rArr = null, tArr= null;
    float xmin = 0f, xmax = 0f;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int noOfPoints = in.readInt();

    // received x and y arrays
    if (in.readByte() == 1) {
      BB = true;
      yArr = new float[noOfPoints];
      rArr = new float[noOfPoints];
      tArr = new float[noOfPoints];

      xmin = in.readFloat();
      xmax = in.readFloat();

      for (int i=0; i<noOfPoints; i++) {
        yArr[i] = in.readFloat();
        rArr[i] = in.readFloat();
        tArr[i] = in.readFloat();
      }

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];
      rArr = new float[noOfPoints];
      tArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
        rArr[i] = in.readFloat();
        tArr[i] = in.readFloat();
      }
    }

    // received cArr
    if (in.readByte() == 1) {
      COLOR_CODE = true;
      cArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        cArr[i] = in.readFloat();
    }  

    // receive reserved 1 item and flush out
    in.readInt();
  
    // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
    if (BB) {
      return null;   // not implemented yet
    } else {
      if (COLOR_CODE) 
        return null;  // not implemented yet
      else 
        return new TDVector(time, noOfPoints, xArr, yArr, rArr, tArr);  
    }
  }


  /**
   * parse 1d connection data
   */
  public static TDCon parse103(DataInputStream in) throws IOException {
    boolean COLOR_CODE = false;
    boolean BB = false;

    float[] xArr = null, yArr = null, cArr = null;
    int[] connection = null; 
    float xmin = 0f, xmax = 0f;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int noOfPoints = in.readInt();

    // received x and y arrays
    if (in.readByte() == 1) {
      BB = true;
      yArr = new float[noOfPoints];

      xmin = in.readFloat();
      xmax = in.readFloat();

      for (int i=0; i<noOfPoints; i++) 
        yArr[i] = in.readFloat();

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
      }
    }

    // received cArr
    if (in.readByte() == 1) {
      COLOR_CODE = true;
      cArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        cArr[i] = in.readFloat();
    }  

    connection = new int[in.readInt()];
    for (int i=0; i<connection.length; i++) 
     connection[i] = in.readInt();

    // receive reserved 1 item and flush out
    in.readInt();
  
    // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
    if (BB) {
      return null;   // not implemented yet
    } else {
      if (COLOR_CODE) 
        return new TDCon(time, noOfPoints, xArr, yArr, cArr, connection); 
      else 
        return null;   // not implemented yet
    }
  }


  /**
   * parse 1d bar data
   */
  public static TDBar parse104(DataInputStream in) throws IOException {
    boolean COLOR_CODE = false;
    boolean BB = false;

    float[] xArr = null, yArr = null, cArr = null, wArr= null;
    float xmin = 0f, xmax = 0f;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int noOfPoints = in.readInt();

    // received x and y arrays
    if (in.readByte() == 1) {
      BB = true;
      yArr = new float[noOfPoints];
      wArr = new float[noOfPoints];

      xmin = in.readFloat();
      xmax = in.readFloat();

      for (int i=0; i<noOfPoints; i++) {
        yArr[i] = in.readFloat();
        wArr[i] = in.readFloat();
      }

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];
      wArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
        wArr[i] = in.readFloat();
      }
    }

    // received cArr
    if (in.readByte() == 1) {
      COLOR_CODE = true;
      cArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        cArr[i] = in.readFloat();
    }  

    // receive reserved 1 item and flush out
    in.readInt();
  
    // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
    if (BB) {
      return null;   // not implemented yet
    } else {
      if (COLOR_CODE) 
        return new TDBar(time, noOfPoints, xArr, yArr, wArr, cArr); 
      else 
         return null;   // not implemented yet
    }
  }

  /**
   * parse 1d Tree data
   */
  public static TDTree parse105(DataInputStream in) throws IOException {

    float[] xArr = null, yArr = null, xtoArr = null, ytoArr= null;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int noOfPoints = in.readInt();

    // received x and y arrays
    if (in.readByte() == 1) {
      in.readInt();
      return null;

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];
      xtoArr = new float[noOfPoints];
      ytoArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
        xtoArr[i] = in.readFloat();
        ytoArr[i] = in.readFloat();
      }
    }

    // receive reserved 1 item and flush out
    in.readInt();
  
    return new TDTree(time, xArr, yArr, xtoArr, ytoArr); 
  }

  /**
   * parse 2d data
   */
  public static TimeData2D parse201(DataInputStream in) 
                                                  throws IOException {
    boolean COLOR_CODE = false;             // not used
    boolean BB = false;

    float[] xArr = null, yArr = null, zArr = null, cArr = null;
    float xmin=0f, ymin=0f, xmax=0f, ymax=0f;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int nx = in.readInt();
    int ny = in.readInt();
    int noOfPoints = in.readInt();

    // receive x, y and z arrays
    if (in.readByte() == 1) {
      BB = true;
      zArr = new float[noOfPoints];
        
      xmin = in.readFloat();
      xmax = in.readFloat();
      ymin = in.readFloat();
      ymax = in.readFloat();

      for (int i=0; i<noOfPoints; i++) 
        zArr[i] = in.readFloat();

    } else {
      xArr = new float[noOfPoints];
      yArr = new float[noOfPoints];
      zArr = new float[noOfPoints];

      for (int i=0; i<noOfPoints; i++) {
        xArr[i] = in.readFloat();
        yArr[i] = in.readFloat();
        zArr[i] = in.readFloat();
      }
    }

    // receive cArr
    if (in.readByte() == 1) {
      COLOR_CODE = true;
      cArr = new float[noOfPoints];
      for (int i=0; i<noOfPoints; i++) 
        cArr[i] = in.readFloat();
    } 

    // receive reserved 1 item and flush out
    in.readInt();
  
    // SHOULD ALLOW OPTIONS
    if (BB) {
      if (COLOR_CODE) 
        return null;
      else 
        return new TimeData2D(time, xmin, xmax, nx, ymin, ymax, ny, zArr);

    } else {
      if (COLOR_CODE) 
        return null;
      else
        return null;
    }
  }

  /**
   * parse 3d data
   */
  public static TimeData3D parse301(DataInputStream in) 
                                                  throws IOException {
    float[] zArr = null;

    // received time
    float t = in.readFloat();
    TimeValue time = new TimeValue(t);

    // number of points in given time and make arrays
    int nx = in.readInt();
    int ny = in.readInt();
    int nz = in.readInt();

    int noOfPoints = in.readInt();
    zArr = new float[noOfPoints];

    for (int i=0; i<noOfPoints; i++) 
      zArr[i] = in.readFloat();

    return new TimeData3D(time, nx, ny, nz, zArr);
  }


  /**
   * parse iso data
   */
  public static TimeData3D parse302(DataInputStream in) 
                                                  throws IOException {
    int size;

    float t = in.readFloat();  // time value
    TimeValue time = new TimeValue(t);

    int dimx = in.readInt();  
    int dimy = in.readInt();  
    int dimz = in.readInt();  

    float threshold = in.readFloat();
    float zmin = in.readFloat();
    float zmax = in.readFloat();

    int vsize = in.readInt();     // read vertex size
    float[][] vertex = new float[vsize][3];
    float[][] normal = new float[vsize][3];

    for (int i=0; i<vsize; i++) {
      for (int j=0; j<3; j++) 
        vertex[i][j] = in.readFloat();
      for (int j=0; j<3; j++) 
        normal[i][j] = in.readFloat();
    }

    int tsize = in.readInt();     // read triangle
    int[][] triangle = new int[tsize][3];
         
    for (int i=0; i<tsize; i++) 
      for (int j=0; j<3; j++) 
        triangle[i][j] = in.readInt();

    return new TimeData3D(time, vsize, vertex, normal, 
                          tsize, triangle, dimx, dimy, dimz, 
                          threshold, zmin, zmax);
  }


} // end of class


