package sv.data;

import java.io.*;
import java.awt.*;
import sv.gui.MarkerChooser;

public class DataSet {

  // data set name
  private String dataName = "Untitled";

  // time data attached to this DataSet
  private TimeData[] dataArr;

  //////////////////////////////////////////////////////////
  // JUST FOR ID DATA SET
  private int markerType = MarkerChooser.CIRCLEMARKER;
  private Color lineColor = Color.black;

  /**
   * constructor
   * @param td time data
   */
  public DataSet(TimeData td) {
    dataArr = new TimeData[1];
    dataArr[0] = td;
  }

  /**
   * return clone data set
   */
  public DataSet getClone() {
    TimeData[] arr = new TimeData[dataArr.length];
    System.arraycopy(dataArr, 0, arr, 0, dataArr.length);
    DataSet ds = new DataSet(arr);
    ds.setDataName(dataName);
    return ds;
  }


  /**
   * constructor for multiple time data
   * @param tds time data array
   */
  public DataSet(TimeData[] tds) {
    // check the time values are increasing order
    boolean increasingOrder = true;
    float prev = Float.MIN_VALUE;

    for (int i=0; i<tds.length; i++) {
      if (prev >= tds[i].getTime().getAbsTime()) {
        increasingOrder = false;
        break;
      }
    }

    if (increasingOrder)
      dataArr = tds;
    else {
      dataArr = new TimeData[1];
      dataArr[0] = tds[0];
      for (int i=1; i<tds.length; i++)
        addTimeData(tds[i]);
    }
  }

  /**
   * add one time data to this data set
   * @param data time data to be added
   */
  public void addTimeData(TimeData data) {

    float src = data.getTime().getAbsTime();
    float target = dataArr[dataArr.length-1].getTime().getAbsTime();
    
    if (src > target) {
      TimeData[] tmp = new TimeData[dataArr.length+1];
      System.arraycopy(dataArr, 0, tmp, 0, dataArr.length);
      tmp[dataArr.length] = data;
      dataArr = tmp;
      return;
    }

    for (int i=0; i<dataArr.length; i++) {
      target = dataArr[i].getTime().getAbsTime();

      if (src > target) 
        continue;
      else if (src == target) {
        dataArr[i] = data;
        break;
      } else {
        TimeData[] tmp = new TimeData[dataArr.length+1];
        System.arraycopy(dataArr, 0, tmp, 0, i);
        tmp[i] = data;
        System.arraycopy(dataArr, i, tmp, i+1, dataArr.length-i);
        dataArr = tmp;
        break;        
      }
    }
  }

  /**
   * garbage collection(I can't rely only on java gabage collector)
   */
  public void garbageCollect() {
    if (dataArr == null) return;
    for (int i=0; i<dataArr.length; i++)
      dataArr[i] = null;
    dataArr = null;
  }

  /**
   * return i-th time value
   */
  public TimeValue getTimeValue(int i) {
     if (dataArr.length > i)
        return dataArr[i].getTime();
     return null;
  }

  /**
   * return i-th time data
   */
  public TimeData getTimeData(int i) {
     if (dataArr.length > i)
        return dataArr[i];
     return null;
  }

  /**
   * return the number of time data
   */
  public int getNoOfTimes() {
    return dataArr.length;
  }

  /**
   * return data set name
   */
  public String getDataName() {
    return dataName;
  }

  /**
   * rename the data set name
   * @param t new data name of the data set
   */
  public void setDataName(String t) {
     dataName = t;
  }

  /**
   * return the data size
   */
  public long getDataSize() {
    long sz = 0;

    for (int i=0; i<dataArr.length; i++)
      sz += dataArr[i].getDataSize();

    return sz;
  }

  /**
   * write the data to the output stream
   */
  public void sendingData(DataOutputStream out) throws IOException {
    if (dataArr == null) return;

    // write the data name
    byte[] byteArr = dataName.getBytes();
    out.writeInt(byteArr.length);
    out.write(byteArr, 0, byteArr.length);

    // write the number of time steps
    out.writeInt(dataArr.length);

    for (int i=0; i<dataArr.length; i++) 
      dataArr[i].sendingData(out);      
  }


   /**
    * set marker type
    */
   public void setMarkerType(int i) {
     markerType = i;
   }

   /**
    * set line color
    */
   public void setLineColor(Color c) {
     lineColor = c;
   }

   /**
    * return marker type
    */ 
   public int getMarkerType() {
     return markerType;
   }

   /**
    * set marker type
    */
   public Color getLineColor() {
     return lineColor;
   }
}  // end of class

