package sv.data;

import java.awt.*;
import java.io.*;
import sv.gui.*;

/**
 * This class defines data set structure.
 */

public class DataSetBrief {

  private long identification = -1;
  private String dataName = "Untitled";
  private int noOfTime = -1;
  private int rank_ = -1;
  private long dataSize = -1;
  private long viewerid = -1;

  private DataSet[] realData = null;
  private boolean hidden_ = false;      // is it a private data set?

  /**
   * constructor
   * @param t data name of this DataSet
   * @param i the size of time data array for all TimeData attached this DataSet
   * @param r rank of the data set
   * @param sz data size(bytes)
   */
  public DataSetBrief(String t, int i, int r, long sz) {
     dataName = t;
     noOfTime = i;
     rank_ = r; 
     dataSize = sz;
  } 

  /**
   * set identification
   */
  public void setID(long id) {
    identification = id;
  }

  /**
   * get identification
   */
  public long getID() {
    return identification;
  }

  /**
   * update information
   */
  public void updateInfo() {
    noOfTime = 0;
    dataSize = 0;

    for (int i=0; i<realData.length; i++) {
      if (realData[i] == null) break;
      noOfTime += realData[i].getNoOfTimes();
      dataSize += realData[i].getDataSize();
    }
  }

  /**
   * gabage collection
   */
  public void setClear() {
    if (realData == null) return;
    for (int i=0; i<realData.length; i++) {
      if (realData[i] == null) break;
      realData[i].garbageCollect();
    }
    realData = null;  
  }

  /**
   * link to readed data set
   */
  public void addDataSet(DataSet data) {
    if (realData == null) {
      realData = new DataSet[1];
      realData[0] = data;
    } else {
      for (int i=0; i<realData.length; i++) {
        if (data.getDataName().equals(realData[i].getDataName())) {
          int sz = data.getNoOfTimes();
          for (int j=0; j<sz; j++) 
            realData[i].addTimeData(data.getTimeData(j));            

          return;         
	} 
      }

      DataSet[] tmp = new DataSet[realData.length+1];
      System.arraycopy(realData, 0, tmp, 0, realData.length);
      tmp[realData.length] = data.getClone();
      realData = tmp;

      if (realData[0].getTimeData(0).getRank() == TimeData.ONED) 
        if (realData.length <= 5) {
          realData[realData.length-1].setLineColor(lineColors[realData.length-1]);
          realData[realData.length-1].setMarkerType(markers[realData.length-1]);   
        }

    }
  }

  /**
   * return data set
   */
  public DataSet[] getDataSet() {
    return realData;
  }


  /**
   * set the data private or public
   * @param b if true, set private
   */
  public void setPrivate(boolean b) {
    hidden_ = b;
  }

  /**
   * return if the data is private
   */
  public boolean isPrivate() {
    return hidden_;
  }

  /**
   * return data set name
   */
  public String getDataName() {
    return dataName;
  }

  /**
   * rename the data set name
   * @param t new data name of the data set
   */
  public void setDataName(String t) {
     dataName = t;
  }

  /**
   * return noOfTime
   */
  public int getNoOfTime() {
    return noOfTime;
  }

  /**
   * return rank
   */
  public int getRank() {
    return rank_;
  }

  /**
   * return dataSize
   */
  public long getDataSize() {
    return dataSize;
  }

  /**
   * set viewer id number 
   */
  public void setViewerID(long id) {
    viewerid = id;
  }

  /**
   * return viewer id number
   */
  public long getViewerID() {
    return viewerid;
  }

  /**
   * write the data to a file
   */
  public void writeToFile(DataOutputStream out) throws IOException {
    // TEMPORARY,CLIENT REQUESTS THE REAL DATA FROM THE SERVER
    if (realData == null) return;   
 
    out.writeInt(realData.length);
    for (int i=0; i<realData.length; i++) 
      realData[i].sendingData(out); 
  }


  // marker for 1D data
  private int markers[] = { MarkerChooser.CIRCLEMARKER,
                            MarkerChooser.SQUAREMARKER,
                            MarkerChooser.DIAMONDMARKER,
                            MarkerChooser.CROSSMARKER,
                            MarkerChooser.XMARKER };
  private Color lineColors[] = { Color.black,
                                Color.blue,
                                Color.magenta,
                                Color.orange,
                                Color.red };

} // end of class



