package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;

/** 
 * Define Bar data in a given time
 */

public class TDBar extends TimeData1D {

  private float[] wArr;     // width vals

  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   * @param wArr bar width value array
   * @param cArr color value array
   */
  public TDBar(TimeValue time, int noOfPoints, float[] xArr, float[] yArr,
                                    float[] wArr, float[] cArr) {
     super(time, noOfPoints, xArr, yArr);
     dataType = TimeData1D.BAR;
     rank = ONED;

     this.wArr = wArr;
     this.cArr = cArr;
     recalcMinMax();
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) throws IOException {

    // send the magic number
    out.writeInt(104);

    // send time
    out.writeFloat(time.getAbsTime());

    // send number of points in given time
    out.writeInt(noOfPoints);
     
    // send x and y arrays
    if (bb) {
      out.writeByte(1);     // bb is true... send boundbox first
      out.writeFloat(xmin);
      out.writeFloat(xmax);

      for (int i=0; i<noOfPoints; i++) {
        out.writeFloat(yArr[i]);
        out.writeFloat(wArr[i]);
      }

    } else {
      out.writeByte(0);      // bb is false

      for (int i=0; i<noOfPoints; i++) {
        out.writeFloat(xArr[i]);
        out.writeFloat(yArr[i]);
        out.writeFloat(wArr[i]);
      }
    }

    // send cArr
    if (cArr == null) {
      out.writeByte(0);
    } else {
      out.writeByte(1);
      for (int i=0; i<noOfPoints; i++) 
         out.writeFloat(cArr[i]);
    }

    // send reserved for future use
    out.writeInt(0);
  }

  /**
   * return data size of this time data
   */
  public long getDataSize() {
    long sz = 4;         // initially time value
    
    if (xArr != null) sz = sz + 4 * xArr.length;
    if (yArr != null) sz = sz + 4 * yArr.length;
    if (cArr != null) sz = sz + 4 * cArr.length;
    if (wArr != null) sz = sz + 4 * wArr.length;
    return sz;
  }

  /**
   * return wArr
   */
  public float[] getwArr() {
    int sz = wArr.length;
    float[] ret = new float[sz];
    System.arraycopy(wArr, 0, ret, 0, sz);

    return ret;
  }

  private void recalcMinMax() {
    // check first value (is it centered?)
    if (cArr[0] < 1000f) 
       xmin = xmin - wArr[0]/2.0f;

    // check first value (is it left aligned?)
    if (cArr[0] >= 2000f) 
       xmin = xmin - wArr[0];

    // check last value (is it centered?)
    if (cArr[cArr.length-1] < 1000f) 
       xmax = xmax + wArr[0]/2.0f;

    // check last value (is it right aligned?)
    if (cArr[cArr.length-1] >= 1000f && cArr[cArr.length-1] < 2000f) 
       xmax = xmax + wArr[0];
  }

} // end of class


