package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;

/** 
 * Define connection list data in a given time
 */

public class TDTree extends TimeData1D {

  private float[] xtoArr;     // x to array
  private float[] ytoArr;     // y to array

  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   */
  public TDTree(TimeValue time, float[] xfromArr, float[] yfromArr,
                              float[] xtoArr, float[] ytoArr) {
     super(time, xfromArr.length, xfromArr, yfromArr);
     dataType = TimeData1D.TREE;
     rank = ONED;

     this.xtoArr = xtoArr;
     this.ytoArr = ytoArr;
  }

  public void addMorePoints(TDTree prevData) {
    recalcMinMax(prevData);
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) throws IOException {

    // send the magic number
    out.writeInt(105);

    // send time
    out.writeFloat(time.getAbsTime());

    // send number of points in given time
    out.writeInt(noOfPoints);
     
    // send arrays
    if (bb) {
      System.out.println("Bounding box is not defined.");
      out.writeByte(1);
    } else {
      out.writeByte(0);      // bb is false

      for (int i=0; i<noOfPoints; i++) {
        out.writeFloat(xArr[i]);
        out.writeFloat(yArr[i]);
        out.writeFloat(xtoArr[i]);
        out.writeFloat(ytoArr[i]);
      }
    }

    // send reserved for future use
    out.writeInt(0);
  }

  /**
   * return data size of this time data
   */
  public long getDataSize() {
    long sz = 4;         // initially time value
    
    if (xArr != null) sz = sz + 4 * xArr.length;
    if (yArr != null) sz = sz + 4 * yArr.length;
    if (xtoArr != null) sz = sz + 4 * xtoArr.length;
    if (ytoArr != null) sz = sz + 4 * ytoArr.length;
    return sz;
  }

  /**
   * return xtoArr
   */
  public float[] getxtoArr() {
     int sz = xtoArr.length;
     float[] ret = new float[sz];
     System.arraycopy(xtoArr, 0, ret, 0, sz);

     return ret;
  }


  /**
   * return ytoArr
   */
  public float[] getytoArr() {
     int sz = ytoArr.length;
     float[] ret = new float[sz];
     System.arraycopy(ytoArr, 0, ret, 0, sz);

     return ret;
  }

  private void recalcMinMax(TDTree prevData) {
     if (prevData == null) return;
     
     float[] prevxArr = prevData.getxArr();
     float[] prevyArr = prevData.getyArr();
     float[] prevxtoArr = prevData.getxtoArr();
     float[] prevytoArr = prevData.getytoArr();

     float[] xfromArr = getxArr();
     float[] yfromArr = getyArr();
     float[] xxtoArr = getxtoArr();
     float[] yytoArr = getytoArr();
     
     noOfPoints = xfromArr.length + prevxArr.length;
     
     xArr = new float[noOfPoints];
     yArr = new float[noOfPoints];
     xtoArr = new float[noOfPoints];
     ytoArr = new float[noOfPoints];

     System.arraycopy(prevxArr, 0, xArr, 0, prevxArr.length);
     System.arraycopy(xfromArr, 0, xArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevyArr, 0, yArr, 0, prevxArr.length);
     System.arraycopy(yfromArr, 0, yArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevxtoArr, 0, xtoArr, 0, prevxArr.length);
     System.arraycopy(xxtoArr, 0, xtoArr, prevxArr.length, xfromArr.length);
     System.arraycopy(prevytoArr, 0, ytoArr, 0, prevxArr.length);
     System.arraycopy(yytoArr, 0, ytoArr, prevxArr.length, xfromArr.length);

     for (int i=0; i< noOfPoints; i++) {
        if (xmin > xArr[i]) xmin = xArr[i];
        if (xmax < xArr[i]) xmax = xArr[i];
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
     }
  }

} // end of class


