package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;

/** 
 * Define vector data in a given time
 */

public class TDVector extends TimeData1D {

  // arrays
  private float[] rArr;     // r vals
  private float[] tArr;     // angle vals

  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   * @param rArr r value array
   * @param tArr t value array
   */
  public TDVector(TimeValue time, int noOfPoints, float[] xArr, float[] yArr,
                                    float[] rArr, float[] tArr) {
     super(time, noOfPoints, xArr, yArr);
     dataType = TimeData1D.VECTOR;
     rank = ONED;

     this.rArr = tArr;
     this.tArr = tArr;
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) throws IOException {

    // send the magic number
    out.writeInt(102);

    // send time
    out.writeFloat(time.getAbsTime());

    // send number of points in given time
    out.writeInt(noOfPoints);
     
    // send x and y arrays
    if (bb) {
      out.writeByte(1);     // bb is true... send boundbox first
      out.writeFloat(xmin);
      out.writeFloat(xmax);

      for (int i=0; i<noOfPoints; i++) {
        out.writeFloat(yArr[i]);
        out.writeFloat(rArr[i]);
        out.writeFloat(tArr[i]);
      }

    } else {
      out.writeByte(0);      // bb is false

      for (int i=0; i<noOfPoints; i++) {
        out.writeFloat(xArr[i]);
        out.writeFloat(yArr[i]);
        out.writeFloat(rArr[i]);
        out.writeFloat(tArr[i]);
      }
    }

    // send cArr
    if (cArr == null) {
      out.writeByte(0);
    } else {
      out.writeByte(1);
      for (int i=0; i<noOfPoints; i++) 
         out.writeFloat(cArr[i]);
    }

    // send reserved for future use
    out.writeInt(0);
  }

  /**
   * return data size of this time data
   */
  public long getDataSize() {
    long sz = 4;         // initially time value
    
    if (xArr != null) sz = sz + 4 * xArr.length;
    if (yArr != null) sz = sz + 4 * yArr.length;
    if (rArr != null) sz = sz + 4 * rArr.length;
    if (tArr != null) sz = sz + 4 * tArr.length;
    return sz;
  }

  /**
   * return rArr
   */
  public float[] getrArr() {
     int sz = rArr.length;
     float[] ret = new float[sz];
     System.arraycopy(rArr, 0, ret, 0, sz);

     return ret;
  }

  /**
   * return tArr
   */
  public float[] gettArr() {
     int sz = tArr.length;
     float[] ret = new float[sz];
     System.arraycopy(tArr, 0, ret, 0, sz);

     return ret;
  }

} // end of class


