package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;

import sv.util.*;
import sv.viewer.builtin.*;
import sv.server.filter.*;

public class TimeData2D extends TimeData  {

  // min and max values
  protected float xmin, ymin, zmin; 
  protected float xmax, ymax, zmax;
  protected float cmin, cmax;

  // raw data array
  protected float[] xArr = null, yArr = null, zArr = null, cArr = null;

  // dimension of the data  
  protected int dimX, dimY;

  // is it bounded
  protected boolean bb = false;

  // is it a normal grid
  protected boolean normal_grid = false;


  ///////////////////////////////////////////////
  // BUILTIN specific (JAVA3D VIEWING OPTION)
  G2DObject j3dObject = null;
  ContourObject[] contourObjects = null;


  /**
   * constructor for 2d dataset
   * @param time time value
   * @param xa x value array
   * @param ya y value array
   * @param za z value array
   */
  public TimeData2D(float time, float[] xa, float[] ya, float[] za) {
    System.out.println("Not support this constructor yet.");
  }

  /**
   * constructor for 2d dataset 
   * @param time time value
   * @param xmin x min value
   * @param xmax x max value
   * @param nx the number of x grid
   * @param ymin y min value
   * @param ymax y max value
   * @param ny the number of y grid
   * @param zArr z value array
   */
  public TimeData2D(TimeValue time, float xmin, float xmax, int nx, 
                    float ymin, float ymax, int ny, float[] zArr) {

     this.time = time;
     rank = TWOD;

     this.zArr = zArr;
     bb = true;
     normal_grid = true;
     noOfPoints = nx * ny;

     this.xmin = xmin;
     this.ymin = ymin;
     this.xmax = xmax;
     this.ymax = ymax;

     zmin = Float.MAX_VALUE;
     zmax = Float.MIN_VALUE;

     for (int i=0; i<noOfPoints; i++) {
        if (zmin > zArr[i]) zmin = zArr[i];
        if (zmax < zArr[i]) zmax = zArr[i];
     }

     dimX = nx;
     dimY = ny;
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) throws IOException {
  
    // send the magic number
    out.writeInt(201);

    // send time
    out.writeFloat(time.getAbsTime());

    // send number of points in given time
    out.writeInt(dimX);
    out.writeInt(dimY);
    out.writeInt(noOfPoints);

    // send x, y, and z arrays
    float[] zArr = getzArr();
    if (bb) {
      out.writeByte(1);     // bb is true... send boundbox first
      out.writeFloat(xmin);
      out.writeFloat(xmax);
      out.writeFloat(ymin);
      out.writeFloat(ymax);

      for (int i=0; i<noOfPoints; i++) 
        out.writeFloat(zArr[i]);

    } else {
      float[] xArr = getxArr();
      float[] yArr = getyArr();
      out.writeByte(0);      // bb is false

      for (int i=0; i<noOfPoints; i++) {
         out.writeFloat(xArr[i]);
         out.writeFloat(yArr[i]);
         out.writeFloat(zArr[i]);
      }
    }

    // send cArr
    if (cArr == null) {
       out.writeByte(0);
    } else {
       out.writeByte(1);
       for (int i=0; i<noOfPoints; i++) 
          out.writeFloat(cArr[i]);
    }

    // write the dummy value
    out.writeInt(0);
  }

  
  /**
   * make vertices from the x, y and z array (ASSUME GLOBAL MIN AND MAX)
   * if b is true, then global mode
   */
  public J3DVertex[] makeVertex(boolean b, float gzmin, float gzmax) {
     J3DVertex[] vertex = null;

     if (zArr == null) return null;

     if (bb) {
       xArr = new float[zArr.length];
       yArr = new float[zArr.length];

       float xinc = (float)(xmax - xmin) / (float)(dimX-1);
       float yinc = (float)(ymax - ymin) / (float)(dimY-1);

       for (int i=0; i<dimX; i++) {
          int index = i*dimY;
          for (int j=0; j<dimY; j++) {
             xArr[index+j] = (float)(xmin + xinc * i);  
             yArr[index+j] = (float)(ymin + yinc * j);  
          }
        }
     }

     try {
       vertex = new J3DVertex[noOfPoints];
     } catch(Exception e) {  // OUT OF MEMORY
       e.printStackTrace();
     } 

     // will be scaled to -1 to 1
     float gap = 1.0f;

     int maxdim = dimX;
     int mindim = dimY;
     boolean xlarge = true;
     if (maxdim < dimY) { 
        maxdim = dimY;
        mindim = dimX;
        xlarge = false;
     }

     float gapx, gapy;

     if (xlarge) {
        gapx =  2.0f / (xmax - xmin);
        gapy =  2.0f * ((float)mindim / (float)maxdim) / (ymax - ymin);
     } else {
        gapx =  2.0f * ((float)mindim / (float)maxdim) / (xmax - xmin);
        gapy =  2.0f / (ymax - ymin);
     }
     //System.out.println(gapx + " " + gapy);
 
     if (b)
        gap = 2.0f / (gzmax - gzmin);
     else 
        gap = 2.0f / (zmax - zmin); 

     float xval, yval, zval;

     for (int i=0; i<noOfPoints; i++) {
        xval = (xArr[i] - xmin) * gapx - 1.0f;
        yval = (yArr[i] - ymin) * gapy - 1.0f;
        if (b)
           zval = (zArr[i] - gzmin) * gap - 1.0f; 
        else
           zval = (zArr[i] - zmin) * gap - 1.0f; 

        vertex[i] = new J3DVertex(xval, yval, zval);
     }
 
     if (bb) {
        xArr = null;
        yArr = null;
     }
     return vertex;
  }

  /**
   * make faces from the vertices
   */
  public void makeFace(J3DVertex[] vertex, int option,
                      float[] rcolors, float[] gcolors, float[] bcolors) {
    //System.out.print("At time: " + time.getAbsTime());
    //System.out.println(" Generating faces .... ");

    J3DFace[] faces = null;

    int start_lx = 0;
    int start_ly = 0;
    int sx = dimY;
    int sy = 1;
    int end_lx = (dimX-1) * dimY;
    int end_ly = dimY-1;
    int lx = 0;
    int ly = 0;

    J3DVertex values[] = new J3DVertex[4]; 
    Vector faceVec = new Vector();

    while (ly != end_ly) {
      values[1] = vertex[lx+ly];
      values[2] = vertex[lx+ly+sy];
        
      while (lx != end_lx) {
        values[0] = values[1];
        values[1] = vertex[lx+sx+ly];
        values[3] = values[2];
        values[2] = vertex[lx+sx+ly+sy];

        faceVec.addElement(new J3DFace(values, false));
        lx += sx; 
      }
      ly += sy; lx = start_lx;
    }

    faces = new J3DFace[faceVec.size()];
    faceVec.copyInto(faces);
    faceVec.removeAllElements();
    faceVec = null;

    int index = 0, pos;
    int sz = faces.length * 4;
    Point3f[] verts = new Point3f[sz];
    //    Vector3f[] normals = new Vector3f[sz];
    Color3f[] vcolors = new Color3f[sz];
    Point3f[] tmp;
    float[] tmpcolor;

    for (int i=0; i< faces.length; i++) {
       tmp = faces[i].getVertices();
       tmpcolor = faces[i].getZInfo();

       //       normals[index] = faces[i].normal; 
       verts[index] = tmp[0];
       pos = (int)((tmpcolor[0]+1.0f) / 2.0f * 100f);

       vcolors[index] = new Color3f(rcolors[pos], gcolors[pos], bcolors[pos]); 
       index++;

       //       normals[index] = faces[i].normal; 
       verts[index] = tmp[1]; 
       pos = (int)((tmpcolor[1]+1.0f) / 2.0f * 100f);
       vcolors[index] = new Color3f(rcolors[pos], gcolors[pos], bcolors[pos]);
       index++;

       //       normals[index] = faces[i].normal;
       verts[index] = tmp[2]; 
       pos = (int)((tmpcolor[2]+1.0f) / 2.0f * 100f);
       vcolors[index] = new Color3f(rcolors[pos], gcolors[pos], bcolors[pos]);
       index++;

       //       normals[index] = faces[i].normal;
       verts[index] = tmp[3]; 
       pos = (int)((tmpcolor[3]+1.0f) / 2.0f * 100f);
       vcolors[index] = new Color3f(rcolors[pos], gcolors[pos], bcolors[pos]);
       index++;
    }

//    j3dObject = new G2DObject(option, sz, verts, normals, vcolors);
    j3dObject = new G2DObject(option, sz, verts, vcolors);
    j3dObject.makeBoundingBox(dimX, dimY);
  }



  /**
   * make contour object
   */
  public void makeContour(boolean force) {
     if (!force && contourObjects != null) return;

     float[] grid = getzArrInverted();
     int contour_lines = 5;

     IsoCurve isocurve = new IsoCurve(grid, dimX, dimY);
     Vector objvec = new Vector();
     float[] isodata;
    
     // will be scaled to -1 to 1
     int maxdim = dimX;
     if (maxdim < dimY) maxdim = dimY;
     float dimgap = 2.0f / maxdim;

     for(int i=0; i<contour_lines; i++) {
        float level = (float)((float)(i)/contour_lines*(zmax-zmin)+zmin);
        isocurve.setValue(level);

        while( (isodata = isocurve.getCurve()) != null ) {
           Vector datavec = new Vector();

           int endpt = isodata.length-3;
  	   for (int j=0; j<endpt; j=j+2) {
              datavec.addElement(new Point3f(isodata[j]*dimgap-1.0f,
                              isodata[j+1]*dimgap-1.0f, 1.0f));
              datavec.addElement(new Point3f(isodata[j+2]*dimgap-1.0f,
                              isodata[j+3]*dimgap-1.0f, 1.0f)); 
	   }

           Point3f[] verts = new Point3f[datavec.size()];
           datavec.copyInto(verts);
	   // FOR THE COLORED CONTOUR PLOT, NEED TO SEND level AND COLOR INFO
           objvec.addElement(new ContourObject(verts));
         }

     }
     isocurve = null;

     contourObjects = new ContourObject[objvec.size()];
     objvec.copyInto(contourObjects);
     objvec = null;
  }

  /**
   * return inverted zArr (since grid array is inverted, 
   *   i.e. for-loop for x-axis and y-axis is inverted)
   */
  public float[] getzArrInverted() {
     
     float[] zzArr = new float[zArr.length];
     int idx = 0;
     for (int i=0; i<dimY; i++) {
       for (int j=0; j<dimX; j++) {
          zzArr[idx] = zArr[j*dimY + i];
          idx++;
       }
     }
     
     return zzArr;
  }

  /**
   * return java 3d object
   */
  public G2DObject getJ3dObject() {
    return j3dObject;
  }

  /**
   * return contour object
   */
  public ContourObject[] getContourObject() {
    return contourObjects;
  }

  /**
   * return xArr
   */
  public float[] getxArr() {
     int length;

     length = zArr.length;

     if (bb) {
       float[] xxArr = new float[length];
       float xinc = (float)(xmax - xmin) / (float)(dimX-1);

       for (int i=0; i<dimX; i++) {
          int index = i*dimY;
          for (int j=0; j<dimY; j++) 
             xxArr[index+j] = (float)(xmin + xinc * i);  
        }
        return xxArr;
     } else {
        float[] xxArr = new float[length];
        System.arraycopy(xArr, 0, xxArr, 0, length);
        return xxArr;
     }
  }

  /**
   * return yArr
   */
  public float[] getyArr() {
     int length;

     length = zArr.length;

     if (bb) {
       float[] yyArr = new float[length];
       float yinc = (float)(ymax - ymin) / (float)(dimY-1);

       for (int i=0; i<dimX; i++) {
          int index = i*dimY;
          for (int j=0; j<dimY; j++) 
             yyArr[index+j] = (float)(ymin + yinc * j);  
        }
        return yyArr;
     } else {
        float[] yyArr = new float[length];
        System.arraycopy(yArr, 0, yyArr, 0, length);
        return yyArr;
     }
  }


  /**
   * return zArr
   */
  public float[] getzArr() {
     int length = zArr.length;
     float[] zzArr = new float[length];
     System.arraycopy(zArr, 0, zzArr, 0, length);
     return zzArr;
  }

  /**
   * return the size of this time data
   */
  public long getDataSize() {
    long sz = 4;         // initially time value
    
    if (zArr != null) sz = sz + 12 * zArr.length;
    if (cArr != null) sz = sz + 4 * cArr.length;
    return sz;
  }

  public float getxmin() {
    return xmin;
  }

  public float getxmax() {
    return xmax;
  }

  public float getymin() {
    return ymin;
  }

  public float getymax() {
    return ymax;
  }

  public float getzmin() {
    return zmin;
  }

  public float getzmax() {
    return zmax;
  }

  public float getcmin() {
    return cmin;
  }

  public float getcmax() {
    return cmax;
  }

  public int getDimX() {
    return dimX;
  }	

  public int getDimY() {
    return dimY;
  }	

  public boolean isBounded() {
    return bb;
  }

  public boolean isNormalGrid() {
    return normal_grid;
  }

} // end of class
