package sv.data;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;

import sv.util.*;
import sv.viewer.builtin.*;
import sv.server.filter.*;

public class TimeData3D extends TimeData  {

  // slice extraction
  public static int XSKIPED = 0;
  public static int YSKIPED = 1;
  public static int ZSKIPED = 2;

  // rendering option
  public static int VOLUME_RENDERING     = 0;
  public static int ISOSURFACE_RENDERING = 1;

  protected int dimx, dimy, dimz;
  protected float xmin, xmax, ymin, ymax, zmin, zmax, valmin, valmax;
  float threshold;

  // raw data array
  protected float[] zArr = null;

  /////////////////////////////////////
  // ONLY FOR JAVA3D OPTION SPECIFIC
  public float[][] vertex;
  public float[][] norms;
  public int[][] triangles;

  ////////////////////////////////////
  // FOR LARGE DATA
  String cache_name = null;

  int nvert, ntri;
  
  boolean CREATED = false;
  boolean OLDSUPPORT = false;

  G3DObject j3dObject = null;

  SliceObject xsliceObjects = null;
  SliceObject ysliceObjects = null;
  SliceObject zsliceObjects = null;

  /**
   * constructor for 3d dataset
   */
  public TimeData3D(TimeValue time, int nx, int ny, int nz, float[] arr) {
    this.time = time;
    rank = THREED;
    cache_name = String.valueOf(System.currentTimeMillis());

    dimx = nx;
    dimy = ny;
    dimz = nz;

    valmin = Float.MAX_VALUE;
    valmax = Float.MIN_VALUE;
    for (int i=0; i< arr.length; i++) {
      if (valmin > arr[i]) valmin = arr[i];
      if (valmax < arr[i]) valmax = arr[i];
    }
    zArr = arr;
  }

  /**
   * constructor for 3d dataset
   */
  public TimeData3D(int nx, int ny, int nz, float min, float max, String cache) {
    this.time = new TimeValue(0.0f);
    rank = THREED;
    cache_name = cache;

    dimx = nx;
    dimy = ny;
    dimz = nz;

    valmin = min;
    valmax = max;
  }

  /**
   * constructor for 3d dataset
   */
  public TimeData3D(TimeValue time, int nvert, float[][] vertex, 
                 float[][] norms, int ntri, int[][] triangles, 
                 int dx, int dy, int dz, 
                 float threshold, float min, float max) {
     this.time = time;
     this.nvert = nvert;
     this.norms = norms;
     this.vertex = vertex;
     this.ntri = ntri;
     this.triangles = triangles;
     this.threshold = threshold;
     rank = THREED;

     dimx = dx;
     dimy = dy;
     dimz = dz;

     /*
     xmin = Float.MAX_VALUE;
     xmax = Float.MIN_VALUE;
     ymin = Float.MAX_VALUE;
     ymax = Float.MIN_VALUE;

     for (int i=0; i<nvert; i++) {
        if (vertex[i][0] < xmin) xmin = vertex[i][0]; 
        else if (vertex[i][0] > xmax) xmax = vertex[i][0];

	if (vertex[i][1] < ymin) ymin = vertex[i][1]; 
        else if (vertex[i][1] > ymax) ymax = vertex[i][1];
     }
     */

     valmin = min;
     valmax = max;

     xmin = 0.0f;
     ymin = 0.0f;
     zmin = 0.0f;
     xmax = (float)(dimx-1);
     ymax = (float)(dimy-1);
     zmax = (float)(dimz-1);

     OLDSUPPORT = true;
     if (nvert > 0) CREATED = true;
      
     //     makeJ3DObject();

  }

  /**
   * make J3D objects from the vertices and faces
   */
  public void makeJ3DObject(int option) {
    if (ntri == 0) return;

System.out.println("makeObject at time: " + time.getAbsTime());

    float A1, A2, A3, B1, B2, B3, C1, C2, C3, vol;
    int pos1, pos2, pos3;

    float minval = xmin;
    float maxval = xmax;
       
    if (minval > ymin) minval = ymin;
    if (minval > zmin) minval = zmin;
    if (maxval < ymax) maxval = ymax;
    if (maxval < zmax) maxval = zmax;

    float gap = maxval - minval;
    float gap2 = 2.0f / gap;

    /*
    int sz = vertex.length * 3;
    float[] verts = new float[sz];
    float[] normals = new float[sz];

    int idx = 0;
    for (int i=0; i<vertex.length; i++) {
       verts[idx] = (vertex[i][0] - minval) * gap2 - 1.0f; 
       normals[idx] = -norms[i][0]; 
       idx++;

       verts[idx] = (vertex[i][1] - minval) * gap2 - 1.0f; 
       normals[idx] = -norms[i][1];
       idx++;

       verts[idx] = (vertex[i][2] - minval) * gap2 - 1.0f; 
       normals[idx] = -norms[i][2];
       idx++;
    }

    sz = ntri * 3;
    int tindex[] = new int[sz];
    idx = 0;
    for (int i=0; i<ntri; i++) {
       pos1 = triangles[i][0];
       pos2 = triangles[i][1];
       pos3 = triangles[i][2];

       A1 = vertex[pos1][0];
       A2 = vertex[pos1][1];
       A3 = vertex[pos1][2];
       B1 = vertex[pos2][0];
       B2 = vertex[pos2][1];
       B3 = vertex[pos2][2];
       C1 = vertex[pos3][0];
       C2 = vertex[pos3][1];
       C3 = vertex[pos3][2];

       vol = A1*(B2*C3 - B3*C2) + A2*(B3*C1 - B1*C3) + A3*(B1*C2 - B2*C1);

       if (vol > 0) {
          tindex[idx] = pos1; idx++;
          tindex[idx] = pos2; idx++;
          tindex[idx] = pos3; idx++;
       } else {
          tindex[idx] = pos1; idx++;
          tindex[idx] = pos3; idx++;
          tindex[idx] = pos2; idx++;
       }

    }
          
    j3dObject = new G3DObject(option, verts, normals, tindex);
    j3dObject.makeBoundingBox();
    */

    J3DFace[] faces = new J3DFace[ntri];
    J3DVertex[] tmpFace = new J3DVertex[3];
    Vector3f[] vec = new Vector3f[3];

    for (int i=0; i<ntri; i++) {

       pos1 = triangles[i][0];
       pos2 = triangles[i][1];
       pos3 = triangles[i][2];

       A1 = (vertex[pos1][0] - minval) * gap2 - 1.0f;
       A2 = (vertex[pos1][1] - minval) * gap2 - 1.0f;
       A3 = (vertex[pos1][2] - minval) * gap2 - 1.0f;
       B1 = (vertex[pos2][0] - minval) * gap2 - 1.0f;
       B2 = (vertex[pos2][1] - minval) * gap2 - 1.0f;
       B3 = (vertex[pos2][2] - minval) * gap2 - 1.0f;
       C1 = (vertex[pos3][0] - minval) * gap2 - 1.0f;
       C2 = (vertex[pos3][1] - minval) * gap2 - 1.0f;
       C3 = (vertex[pos3][2] - minval) * gap2 - 1.0f;

       if (OLDSUPPORT) {
         vol =
             A1*(B2*C3 - B3*C2) + A2*(B3*C1 - B1*C3) + A3*(B1*C2 - B2*C1);

         if (vol > 0) {
            tmpFace[0] = new J3DVertex(A1, A2, A3);
            tmpFace[1] = new J3DVertex(C1, C2, C3);
            tmpFace[2] = new J3DVertex(B1, B2, B3);
            vec[0] = new Vector3f(norms[pos1][0],norms[pos1][1],norms[pos1][2]);
            vec[1] = new Vector3f(norms[pos3][0],norms[pos3][1],norms[pos3][2]);   
            vec[2] = new Vector3f(norms[pos2][0],norms[pos2][1],norms[pos2][2]);
          
         } else {
            tmpFace[0] = new J3DVertex(A1, A2, A3);
            tmpFace[1] = new J3DVertex(B1, B2, B3);
            tmpFace[2] = new J3DVertex(C1, C2, C3);
            vec[0] = new Vector3f(norms[pos1][0],norms[pos1][1],norms[pos1][2]);
            vec[1] = new Vector3f(norms[pos2][0],norms[pos2][1],norms[pos2][2]);   
            vec[2] = new Vector3f(norms[pos3][0],norms[pos3][1],norms[pos3][2]);
         }

       } else {
          tmpFace[0] = new J3DVertex(A1, A2, A3);
          tmpFace[1] = new J3DVertex(B1, B2, B3);
          tmpFace[2] = new J3DVertex(C1, C2, C3);
       }

       faces[i] = new J3DFace(tmpFace, vec, true);
    }

    int index = 0;
    int sz = faces.length * 3;
    Point3f[] verts = new Point3f[sz];
    Vector3f[] normals = new Vector3f[sz];
    Vector3f[] tmpnorm;
    Point3f[] tmp;

    for (int i=0; i< faces.length; i++) {
       tmp = faces[i].getTriVertices();
       tmpnorm = faces[i].getNormals();

       normals[index] = tmpnorm[0];
       verts[index] = tmp[0];
       index++;

       normals[index] = tmpnorm[1];
       verts[index] = tmp[1]; 
       index++;

       normals[index] = tmpnorm[2];
       verts[index] = tmp[2]; 
       index++;
    }
    j3dObject = new G3DObject(option, sz, verts, normals);
    j3dObject.makeBoundingBox();
  }

  /**
   * make slice objects
   */
  public void makeSliceObject(GridInfo3D grids, int skipmode, 
                        float[] rcolors, float[] gcolors, float[] bcolors) {

     if (skipmode == XSKIPED)
        xsliceObjects = new SliceObject(grids, dimx, dimy, dimz, 
                    valmin, valmax, rcolors, gcolors, bcolors);
     else if (skipmode == YSKIPED)
        ysliceObjects = new SliceObject(grids, dimx, dimy, dimz, 
                    valmin, valmax, rcolors, gcolors, bcolors);
     else if (skipmode == ZSKIPED)
        zsliceObjects = new SliceObject(grids, dimx, dimy, dimz, 
                    valmin, valmax, rcolors, gcolors, bcolors);
  }


  public int getNVERT() {
     return nvert;
  }

  public int getNTRI() {
     return ntri;
  }

  public boolean isCreated() {
     return CREATED;
  }

  public G3DObject getJ3dObject() {
     return j3dObject;
  }

  /**
   * perform marching cube routine
   * @param cache_dir cache directory
   * @param thresh threshold
   */
  public void marchingCube(File cache_dir, float thresh) {
    threshold = thresh;

    int sz = dimx*dimy*dimz;
    float grid[] = new float[sz];

    if (zArr != null) {
      for(int i=0; i<sz; i++)
        grid[i] = zArr[i];

    } else {
      try {
        DataInputStream in = new DataInputStream(new BufferedInputStream(
                  new FileInputStream(new File(cache_dir, cache_name))));
        in.readInt();
        in.readFloat();
        in.readInt();
        in.readInt();
        in.readInt();

        int idx = 0;
        for(int k=0; k<dimz; k++)
          for(int i=0; i<dimy; i++)
            for(int j=0; j<dimx; j++) 
              grid[idx++] = in.readFloat();


      } catch (Exception e) {
        System.out.println("Cache file reading error.");
      }
    }

    String path = cache_dir.getAbsolutePath();
    if (!path.endsWith(File.separator))
       path = path + File.separator + cache_name;
    IsoFilter.executeFilter(path, dimx, dimy, dimz, thresh, grid);
  }


  /** 
   *  sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) throws IOException {
  }

  /**
   * return the size of this time data
   */
  public long getDataSize() {
    long sz = 4;         // initially time value
  
    if (zArr != null) sz = sz + 12 * zArr.length;
    else sz = sz + (dimx * dimy * dimz * 4);
    return sz;
  }

  public int getDimX() {
     return dimx;
  }	

  public int getDimY() {
     return dimy;
  }	

  public int getDimZ() {
     return dimz;
  }	

  public float getzmin() {
     return valmin;
  }

  public float getzmax() {
     return valmax;
  }

  public float getThreshold() {
     return threshold;
  }

  public String getCacheName() {
     return cache_name;
  }

  public boolean isCached() {
    boolean flag = false;
    if (zArr == null)
      flag = true;
    return flag;
  }

  /**
   * return grid data
   */
  public float[] getGridData(File cache_dir) {
    if (zArr != null)
      return zArr;
    else {
      try {
        DataInputStream in = new DataInputStream(new BufferedInputStream(
                  new FileInputStream(new File(cache_dir, cache_name))));
        in.readInt();
        in.readFloat();
        in.readInt();
        in.readInt();
        in.readInt();

        int sz = dimx * dimy * dimz;
        float[] grid = new float[sz];
        for(int i=0; i<sz; i++)
          grid[i] = in.readFloat();

        return grid;
      } catch (Exception e) {
        System.out.println("Cache file reading error.");
      }
    }
    return null;
  }


  /**
   * down size the data
   */
  public TimeData3D downsize(File cache_dir, int factor) {
    float[] grid = getGridData(cache_dir);

    String id = String.valueOf(System.currentTimeMillis());
    int xsize = (int)(dimx/factor);
    int ysize = (int)(dimy/factor);
    int zsize = (int)(dimz/factor);
    float min = Float.MAX_VALUE;
    float max = Float.MIN_VALUE;

    try {
      File file = new File(cache_dir, id);
      DataOutputStream outfile = 
 new DataOutputStream(new BufferedOutputStream(new FileOutputStream(file)));

      outfile.writeInt(1);   // the number of time step
      outfile.writeFloat(0.0f);   // time value

      outfile.writeInt(xsize);
      outfile.writeInt(ysize);
      outfile.writeInt(zsize);

      int totsz = xsize * ysize * zsize;

      int m1 = factor*dimy*dimx;
      int m2 = factor*dimx;
      float val;

      for(int k=0; k<zsize; k++) 
        for(int i=0; i<ysize; i++) 
          for(int j=0; j<xsize; j++) {  
             val = grid[k*m1 + i*m2 + j*factor];
             outfile.writeFloat(val);
             if (val < min) min = val;
             if (val > max) max = val;
          }

      outfile.close();

    } catch (Exception e) {
      System.out.println("Cache writing error(downsize).");
      return null;
    }

    TimeData3D td = new TimeData3D(xsize, ysize, zsize, min, max, id);
    return td;
  }

 /**
   * smoothe the data
   */
  public TimeData3D smoothe(File cache_dir) {
    float[] grid = getGridData(cache_dir);

    String id = String.valueOf(System.currentTimeMillis());
    float min = Float.MAX_VALUE;
    float max = Float.MIN_VALUE;

    // process the grid
    int m1 = dimy*dimx;
    int m2 = dimx;
    int cur, up, down, left, right, front, back, t1, t2;

    int xs = dimx-1;
    int ys = dimy-1;
    int zs = dimz-1;
    
    for(int i=1; i<zs; i++) 
      for(int j=1; j<ys; j++) 
        for(int k=1; k<xs; k++) {  
          t1 = i*m1;
          t2 = j*m2;
          cur   = t1 + t2 + k;
          up    = t1+m1 + t2 + k;
          down  = t1-m1 + t2 + k;
          left  = t1 + t2+m2 + k;
          right = t1 + t2-m2 + k;
          front = t1 + t2 + k+1;
          back  = t1 + t2 + k-1; 
          grid[cur] = grid[cur]/2.0f + (grid[up]+grid[down]+grid[left]+
                       grid[right]+grid[front]+grid[back])/12.0f;
        }

    try {
      File file = new File(cache_dir, id);
      DataOutputStream outfile = 
 new DataOutputStream(new BufferedOutputStream(new FileOutputStream(file)));

      outfile.writeInt(1);   // the number of time step
      outfile.writeFloat(0.0f);   // time value

      outfile.writeInt(dimx);
      outfile.writeInt(dimy);
      outfile.writeInt(dimz);

      int totsz = dimx * dimy * dimz;

      for(int i=0; i<totsz; i++) {
        outfile.writeFloat(grid[i]);
        if (grid[i] < min) min = grid[i];
        if (grid[i] > max) max = grid[i];
      }

      outfile.close();

    } catch (Exception e) {
      System.out.println("Cache writing error(downsize).");
      return null;
    }

    TimeData3D td = new TimeData3D(dimx, dimy, dimz, min, max, id);
    return td;
  }

 /**
  * down size and smoothe the data
  */
  public TimeData3D downsize_smoothe(File cache_dir, int factor) {
    float[] grid = getGridData(cache_dir);

    String id = String.valueOf(System.currentTimeMillis());
    int xsize = (int)(dimx/factor);
    int ysize = (int)(dimy/factor);
    int zsize = (int)(dimz/factor);
    float min = Float.MAX_VALUE;
    float max = Float.MIN_VALUE;
    int totsz = xsize * ysize * zsize;
    float[] new_grid = new float[totsz];
    int cur, up, down, left, right, front, back, t1, t2;

    int m1 = factor*dimy*dimx;
    int m2 = factor*dimx;

    // down size it
    int idx = 0;
    for(int i=0; i<zsize; i++) 
      for(int j=0; j<ysize; j++) 
        for(int k=0; k<xsize; k++) {  
          new_grid[idx] = grid[i*m1 + j*m2 + k*factor];
          idx++;
        }

    int xs = xsize-1;
    int ys = ysize-1;
    int zs = zsize-1;
    m1 = ysize*xsize;
    m2 = xsize;  
 
    for(int i=1; i<zs; i++) 
      for(int j=1; j<ys; j++) 
        for(int k=1; k<xs; k++) {  
          t1 = i*m1;
          t2 = j*m2;
          cur   = t1 + t2 + k;
          up    = t1+m1 + t2 + k;
          down  = t1-m1 + t2 + k;
          left  = t1 + t2+m2 + k;
          right = t1 + t2-m2 + k;
          front = t1 + t2 + k+1;
          back  = t1 + t2 + k-1; 
          new_grid[cur] = new_grid[cur]/2.0f + 
                       (new_grid[up]+new_grid[down]+new_grid[left]+
                       new_grid[right]+new_grid[front]+new_grid[back])/12.0f;
        }

    try {
      File file = new File(cache_dir, id);
      DataOutputStream outfile = 
 new DataOutputStream(new BufferedOutputStream(new FileOutputStream(file)));

      outfile.writeInt(1);   // the number of time step
      outfile.writeFloat(0.0f);   // time value

      outfile.writeInt(xsize);
      outfile.writeInt(ysize);
      outfile.writeInt(zsize);

      for(int i=0; i<totsz; i++) {
        outfile.writeFloat(grid[i]);
        if (grid[i] < min) min = grid[i];
        if (grid[i] > max) max = grid[i];
      }

      outfile.close();

    } catch (Exception e) {
      System.out.println("Cache writing error(downsize).");
      return null;
    }

    TimeData3D td = new TimeData3D(xsize, ysize, zsize, min, max, id);
    return td;
  }

  public SliceObject getXSliceObjects() {
    return xsliceObjects;
  }

  public SliceObject getYSliceObjects() {
    return ysliceObjects;
  }

  public SliceObject getZSliceObjects() {
    return zsliceObjects;
  }
} // end of class

