package sv.engine;

import java.io.*;
import sv.data.*;
import sv.gui.*;
import sv.viewer.*;

/**
 * the SV2 builtin engine. 
 */

public class SV2Engine {

  public static final int VIEWER_BUILTIN = 0;
  public static final int VIEWER_BRUTE   = 1;
  public static final int VIEWER_VRML    = 2;
  public static final int VIEWER_AVS     = 3;

  public static final int COLLAB_BUILTIN = 0;
  public static final int COLLAB_TANGO   = 1;

  static SV2GUI gui;
  static SV2Engine engine;

  // THIS CLIENT'S IDENTIFICATION
  static long identification = -1;

  // cache directory
  static File cache_dir = null;

  /**
   * visualization frame manager
   */
  public static ViewManager vmanager;

  /**
   * collanoration handler
   */
  public static cSendChannel sender;

  /**
   * SV2 engine
   * @param h server host name
   * @param sp server port number
   * @param cp client port number
   * @param s configuration directory
   * @param cache cache directory
   * @param c collaboration option
   * @param v viewer option
   */
  public SV2Engine(String h, int sp, int cp, String s, String cache, int c, int v) {
    engine = this;

    String cache_dir_name = "";

    if (cache == null) {
      String tmp = System.getProperty("os.name");
      if (tmp.indexOf("indow") > 0) 
        cache_dir_name = "C:\\temp\\sv2C\\";
      else
        cache_dir_name = "/tmp/sv2/sv2C/";
    } else
      cache_dir_name = cache;

    initChacheDir(cache_dir_name);

    // first setup gui
    gui = new SV2GUI(this);
    gui.setLocation(100, 100);
    gui.setSize(640, 480);
    gui.setVisible(true);

    // set up various work module and start those
    vmanager = new ViewManager(engine, v, s);
    cRecvChannel receiver = new cRecvChannel(engine, cp);
    sender = new cSendChannel(engine, h, sp, cp, c);

    receiver.start();
  }

  /**
   * received brief information of the data set from the server
   * @param brief received brief information 
   */
  public void receivedBriefInfo(DataSetBrief brief) {
    gui.addBriefInfo(brief);    
  }

  /**
   * read data from a file
   */
  public static void readFile(String filename) {

    gui.progress(0);

    try {
      DataInputStream in = new DataInputStream(
                new BufferedInputStream(new FileInputStream(filename)));
      DataReader.readDataFile(engine, in);
      in.close();

    } catch (Exception e) {
e.printStackTrace();
      gui.message("Reading file failed: " + filename + " ...", true);
      gui.progress(0);  
      return;
    }

    gui.message("Reading: done.", false);
    gui.progress(0);  
  }

  /**
   * return GUI component
   */
  public SV2GUI getGUI() {
    return gui;
  }

  /**
   * return identification
   */
  public long getID() {
    return identification;
  }

  /**
   * set identification
   */
  public void setID(long id) {
    identification = id;
  }

  /**
   * exit from SV2 session
   */
  public static void exitSession() {
    if (cache_dir != null && cache_dir.exists()) {
      String[] files = cache_dir.list();
      for (int i=0; i<files.length; i++) {
        File elem = new File(cache_dir, files[i]);
        elem.delete();
      }
      //cache_dir.delete();
    }

    try {
      sender.notifyExit(identification);
    } catch (Exception e) { }

    System.out.println("\nThe SV2 exits.");
    System.exit(0);
  }

  /**
   * return cooked cache dierctory
   */
  public File getCacheDir() {
    return cache_dir;
  }

  // initialize the cache directory
  private void initChacheDir(String dir) {
    cache_dir = new File(dir);

    if (!cache_dir.exists()) {
      if (!cache_dir.mkdir()) {
        System.out.println("Cache directory creation failed.");
        System.exit(1);
      }
    }

    if (!cache_dir.canRead() || !cache_dir.canWrite()) {
      System.out.println("Can't read or write the cache directory.");
      System.exit(1);
    }
  }


} // end of class




