package sv.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.image.ImageObserver;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import sv.engine.*;
import sv.data.*;

public class DataIconDisplay extends JComponent implements
                                            AdjustmentListener {

  SV2Engine engine;
  JScrollBar scroller;          // vertical scroll bar 
  int scrollValue = 0;

  static final int LARGE_ICON_VIEW = 0;
  static final int SMALL_ICON_VIEW = 1;
  static final int DETAIL_VIEW     = 2;

  // only for large icon view
  int ICON_WIDTH     = 80;
  int ICON_HEIGHT    = 80; 
  int STRING_OFFSET  = 60;
  int SMALL_ICON_GAP = 25;
  Point START_ICON  = new Point(150, 60);

  int displayOption = LARGE_ICON_VIEW;
      
  // double buffering
  Graphics offScrGc;
  Image offScrImage;

  Dimension offScrSize; 
  Dimension oldScrSize;
  Dimension scrSize;

  Image r1_img[], r2_img[], r3_img[];
  Image nextImg[], prevImg[];
  Image lineImg, titleImg, logoImg; 

  // icon related variables
  Vector iconVec;
  int currentIconPos = -1;
  int large_icon_col;
  int large_icon_row;

  // mouse option
  boolean mergeOption = false;

  /**
   * constructor
   * @param eng SV2Engine
   * @param scroll scroll bar 
   */
  public DataIconDisplay(SV2Engine eng, JScrollBar scroll) {
     engine = eng;
     scroller = scroll;      
     scroller.addAdjustmentListener(this);

     offScrSize = new Dimension();
     oldScrSize = new Dimension();
     scrSize = new Dimension();
     iconVec = new Vector();
     
     loadImages();
     addMouseHandle();
  }

  /**
   * set merge option (only for merge filter)
   */
  public void setMergeOption(boolean b) {
    mergeOption = b;
  }

  /**
   * change view of the icons
   * @param opt view display option
   */
  public void changeView(int opt) {
    if (opt == SV2Action.LARGE_ICON_VIEW) 
       displayOption = LARGE_ICON_VIEW;
    else if (opt == SV2Action.SMALL_ICON_VIEW) 
       displayOption = SMALL_ICON_VIEW;
    else {
       engine.getGUI().updateMIs(2, true);
       displayOption = DETAIL_VIEW;
    }

    engine.getGUI().validate();
    adjustScrollBar();
    repaint();
  }

  /**
   * add data set brief info
   */
  public void addBriefInfo(DataSetBrief brief) {
    int idx = iconVec.size();
    if (idx == 0) engine.getGUI().updateMIs(1, true);
    iconVec.addElement(new DataIcon(brief, idx));
    engine.getGUI().validate();
    adjustScrollBar();
    engine.getGUI().message("Added data information: "+brief.getDataName(), false);
    repaint();
  }

  /**
   * delete selected data set
   * @param sel data set's id to be deleted
   */
  public void deleteDataIcon(DataSetBrief brief) {
    boolean ok = false;                // if nothing to be deleted, error
    String name = brief.getDataName();

    for (Enumeration e=iconVec.elements(); e.hasMoreElements();) {
      DataIcon elem = (DataIcon)e.nextElement();
      if (elem.brief.getID() == brief.getID()) {
        iconVec.removeElement(elem);
        elem.brief.setClear();
        ok = true;
        break;
      }
    }

    if (!ok) engine.getGUI().message("Data set deletion error.", true);
    if (iconVec.size() > 0) {
      DataIcon[] icons = new DataIcon[iconVec.size()];
      iconVec.copyInto(icons);
      iconVec.removeAllElements();
      for (int i=0; i<icons.length; i++) 
        iconVec.addElement(new DataIcon(icons[i].brief, i));
    }
    currentIconPos = -1;
    engine.getGUI().updateMIs(1, false);
    engine.getGUI().updateMIs(2, false);
    engine.getGUI().validate();
    adjustScrollBar();
    engine.getGUI().message("Deleted data set: " + name, false);
    repaint();
  }

  /**
   * This method is called to repaint this canvas. 
   */
  public void paint(Graphics g) {
     scrSize = getSize();

     if ((offScrImage == null) || 
                      (scrSize.width != oldScrSize.width) || 
                      (scrSize.height != oldScrSize.height)) {
       adjustScrollBar();
     }

     // double check
     if (offScrImage == null) return;

     offScrGc.setColor(Color.white);
     offScrGc.fillRect(0, 0, offScrSize.width, offScrSize.height);
     offScrGc.drawImage(logoImg, 2, 2, this);
     offScrGc.drawImage(titleImg, 120, 10, this);
     offScrGc.drawImage(lineImg, 96, 45, this);

     drawIcons();
     drawLefts();

     g.drawImage(offScrImage, 0, -scrollValue, this);
  } 

  /**
   * Updates this component.
   */
  public void update(Graphics g) {
     paint(g);
  }

  /**
   * invoked when the value of the adjustable has changed. 
   */
  public void adjustmentValueChanged(AdjustmentEvent e) {
    if (e.getAdjustable() == scroller) {
      scrollValue = scroller.getValue();
      repaint();
    } 
  }

  /**
   * return current selected data set's info
   */
  public DataSetBrief getSelectedData() {
    if (currentIconPos < 0) return null;
    DataIcon icon = (DataIcon)iconVec.elementAt(currentIconPos);
    return icon.brief;
  }

  /**
   * return dataset information by using id
   */
  public DataSetBrief getDataSetBrief(long id) {
    for (Enumeration e=iconVec.elements(); e.hasMoreElements();) {
      DataIcon elem = (DataIcon)e.nextElement();
      if (elem.brief.getID() == id) 
        return elem.brief;
    }
    return null;
  }

  // add mouse related listener
  private void addMouseHandle() {

     MouseListener m = new MouseAdapter() {
       public void mouseReleased(MouseEvent e) {

          int release_x = e.getX();
          int release_y = e.getY() + scrollValue;

          if (release_x > 5 && release_x < 100)
            if (release_y > 190 && release_y < 205) 
               System.out.println("Go to home page, skip it right now.");
       }

       public void mousePressed(MouseEvent e) {

          int press_x = e.getX();
          int press_y = e.getY() + scrollValue;

          if (displayOption == DETAIL_VIEW) {
            int wid = scrSize.width;

            if (currentIconPos > 0) {
               if (press_x > (wid-230) && press_x < (wid-110))
                 if (press_y > START_ICON.y && press_y < (START_ICON.y+33)) 
                   currentIconPos--;
            }

            if (currentIconPos < (iconVec.size()-1)) {
               if (press_x > (wid-100) && press_x < (wid-10))
                 if (press_y > START_ICON.y && press_y < (START_ICON.y+33)) 
                   currentIconPos++;
            }

          } else {
            currentIconPos = -1;
            engine.getGUI().updateMIs(2, false);

	    for (int i=0; i<iconVec.size(); i++) {
              DataIcon icon = (DataIcon)iconVec.elementAt(i);
              if (icon.contains(press_x, press_y)) {
                 currentIconPos = i;
                 engine.getGUI().updateMIs(2, true);

                 if (e.isPopupTrigger()) { 
                   engine.getGUI().popupTriggered(e.getComponent(), 
                       e.getX(), e.getY(), icon.brief);
                   return;                 
                 }

                 if (mergeOption) {
                   engine.vmanager.mergeAction(icon.brief);
                   mergeOption = false;
                 }
   
                 break;
              }
            }
          } 

          repaint();
       }
     };
     this.addMouseListener(m);

     MouseMotionListener mm = new MouseMotionAdapter() {
       public void mouseMoved(MouseEvent e) {
          int x = e.getX();
          int y = e.getY() + scrollValue;

          if (x > 5 && x < 100)
            if (y > 190 && y < 205) {
               setCursor(new Cursor(Cursor.HAND_CURSOR));
               return;
            }        
          
          if (displayOption == DETAIL_VIEW) {
            int wid = scrSize.width;

            if (currentIconPos > 0) {
               if (x > (wid-230) && x < (wid-110))
                 if (y > START_ICON.y && y < (START_ICON.y+33)) {
                   setCursor(new Cursor(Cursor.HAND_CURSOR));
                   return;
                 }
            }

            if (currentIconPos < (iconVec.size()-1)) {
               if (x > (wid-100) && x < (wid-10))
                 if (y > START_ICON.y && y < (START_ICON.y+33)) {
                   setCursor(new Cursor(Cursor.HAND_CURSOR));
                   return;
                 }
            }

          } else {
	    for (int i=0; i<iconVec.size(); i++) {
              DataIcon icon = (DataIcon)iconVec.elementAt(i);
              if (icon.contains(x, y)) {
                 setCursor(new Cursor(Cursor.HAND_CURSOR));
                 return;
              }
            }
          } 
          setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
       }
     };
     this.addMouseMotionListener(mm);
  }


  // adjust scroll bar
  private void adjustScrollBar() {
    if (displayOption == LARGE_ICON_VIEW) {
      large_icon_col = (int)((scrSize.width - 180) / ICON_WIDTH);
      large_icon_row = (int)(iconVec.size() / large_icon_col) + 1;
      
      int wid = scrSize.width;
      int hgt = START_ICON.y + large_icon_row * ICON_HEIGHT;
      if (hgt < scrSize.height) hgt = scrSize.height;

      offScrSize = new Dimension(wid, hgt);
      offScrImage = createImage(wid, hgt);
      offScrGc = offScrImage.getGraphics();

      if (offScrSize.height > scrSize.height) {
        scroller.setMaximum(offScrSize.height);
        scroller.setVisibleAmount(scrSize.height);
        scroller.setUnitIncrement(10);
        scroller.setBlockIncrement(ICON_HEIGHT);
        scroller.setVisible(true);
      } else {
        scrollValue = 0;
        scroller.setValue(0);
        scroller.setVisible(false);
      }

      for (Enumeration e = iconVec.elements(); e.hasMoreElements();) {
        DataIcon icon = (DataIcon)e.nextElement();
        icon.invalidate = true;
      }

    } else if (displayOption == SMALL_ICON_VIEW) {

      int wid = scrSize.width;
      int hgt = START_ICON.y + iconVec.size() * SMALL_ICON_GAP;
      if (hgt < scrSize.height) hgt = scrSize.height;

      offScrSize = new Dimension(wid, hgt);
      offScrImage = createImage(wid, hgt);
      offScrGc = offScrImage.getGraphics();

      if (offScrSize.height > scrSize.height) {
        scroller.setMaximum(offScrSize.height);
        scroller.setVisibleAmount(scrSize.height);
        scroller.setUnitIncrement(10);
        scroller.setBlockIncrement(ICON_HEIGHT);
        scroller.setVisible(true);
      } else {
        scrollValue = 0;
        scroller.setValue(0);
        scroller.setVisible(false);
      }

    } else if (displayOption == DETAIL_VIEW) {
      offScrSize = new Dimension(scrSize.width, scrSize.height);
      offScrImage = createImage(scrSize.width, scrSize.height);
      offScrGc = offScrImage.getGraphics();

      if (scroller.isVisible())      
        scroller.setVisible(false);
    }
  }

  // draw icons
  private void drawIcons() {
    offScrGc.setColor(Color.black);

    if (iconVec.size() < 1) return;

    if (displayOption == DETAIL_VIEW) {

      if (currentIconPos < 0)
         currentIconPos = 0;

      DataIcon elem = (DataIcon)iconVec.elementAt(currentIconPos);
      elem.paint(true, this); 

    } else {
      for (int i=0; i<iconVec.size(); i++) {       
         DataIcon elem = (DataIcon)iconVec.elementAt(i);
         if (currentIconPos == i) 
            elem.paint(true, this); 
         else
            elem.paint(false, this); 
      }
    } 
  }

  // draw left part information
  private void drawLefts() {
    offScrGc.setColor(Color.black);

    if (currentIconPos < 0) {
       offScrGc.setFont(new Font("TimesRoman", Font.BOLD, 12));
       offScrGc.drawString("No selected icon", 5, 140);
       offScrGc.setFont(new Font("TimesRoman", Font.PLAIN, 12));
       offScrGc.drawString("No data information", 5, 155);

    } else {
       DataIcon icon = (DataIcon)iconVec.elementAt(currentIconPos);
       String title = icon.brief.getDataName();
       if (title.length() > 20 ) title = title.substring(0, 18) + "...";
       String misc1 = String.valueOf(icon.brief.getNoOfTime()) + " time steps";
       String misc2 = String.valueOf(icon.brief.getDataSize()) + " bytes";

       offScrGc.setFont(new Font("TimesRoman", Font.BOLD, 12));
       offScrGc.drawString(title, 5, 140);
       offScrGc.setFont(new Font("TimesRoman", Font.PLAIN, 12));
       offScrGc.drawString(misc1, 5, 155);
       offScrGc.drawString(misc2, 5, 170);
    }     

    offScrGc.setColor(Color.red);
    FontMetrics fm = offScrGc.getFontMetrics();

    String str = "sv2 Home Page";
    offScrGc.drawString(str, 5, 200);
    int wid = fm.stringWidth(str);
    offScrGc.drawLine(5, 200, 5+wid, 200);

  }

  class DataIcon {
    DataSetBrief brief;
    int index;
    int x, y;       // start position
    boolean invalidate = true;

    DataIcon(DataSetBrief b, int i) {
       brief = b;
       index = i;
    }

    void paint(boolean selected, ImageObserver observer) {
       
      if (displayOption == LARGE_ICON_VIEW) {
        Image img;

        if (invalidate) {
           int row = (int)(index / large_icon_col);
           int col = (int)(index % large_icon_col);

           x = START_ICON.x + col*ICON_WIDTH;
           y = START_ICON.y + row*ICON_HEIGHT;
           invalidate = false;
        }

        int rank = brief.getRank();

        if (rank == TimeData.ONED && !selected) 
           img = r1_img[1];
        else if (rank == TimeData.ONED && selected) 
           img = r1_img[2];
        else if (rank == TimeData.TWOD && !selected) 
           img = r2_img[1];
        else if (rank == TimeData.TWOD && selected) 
           img = r2_img[2];
        else if (rank == TimeData.THREED && !selected) 
           img = r3_img[1];
        else 
           img = r3_img[2];  

        offScrGc.drawImage(img, x, y, observer);
        String title = brief.getDataName();
        if (title.length() > 12) title = title.substring(0, 10) + "...";
        FontMetrics fm = offScrGc.getFontMetrics();
        int w = fm.stringWidth(title);
        int pos;
        if (w < 58) pos = x + (int)(58-w)/2; 
        else        pos = x;

        if (selected) {
          offScrGc.setColor(Color.blue);
          offScrGc.fillRect(pos-5, y+48, w+10, 17);
          offScrGc.setColor(Color.white);
        } else
           offScrGc.setColor(Color.black);

        offScrGc.drawString(title, pos, y + STRING_OFFSET);

      } else if (displayOption == SMALL_ICON_VIEW) {
        Image img;

        int rank = brief.getRank();

        if (rank == TimeData.ONED)
           img = r1_img[0];
        else if (rank == TimeData.TWOD)
           img = r2_img[0];
        else 
           img = r3_img[0];

        x = START_ICON.x;
        y = START_ICON.y + index * SMALL_ICON_GAP;

        offScrGc.drawImage(img, x, y, observer);
        String title = brief.getDataName();
        FontMetrics fm = offScrGc.getFontMetrics();
        int w = fm.stringWidth(title);

        if (selected) {
          offScrGc.setColor(Color.blue);
          offScrGc.fillRect(x+20, y-3, w+25, 17);
          offScrGc.setColor(Color.white);
        } else
           offScrGc.setColor(Color.black);

        offScrGc.drawString(title, x+22, y+10);

      } else if (displayOption == DETAIL_VIEW) {
        Image img;

        int rank = brief.getRank();

        if (rank == TimeData.ONED) 
           img = r1_img[0];
        else if (rank == TimeData.TWOD)
           img = r2_img[0];
        else
           img = r3_img[0];  

        offScrGc.drawImage(img, START_ICON.x, START_ICON.y+50, observer);

        if (index == 0) img = prevImg[0];
        else            img = prevImg[1];
        offScrGc.drawImage(img, scrSize.width-230, START_ICON.y, observer);

        if (index == (iconVec.size()-1)) img = nextImg[0];
        else                             img = nextImg[1]; 
        offScrGc.drawImage(img, scrSize.width-100, START_ICON.y, observer);

        String str = brief.getDataName();
        offScrGc.drawString(str, START_ICON.x+22, START_ICON.y+60);
   
        str = "Rank            : " + String.valueOf(brief.getRank());
        offScrGc.drawString(str, START_ICON.x+30, START_ICON.y+90);
        str = "Total time steps: " + String.valueOf(brief.getNoOfTime());
        offScrGc.drawString(str, START_ICON.x+30, START_ICON.y+110);
        str = "Total data Size : " + String.valueOf(brief.getDataSize()) + " bytes";
        offScrGc.drawString(str, START_ICON.x+30, START_ICON.y+130);

      }
    }

    boolean contains(int cx, int cy) {
      if (displayOption == LARGE_ICON_VIEW) {
        if (cx > x && cx < (x+ICON_WIDTH))
          if (cy > y && cy < (y+ICON_HEIGHT))
            return true;
      } else if (displayOption == SMALL_ICON_VIEW) {
        if (cy > y && cy < (y+SMALL_ICON_GAP))
          if (cx > x && cx < (x+250))
            return true;
      }
      return false;
    }

  } // end of inner class DataIcon


  // load icon images
  private void loadImages() {

    Class obj = getClass();

    lineImg = (new ImageIcon(obj.getResource("images/line.gif"))).getImage();
    titleImg = (new ImageIcon(obj.getResource("images/title.gif"))).getImage();
    logoImg = (new ImageIcon(obj.getResource("images/logo.gif"))).getImage();

    r1_img = new Image[3];
    r1_img[0] = (new ImageIcon(obj.getResource("images/rank1_small.gif"))).getImage();
    r1_img[1] = (new ImageIcon(obj.getResource("images/rank1.gif"))).getImage();
    r1_img[2] = (new ImageIcon(obj.getResource("images/rank1_on.gif"))).getImage();

    r2_img = new Image[3];
    r2_img[0] = (new ImageIcon(obj.getResource("images/rank2_small.gif"))).getImage();
    r2_img[1] = (new ImageIcon(obj.getResource("images/rank2.gif"))).getImage();
    r2_img[2] = (new ImageIcon(obj.getResource("images/rank2_on.gif"))).getImage();

    r3_img = new Image[3];
    r3_img[0] = (new ImageIcon(obj.getResource("images/rank3_small.gif"))).getImage();
    r3_img[1] = (new ImageIcon(obj.getResource("images/rank3.gif"))).getImage();
    r3_img[2] = (new ImageIcon(obj.getResource("images/rank3_on.gif"))).getImage();

    prevImg = new Image[2];
    prevImg[0] = (new ImageIcon(obj.getResource("images/prev_grey.gif"))).getImage();
    prevImg[1] = (new ImageIcon(obj.getResource("images/prev.gif"))).getImage();

    nextImg = new Image[2];
    nextImg[0] = (new ImageIcon(obj.getResource("images/next_grey.gif"))).getImage();
    nextImg[1] = (new ImageIcon(obj.getResource("images/next.gif"))).getImage();
  }

} // end of class

