package sv.gui;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.vecmath.Color3f;
import javax.media.j3d.*;

public class MaterialEditor extends JDialog {

  Color3f ambientColor, emissiveColor, diffuseColor, specularColor; 
  float shininess;
  int cullMode;
  String results[];

  public String[] getResults() {
    return results;
  }

  private void closeDialog() {
    setVisible(false);
  }

  // gui elements
  final Banner abanner = new Banner();
  final Banner sbanner = new Banner();
  final Banner ebanner = new Banner();
  final Banner dbanner = new Banner();
  JRadioButton mode1, mode2, mode3;

  // constructor
  public MaterialEditor(Frame frame,
              Color3f a, Color3f e, Color3f d, Color3f s, float shine, int culling) {
    super(frame, "Material Editor", true);

    ambientColor = a;
    emissiveColor = e;
    diffuseColor = d;
    specularColor = s;
    shininess = shine;
    cullMode = culling;

    //buttons
    JButton cancelButton = new JButton("Cancel");
    final JButton setButton = new JButton("Set");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        closeDialog();
      }
    });

    setButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        results = new String[14];
        results[0] = String.valueOf(ambientColor.x);
        results[1] = String.valueOf(ambientColor.y);
        results[2] = String.valueOf(ambientColor.z);
        results[3] = String.valueOf(emissiveColor.x);
        results[4] = String.valueOf(emissiveColor.y);
        results[5] = String.valueOf(emissiveColor.z);
        results[6] = String.valueOf(diffuseColor.x);
        results[7] = String.valueOf(diffuseColor.y);
        results[8] = String.valueOf(diffuseColor.z);
        results[9] = String.valueOf(specularColor.x);
        results[10] = String.valueOf(specularColor.y);
        results[11] = String.valueOf(specularColor.z);
        results[12] = String.valueOf(shininess);
        if (mode2.isSelected())
           results[13] =  String.valueOf(PolygonAttributes.CULL_BACK);
        else if (mode3.isSelected())
           results[13] =  String.valueOf(PolygonAttributes.CULL_FRONT);
        else 
           results[13] =  String.valueOf(PolygonAttributes.CULL_NONE);
        closeDialog();
      }
    });
    getRootPane().setDefaultButton(setButton);

    JTabbedPane tabbedPane = new JTabbedPane();

    ///////////////////////////////////////////////////////
    // Ambient color
    JPanel panel = new JPanel(true);
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    JLabel sliderLabel = new JLabel("Red", JLabel.LEFT);
    sliderLabel.setForeground(Color.red);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    JSlider slider = getSlider(convertToInt(ambientColor.x));
    slider.addChangeListener(new ambientRedListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Green", JLabel.LEFT);
    sliderLabel.setForeground(Color.green);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(ambientColor.y));
    slider.addChangeListener(new ambientGreenListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Blue", JLabel.LEFT);
    sliderLabel.setForeground(Color.blue);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(ambientColor.z));
    slider.addChangeListener(new ambientBlueListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    
    abanner.setPreferredSize(new Dimension(100, 90));
    abanner.setBorder(BorderFactory.createTitledBorder("Result"));
    abanner.setColor(new Color(ambientColor.x, ambientColor.y, ambientColor.z));
    abanner.setText("Ambient");
    panel.add(abanner);

    tabbedPane.addTab("Ambient", panel);
    tabbedPane.setSelectedIndex(0);

   
    ///////////////////////////////////////////////////////
    // Emissive color
    panel = new JPanel(true);
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    sliderLabel = new JLabel("Red", JLabel.LEFT);
    sliderLabel.setForeground(Color.red);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(emissiveColor.x));
    slider.addChangeListener(new emissiveRedListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Green", JLabel.LEFT);
    sliderLabel.setForeground(Color.green);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(emissiveColor.y));
    slider.addChangeListener(new emissiveGreenListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Blue", JLabel.LEFT);
    sliderLabel.setForeground(Color.blue);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(emissiveColor.z));
    slider.addChangeListener(new emissiveBlueListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    
    ebanner.setPreferredSize(new Dimension(100, 90));
    ebanner.setBorder(BorderFactory.createTitledBorder("Result"));
    ebanner.setColor(new Color(emissiveColor.x, emissiveColor.y, emissiveColor.z));
    ebanner.setText("Emissive");
    panel.add(ebanner);

    tabbedPane.addTab("Emissive", panel);
   
    ///////////////////////////////////////////////////////
    // Diffuse color
    panel = new JPanel(true);
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    sliderLabel = new JLabel("Red", JLabel.LEFT);
    sliderLabel.setForeground(Color.red);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(diffuseColor.x));
    slider.addChangeListener(new diffuseRedListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Green", JLabel.LEFT);
    sliderLabel.setForeground(Color.green);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(diffuseColor.y));
    slider.addChangeListener(new diffuseGreenListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Blue", JLabel.LEFT);
    sliderLabel.setForeground(Color.blue);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(diffuseColor.z));
    slider.addChangeListener(new diffuseBlueListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    
    dbanner.setPreferredSize(new Dimension(100, 90));
    dbanner.setBorder(BorderFactory.createTitledBorder("Result"));
    dbanner.setColor(new Color(diffuseColor.x, diffuseColor.y, diffuseColor.z));
    dbanner.setText("Diffuse");
    panel.add(dbanner);

    tabbedPane.addTab("Diffuse", panel);

    ///////////////////////////////////////////////////////
    // Specular color
    panel = new JPanel(true);
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    sliderLabel = new JLabel("Red", JLabel.LEFT);
    sliderLabel.setForeground(Color.red);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(specularColor.x));
    slider.addChangeListener(new specularRedListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Green", JLabel.LEFT);
    sliderLabel.setForeground(Color.green);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(specularColor.y));
    slider.addChangeListener(new specularGreenListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    sliderLabel = new JLabel("Blue", JLabel.LEFT);
    sliderLabel.setForeground(Color.blue);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = getSlider(convertToInt(specularColor.z));
    slider.addChangeListener(new specularBlueListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 10)));

    panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    
    sbanner.setPreferredSize(new Dimension(100, 90));
    sbanner.setBorder(BorderFactory.createTitledBorder("Result"));
    sbanner.setColor(new Color(specularColor.x, specularColor.y, specularColor.z));
    sbanner.setText("Specular");
    panel.add(sbanner);

    tabbedPane.addTab("Specular", panel);

    ///////////////////////////////////////////////////////
    // Others

    panel = new JPanel(true);
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    sliderLabel = new JLabel("Shininess", JLabel.LEFT);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);

    slider = new JSlider(JSlider.HORIZONTAL, 0, 100, (int)shininess);
    slider.setMajorTickSpacing(20);
    slider.setMinorTickSpacing(5);
    slider.setPaintTicks(true);
    // turns on labels at major tick marks
    // requires spacing for major ticks to be non-zero
    slider.setPaintLabels(true);
    slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
    slider.addChangeListener(new shininessListener());

    panel.add(sliderLabel);
    panel.add(slider);
    panel.add(Box.createRigidArea(new Dimension(0, 30)));

    sliderLabel = new JLabel("Face Cull Mode", JLabel.LEFT);
    sliderLabel.setAlignmentX(Component.LEFT_ALIGNMENT);   

    panel.add(sliderLabel);
    panel.add(mode1 = new JRadioButton("CULL_NONE"));
    panel.add(mode2 = new JRadioButton("CULL_BACK"));
    panel.add(mode3 = new JRadioButton("CULL_FRONT"));
    mode1.setSelected(true);
    ButtonGroup group = new ButtonGroup();
    group.add(mode1);
    group.add(mode2);
    group.add(mode3);

    panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

    tabbedPane.addTab("Others", panel);

    //Lay out the buttons from left to right.
    JPanel buttonPane = new JPanel();
    buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
    buttonPane.setBorder(BorderFactory.createEmptyBorder(20, 10, 10, 10));
    buttonPane.add(Box.createHorizontalGlue());
    buttonPane.add(setButton);
    buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
    buttonPane.add(cancelButton);

    //Put everything together, using the content pane's BorderLayout.
    Container contentPane = getContentPane();
    contentPane.add(tabbedPane, BorderLayout.CENTER);
    contentPane.add(buttonPane, BorderLayout.SOUTH);

    pack();
  }

  class specularRedListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        specularColor.x = convertToFloat(val);
        Color c = new Color(specularColor.x, specularColor.y, specularColor.z);
        sbanner.setColor(c);
      }
    }
  }    

  class specularGreenListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        specularColor.y = convertToFloat(val);
        Color c = new Color(specularColor.x, specularColor.y, specularColor.z);
        sbanner.setColor(c);
      }
    }
  }    

  class specularBlueListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        specularColor.z = convertToFloat(val);
        Color c = new Color(specularColor.x, specularColor.y, specularColor.z);
        sbanner.setColor(c);
      }
    }
  }    


  class emissiveRedListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        emissiveColor.x = convertToFloat(val);
        Color c = new Color(emissiveColor.x, emissiveColor.y, emissiveColor.z);
        ebanner.setColor(c);
      }
    }
  }    

  class emissiveGreenListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        emissiveColor.y = convertToFloat(val);
        Color c = new Color(emissiveColor.x, emissiveColor.y, emissiveColor.z);
        ebanner.setColor(c);
      }
    }
  }    

  class emissiveBlueListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        emissiveColor.z = convertToFloat(val);
        Color c = new Color(emissiveColor.x, emissiveColor.y, emissiveColor.z);
        ebanner.setColor(c);
      }
    }
  }    


  class diffuseRedListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        diffuseColor.x = convertToFloat(val);
        Color c = new Color(diffuseColor.x, diffuseColor.y, diffuseColor.z);
        dbanner.setColor(c);
      }
    }
  }    

  class diffuseGreenListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        diffuseColor.y = convertToFloat(val);
        Color c = new Color(diffuseColor.x, diffuseColor.y, diffuseColor.z);
        dbanner.setColor(c);
      }
    }
  }    

  class diffuseBlueListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        diffuseColor.z = convertToFloat(val);
        Color c = new Color(diffuseColor.x, diffuseColor.y, diffuseColor.z);
        dbanner.setColor(c);
      }
    }
  }    


  class ambientRedListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        ambientColor.x = convertToFloat(val);
        Color c = new Color(ambientColor.x, ambientColor.y, ambientColor.z);
        abanner.setColor(c);
      }
    }
  }    

  class ambientGreenListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        ambientColor.y = convertToFloat(val);
        Color c = new Color(ambientColor.x, ambientColor.y, ambientColor.z);
        abanner.setColor(c);
      }
    }
  }    

  class ambientBlueListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        ambientColor.z = convertToFloat(val);
        Color c = new Color(ambientColor.x, ambientColor.y, ambientColor.z);
        abanner.setColor(c);
      }
    }
  }  

  class shininessListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if (!source.getValueIsAdjusting()) {
        int val = (int)((JSlider)e.getSource()).getValue();
        shininess = (float)val;
      }
    }
  }  

  // make slider
  JSlider getSlider(int initial) {
    JSlider slider = new JSlider(JSlider.HORIZONTAL, 0, 255, initial);
    slider.setMajorTickSpacing(50);
    slider.setMinorTickSpacing(10);
    slider.setPaintTicks(true);
    // turns on labels at major tick marks
    // requires spacing for major ticks to be non-zero
    slider.setPaintLabels(true);
    slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
    return slider;
  }

  // convert to int value of color
  int convertToInt(float val) {
    return (int)(val * 255);
  }

  // convert to float value of color
  float convertToFloat(int val) {
    return (float)val / 255f;
  }

} // end of class



class Banner extends JComponent {

    private Color currentColor = Color.blue;
    private Color textColor = Color.yellow;
    private String text = "Ambient";

    public void paintComponent(Graphics g) {
        g.setColor(currentColor);

        //Draw the background
        Dimension drawHere = getSize();
        Insets borderInsets = getBorder().getBorderInsets(this);
        g.fillRect(borderInsets.left, borderInsets.top,
                   (drawHere.width - borderInsets.right - borderInsets.left),
                   (drawHere.height - borderInsets.top - borderInsets.bottom));

        //Draw the text
        g.setColor(textColor);
        g.setFont(new Font("SansSerif", Font.BOLD, 24));

        FontMetrics fontMetrics = g.getFontMetrics();
        int stringWidth = fontMetrics.stringWidth(text);
        int stringHeight = fontMetrics.getHeight();
        g.drawString(text, drawHere.width/2 - stringWidth/2,
                     drawHere.height/2 + stringHeight/2);
    }

    //Setters and getters
    public void setColor(Color newColor) {
        currentColor = newColor;
        repaint();
    }

    public Color getColor() {
        return currentColor;
    }

    public void setTextColor(Color newColor) {
        textColor = newColor;
        repaint();
    }

    public Color getTextColor() {
        return textColor;
    }

    public void setText(String newText) {
        text = newText;
        repaint();
    }

    public String getText() {
        return text;
    }
}
