package sv.gui;

import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

public class ProgressBar extends JPanel {
  Image offscreen;

  int percent = 0;

  public ProgressBar() {
     setLayout(new BorderLayout());
     setBorder(Borders.lightLoweredBorder);
  }  

  public Dimension getPreferredSize() {
    return new Dimension(100, 20);  
  }

  /**
   * null out the offscreen buffer as part of invalidation
   */
  public void invalidate() {
    super.invalidate();
    offscreen = null;
  }

  /**
   * override update to *not* erase the background before painting
   */
  public void update(Graphics g) {
    paint(g);
  }

  /**
   * paint children into an offscreen buffer, then blast entire image
   * at once.
   */
  public void paint(Graphics g) {
    super.paint(g);

    int wid = getSize().width;
    int hgt = getSize().height;
    String label = String.valueOf(percent) + "%";

    if(offscreen == null) 
      offscreen = createImage(wid, hgt);

    Graphics og = offscreen.getGraphics();

    if (percent <= 0 || percent > 100) {
      og.setColor(getBackground());
      og.fillRect(0, 0, wid, hgt);
    } else {
      int val = (int)(wid/100.0 * percent);
      og.setColor(getBackground().darker());
      og.fillRect(0, 0, val, hgt);

      // draw the label centered in the button
      Font f = getFont();
      if(f != null) {
        FontMetrics fm = getFontMetrics(getFont());
        og.setColor(Color.blue);
        og.drawString(label, 
                   wid/2 - fm.stringWidth(label)/2, 
                   hgt/2 + fm.getMaxDescent());
      }
    }

    g.drawImage(offscreen, 4, 4, wid-8, hgt-8, null);
    og.dispose();
  }

  /**
   * set percent value
   * @param val percent value
   */
  public void setPercent(int val) {
    percent = val;
    invalidate();
    repaint();
  }

} // end of class
