package sv.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import sv.engine.*;
import sv.data.*;
import sv.config.Config3D;

public class SV2GUI extends JFrame {

  public DataIconDisplay iconDisplay; // friend with parent

  private ProgressBar progressBar;
  private StatusBar statusBar;
  private JPopupMenu popup;

  private Dialog aboutBox = null;

  SV2Engine engine;

  // menu items and buttons
  private JMenuItem viewitMI, visualMI, saveasMI, deleteMI,
                    cutMI, copyMI, pasteMI, pastePopMI,
                    lviewMI, sviewMI, dviewMI;
  private JCheckBoxMenuItem globalMI;                   
  private JButton cutBtn, copyBtn, pasteBtn, deleteBtn, viewBtn;

  public SV2GUI(SV2Engine eng) {
    super("The NPAC SV2");
    engine = eng;
    initialize();
  }

  /**
   * return if it is global action mode
   */
  public boolean isGlobalActionMode() {
    return globalMI.getState();
  }

  /**
   * display message to the status bar
   */
  public void message(String msg, boolean b) {
    statusBar.display(msg, b);
  }

  /**
   * display progress information
   */
  public void progress(int percent) {
    progressBar.setPercent(percent);
  }

  /**
   * add data set brief info
   */
  public void addBriefInfo(DataSetBrief brief) {
    iconDisplay.addBriefInfo(brief);
  }

  /**
   * popup menu triggered
   */
  public void popupTriggered(Component invoker, 
                               int x, int y, DataSetBrief brief) {
    popup.show(invoker, x, y);
  }

  /**
   * enable or diable the components according to current system state
   * @param type type 1 - icon view
   *             type 2 - cut, copy, delete, visualize, view in editor, save as
   *             type 3 - paste
   * @param enabled if true, enable the components 
   */
  public void updateMIs(int type, boolean enabled) {
    switch (type) {
      case 1:
        lviewMI.setEnabled(enabled); 
        sviewMI.setEnabled(enabled); 
        dviewMI.setEnabled(enabled); 
        break;

      case 2:
        cutMI.setEnabled(enabled); 
        copyMI.setEnabled(enabled); 
        cutBtn.setEnabled(enabled); 
        copyBtn.setEnabled(enabled); 
        deleteMI.setEnabled(enabled); 
        deleteBtn.setEnabled(enabled); 
        viewBtn.setEnabled(enabled); 
        viewitMI.setEnabled(enabled); 
        visualMI.setEnabled(enabled); 
        saveasMI.setEnabled(enabled); 

        if (engine.vmanager.getClipboard() != null) {
          pasteMI.setEnabled(enabled); 
          pastePopMI.setEnabled(enabled); 
          pasteBtn.setEnabled(enabled); 
        } else {
          pasteMI.setEnabled(false); 
          pastePopMI.setEnabled(false); 
          pasteBtn.setEnabled(false); 
        }

        break;
    }
  }

  // initialize GUI
  private void initialize() {
    JMenuItem mi;

    setJMenuBar(createMenuBar());

    // popup menu creation
    popup = new JPopupMenu("Data set: ");     

    mi = new JMenuItem("Visualize");
    mi.addActionListener(visualListener);
    popup.add(mi);

    mi = new JMenuItem("View in Editor");
    mi.addActionListener(viewitListener);
    popup.add(mi);
  
    popup.addSeparator();
     
    mi = new JMenuItem("Cut");
    mi.addActionListener(cutListener);
    popup.add(mi);

    mi = new JMenuItem("Copy");
    mi.addActionListener(copyListener);
    popup.add(mi);

    pastePopMI = new JMenuItem("Paste");
    pastePopMI.addActionListener(pasteListener);
    popup.add(pastePopMI);

    popup.addSeparator();

    mi = new JMenuItem("Save as...");
    mi.addActionListener(saveasListener);
    popup.add(mi);

    popup.addSeparator();
   
    mi = new JMenuItem("Delete");
    mi.addActionListener(deleteListener);
    popup.add(mi);

    getContentPane().add(popup);
    getContentPane().setLayout(new BorderLayout());

    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(true);

    cutBtn = addTool(toolBar, "cut");
    cutBtn.addActionListener(cutListener);
    cutBtn.setEnabled(false);

    copyBtn = addTool(toolBar, "copy");
    copyBtn.addActionListener(copyListener);
    copyBtn.setEnabled(false);

    pasteBtn = addTool(toolBar, "paste");
    pasteBtn.addActionListener(pasteListener);
    pasteBtn.setEnabled(false);
 
    toolBar.addSeparator();

    deleteBtn = addTool(toolBar, "delete");
    deleteBtn.addActionListener(deleteListener);
    deleteBtn.setEnabled(false);

    viewBtn = addTool(toolBar, "view"); 
    viewBtn.addActionListener(visualListener);
    viewBtn.setEnabled(false);

    toolBar.addSeparator();

    JButton btn = addTool(toolBar, "server");
    btn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Server action");
      }
    });

    toolBar.putClientProperty( "JToolBar.isRollover", Boolean.FALSE);

    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new BorderLayout());
    JScrollBar scroller = new JScrollBar();
    scroller.setVisible(false);

    controlPanel.add("Center", iconDisplay = 
                            new DataIconDisplay(engine, scroller));
    controlPanel.add("East", scroller);

    JPanel bottom = new JPanel();
    bottom.setLayout(new BorderLayout());
    progressBar = new ProgressBar();
    statusBar = new StatusBar();

    bottom.add("West", progressBar);
    bottom.add("Center", statusBar);

    getContentPane().add("North", toolBar);
    getContentPane().add("Center", controlPanel);
    getContentPane().add("South", bottom);
    popup.setInvoker(iconDisplay);
 
    WindowListener l = new WindowAdapter() {
       public void windowClosing(WindowEvent e) {
         SV2Engine.exitSession();
       }
    };
    this.addWindowListener(l);
  }

  // create the menu bar
  private JMenuBar createMenuBar() {
     JMenuItem mi;

     JMenuBar menuBar = new JMenuBar();

     // File Menu
     JMenu file = (JMenu) menuBar.add(new JMenu("File"));

     mi = (JMenuItem) file.add(new JMenuItem("Open"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         FileDialog openFileDialog = new FileDialog(SV2GUI.this,
                        "Open local data file", FileDialog.LOAD);
         openFileDialog.show();

         String fname = openFileDialog.getFile();
         if (fname == null) 
            return;

         fname = openFileDialog.getDirectory() + fname;
         SV2Engine.readFile(fname);
       }
     });

     saveasMI = (JMenuItem) file.add(new JMenuItem("Save As..."));
     saveasMI.addActionListener(saveasListener);
     saveasMI.setEnabled(false);

     file.add(new JSeparator());
     mi = (JMenuItem) file.add(new JMenuItem("Exit"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         SV2Engine.exitSession();
       }
     });

     menuBar.add(file);

     // Edit Menu
     JMenu edit = (JMenu) menuBar.add(new JMenu("Edit"));

     cutMI = (JMenuItem) edit.add(new JMenuItem("Cut"));
     cutMI.addActionListener(cutListener);
     cutMI.setEnabled(false);

     copyMI = (JMenuItem) edit.add(new JMenuItem("Copy"));
     copyMI.addActionListener(copyListener);
     copyMI.setEnabled(false);

     pasteMI = (JMenuItem) edit.add(new JMenuItem("Paste"));
     pasteMI.addActionListener(pasteListener);
     pasteMI.setEnabled(false);

     edit.add(new JSeparator());

     deleteMI = (JMenuItem) edit.add(new JMenuItem("Delete"));
     deleteMI.addActionListener(deleteListener);
     deleteMI.setEnabled(false);

     edit.add(new JSeparator());
 
     mi = (JMenuItem) edit.add(new JMenuItem("Preferences..."));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
	 System.out.println("Preference");
       }
     });

     menuBar.add(edit);

     // View Menu
     JMenu view = (JMenu) menuBar.add(new JMenu("View"));

     visualMI = (JMenuItem) view.add(new JMenuItem("Visualize"));
     visualMI.addActionListener(visualListener);
     visualMI.setEnabled(false);

     viewitMI = (JMenuItem) view.add(new JMenuItem("View in Editor"));
     viewitMI.addActionListener(viewitListener);
     viewitMI.setEnabled(false);
 
     view.add(new JSeparator());
     
     lviewMI = (JMenuItem) view.add(new JMenuItem("Large Icon"));
     lviewMI.setEnabled(false);
     lviewMI.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         iconDisplay.changeView(SV2Action.LARGE_ICON_VIEW);
       }
     });
   
     sviewMI = (JMenuItem) view.add(new JMenuItem("Small Icon"));
     sviewMI.setEnabled(false);
     sviewMI.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         iconDisplay.changeView(SV2Action.SMALL_ICON_VIEW);
       }
     });

     dviewMI = (JMenuItem) view.add(new JMenuItem("Detail View"));
     dviewMI.setEnabled(false);
     dviewMI.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         iconDisplay.changeView(SV2Action.DETAIL_VIEW);
       }
     });

     menuBar.add(view);

     // Utility Menu
     JMenu utility = (JMenu) menuBar.add(new JMenu("Utility"));

     JMenu action = (JMenu) utility.add(new JMenu("Action"));
     
     globalMI = (JCheckBoxMenuItem) action.add(new 
                                 JCheckBoxMenuItem("Global Mode"));
     globalMI.setState(false);
     globalMI.addItemListener(new ItemListener() {
       public void itemStateChanged(ItemEvent e) {
         System.out.println("Global action mode");
       }
     });

     utility.add(new JSeparator());

     mi = (JMenuItem) utility.add(new JMenuItem("Chat"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         System.out.println("Chat");
       }
     });

     mi = (JMenuItem) utility.add(new JMenuItem("Drawing Tool"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         System.out.println("Drawing tool");
       }
     });

     mi = (JMenuItem) utility.add(new JMenuItem("Filter IDE"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         System.out.println("Filter IDE");
       }
     });
 
     menuBar.add(utility);

     // Help Menu
     JMenu help = (JMenu) menuBar.add(new JMenu("Help"));

     mi = (JMenuItem) help.add(new JMenuItem("Guide"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         System.out.println("Guide");
       }
     });

     mi = (JMenuItem) help.add(new JMenuItem("About"));
     mi.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         if(aboutBox == null) {
           aboutBox = new Dialog(SV2GUI.this, "About sv2", false);
           JPanel aboutPanel = new JPanel(new BorderLayout());
           ImageIcon aboutPicture = 
               new ImageIcon(getClass().getResource("images/about.gif"),
                                                               "about sv2");
           aboutBox.add(aboutPanel, BorderLayout.CENTER);
           JLabel aboutLabel = (new JLabel(aboutPicture));
           aboutPanel.add(aboutLabel, BorderLayout.CENTER);
           JPanel buttonPanel = new JPanel(true);
           aboutPanel.add(buttonPanel, BorderLayout.SOUTH);
           JButton button = (JButton) buttonPanel.add(new JButton("Ok"));
           button.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
               aboutBox.setVisible(false);
             }
           });
         }
         aboutBox.pack();
         aboutBox.show();
       }
     });

     //     menuBar.setHelpMenu(help);
     menuBar.add(help);
     return menuBar;
  }

  // add item into ToolBar
  private JButton addTool(JToolBar toolBar, String name) {
     JButton b = (JButton)toolBar.add(new JButton(name, 
         new ImageIcon(getClass().getResource("images/"+name+".gif"), name)));
     b.setMargin(new Insets(0,0,0,0));
     return b;
  }

  // cut action
  ActionListener cutListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      DataSetBrief sel = iconDisplay.getSelectedData();   
      if (sel == null) return;      // double check

      if (!sel.isPrivate() && engine.sender.getIsMaster()) {
        String comm = String.valueOf(SV2Action.CUT) + ";" +
                  	              String.valueOf(sel.getID()); 
        byte[] datum = comm.getBytes();
       
        try {
	  engine.sender.serverCommand(datum);
          engine.sender.broadcastCommand(datum);
        } catch (Exception E) {
          System.err.println("Error: connection to server");
        }
      }
      engine.vmanager.setClipboard(sel);
      engine.vmanager.deleteDataSet(sel);
    }
  };
   
  // copy action
  ActionListener copyListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      DataSetBrief sel = iconDisplay.getSelectedData();   
      if (sel == null) return;      // double check

      if (!sel.isPrivate() && engine.sender.getIsMaster()) {
        String comm = String.valueOf(SV2Action.COPY) + ";" +
                  	              String.valueOf(sel.getID()); 
        byte[] datum = comm.getBytes();
       
        try {
	  engine.sender.serverCommand(datum);
          engine.sender.broadcastCommand(datum);
        } catch (Exception E) {
          System.err.println("Error: connection to server");
        }
      }
      engine.vmanager.setClipboard(sel);
    }
  };

  // paste action
  ActionListener pasteListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      DataSetBrief sel = iconDisplay.getSelectedData();   
      if (sel == null) return;      // double check

      if (engine.vmanager.getClipboard() == null)
        return;                      // double check

      if (!sel.isPrivate() && engine.sender.getIsMaster()) {
        String comm = String.valueOf(SV2Action.PASTE) + ";" +
              	             String.valueOf(sel.getID());
        byte[] datum = comm.getBytes();
       
        try {
	  engine.sender.serverCommand(datum);
          engine.sender.broadcastCommand(datum);
        } catch (Exception E) {
          System.err.println("Error: connection to server");
        }
      }
      engine.vmanager.pasteAction(sel);
    }
  };

  // delete action
  ActionListener deleteListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      DataSetBrief sel = iconDisplay.getSelectedData();   
      if (sel == null) return;      // double check

      if (!sel.isPrivate()) {
         // does NOT need to add global state in this case
        String comm = String.valueOf(SV2Action.DELETE) + ";" +
                  	              String.valueOf(sel.getID()); 
        byte[] datum = comm.getBytes();
       
        try {
          engine.sender.serverCommand(datum);
          engine.sender.broadcastCommand(datum);
        } catch (Exception E) {
          System.err.println("Error: connection to server");
        }

        if (engine.sender.getIsMaster())
          engine.vmanager.deleteDataSet(sel);
        else
          message("(Slave)Event action to shared data was disabled.", true);
      } else
        engine.vmanager.deleteDataSet(sel);
    }
  };

  // visualization action
  ActionListener visualListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      DataSetBrief sel = iconDisplay.getSelectedData(); 
      if (sel == null) return;      // double check

      long viewerid = sel.getViewerID();
      if (viewerid > 0) {
        message("It is already visualized. Ignore the event.", true);
        return;
      }

      byte MasterSlave = 0;
      if (engine.sender.getIsMaster()) MasterSlave = 1;  // master
      long id = System.currentTimeMillis();   

      String comm = null;
      // does NOT need to add global state in this case
      if (sel.getRank() == TimeData.THREED) {
        RenderingChooser dialog = new RenderingChooser(SV2GUI.this);
        dialog.show();
        int opt = dialog.getSelected();
        if (opt < 0) return;

        if (opt == TimeData3D.ISOSURFACE_RENDERING) {
          comm = String.valueOf(SV2Action.VISUALIZE) + ";" +
              String.valueOf(sel.getID()) + ";" + 
              String.valueOf(opt) + ";" + 
              String.valueOf(id) + ";" +
              String.valueOf(engine.getID()) + ";" +
              String.valueOf(MasterSlave);
        } else {
          Config3D config = engine.vmanager.getConfig().getConfig3D();
          double[] rayColor = config.getRayColor();
          double r = rayColor[0];
          double g = rayColor[1];
          double b = rayColor[2];

          comm = String.valueOf(SV2Action.VISUALIZE) + ";" +
              String.valueOf(sel.getID()) + ";" + 
              String.valueOf(opt) + ";" + 
              String.valueOf(id) + ";" +
              String.valueOf(engine.getID()) + ";" +
              String.valueOf(r) + ";" +
              String.valueOf(g) + ";" +
              String.valueOf(b) + ";" +
              String.valueOf(config.getRotationX()) + ";" +
              String.valueOf(config.getRotationY()) + ";" +
              String.valueOf(MasterSlave);
        }         
      } else
        comm = String.valueOf(SV2Action.VISUALIZE) + ";" +
            String.valueOf(sel.getID()) + ";" + 
            String.valueOf(id) + ";" +
            String.valueOf(engine.getID()) + ";" +
            String.valueOf(MasterSlave);

      byte[] datum = comm.getBytes();
       
      try {
         engine.sender.forceServerCommand(datum);
      } catch (Exception E) {
         System.err.println("Error: connection to server");
      }
    }
  };

  // viewit action
  ActionListener viewitListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      System.out.println("View in Editor action");
    }
  };

  // save as action
  ActionListener saveasListener = new ActionListener() {
    public void actionPerformed(ActionEvent event) {
      DataSetBrief sel = iconDisplay.getSelectedData(); 
      if (sel == null) return;      // double check

      FileDialog openFileDialog = new FileDialog(SV2GUI.this,
                                 "Save DataSet", FileDialog.SAVE);
      openFileDialog.show();
      String input = openFileDialog.getFile();
      if (input == null) return;

      boolean b = input.indexOf(".") >=0 ? true : false; 
      if (!b)  input += ".sv2";
      input = openFileDialog.getDirectory() + input;

      DataOutputStream os = null;
      try {
        os = new DataOutputStream(
               new BufferedOutputStream(new FileOutputStream(input)));
        sel.writeToFile(os);
	os.writeByte(0);   // no aux info here
        os.close();

      } catch (Exception e) {
        System.out.println("File writing error...");
        return;
      }


    }
  };

} // end of class
