package sv.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.event.*;

public class UserInputDialog extends Dialog implements ActionListener {
  public static int INT_CHOOSER   = 0;
  public static int FLOAT_CHOOSER = 1;

  private JSlider slider;
  private JTextField slideValue;

  // choosed value by the user
  Number choosed;

  // if the user clicked the cancel button
  boolean cancel_flag = false;

  // option if the slider will use float or int
  int option;

  // min and max value for the slider
  Number min, max;

  /**
   * constructor
   */
  public UserInputDialog(Frame frame, String title, String lbl1, String lbl2, 
                             int opt, Number min, Number max, Number val) {

     super(frame, title, true);   // make it modal
     choosed = val;
     this.min = min;
     this.max = max;
     this.option = opt;
     makeGUI(lbl1, lbl2);
     pack();
  }

  public void actionPerformed(ActionEvent e) {
    String command = e.getActionCommand();

    if (command.equals("Cancel")) {
      cancel_flag = true;    
    } else {
      if (option == INT_CHOOSER) {
        int sval = Integer.parseInt(slideValue.getText());
        if (sval < min.intValue() || sval > max.intValue()) {
          cancel_flag = true;
          System.out.println("Invalid range.");
        }
        choosed = new Integer(sval);

      } else {
        float sval = Float.valueOf(slideValue.getText()).floatValue(); 
        if (sval < min.floatValue() || sval > max.floatValue()) {
          cancel_flag = true;
          System.out.println("Invalid range.");
        }
        choosed = new Float(sval);
      }
    } 

    dispose();
  }

  /**
   * return the user selected value
   */
  public Number getChoosedValue() {
    return choosed;
  }

  /**
   * if the user click the cancel button
   */
  public boolean clickedCancel() {
    return cancel_flag;
  }

  public Dimension getPreferredSize() {
    return new Dimension(320, 160);
  }

  private void makeGUI(String lbl1, String lbl2) {

     setLayout(new BorderLayout());

     JPanel p = new JPanel();
     p.setBorder(new CompoundBorder(new EmptyBorder(5,5,5,5), 
                                    new SoftBevelBorder(BevelBorder.RAISED)));
     p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

     p.add(new Label(lbl1));
     p.add(new Label(lbl2));

     JPanel p2 = new JPanel();
     p2.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
     p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
     p2.add(slider = 
                  new JSlider(JSlider.HORIZONTAL, 0, 100, getSliderValue()));
     p2.add(slideValue = new JTextField(5));
     slideValue.setText(getTextValue());

     slider.putClientProperty( "JSlider.isFilled", Boolean.TRUE ); 
     p.add(p2);

     //p.add(Box.createRigidArea(new Dimension(1, 20)));
     createSlideListener();

     JPanel btnp = new JPanel();
     btnp.setLayout(new FlowLayout(FlowLayout.RIGHT));

     Button button = new Button("  Ok  ");
     button.addActionListener(this);
     btnp.add(button);

     button = new Button("Cancel");
     button.addActionListener(this);
     btnp.add(button);

     add("Center", p);
     add("South", btnp);
  }

  // slide value changes
  void createSlideListener() {
     ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
	  slideValue.setText(getTextValue());
        }
     };
     slider.addChangeListener(changeListener);
  }

  // change the textfield value
  String getTextValue() {
    if (option == INT_CHOOSER) {
      double val = min.intValue() + 
                 slider.getValue() * (max.intValue()-min.intValue()) / 100;
      return String.valueOf((int)val);

    } else {
      float val = min.floatValue() + 
       (float)slider.getValue() * (max.floatValue()-min.floatValue()) / 100.0f;
      return String.valueOf(val);
    }
  }

  // get the slider value
  int getSliderValue() {
    if (option == INT_CHOOSER) {
      double val = (choosed.intValue()-min.intValue()) / 
                                         (max.intValue()-min.intValue());
      return (int)(val * 100.0);        
    } else {
      float val = (choosed.floatValue()-min.floatValue()) / 
                                         (max.floatValue()-min.floatValue());
      return (int)(val * 100.0f); 
    }
  }

 
} // end of class



