package sv.server;

import java.io.*;
import java.net.*;
import java.util.*;

public class RecvChannel extends Thread {

  private ServerSocket sock = null;
  private SV2Server server;

  private boolean listening = true;
  private File cache_dir;
 

  /**
   * constructor
   * @param p port for the server socket
   * @param serv parent module(SV2Server)
   */
  public RecvChannel(int p, File cache, SV2Server serv) {
    server = serv;
    cache_dir = cache;

    try {
      sock = new ServerSocket(p);
    } catch(Exception e) { 
      System.err.println("Can't create SV2 server socket.");
      System.exit(1);
    }  
  }

  /**
   * waits for a connection 
   */
  public void run() {
    Vector convec = new Vector(5, 2);
     
    // continuously listen the socket
    while (listening) {
      try {
        Socket incoming = sock.accept();
	
        RecvHandler handler = new RecvHandler(incoming, cache_dir, server);
        handler.start();
        convec.addElement(handler);
      } catch (Exception e) {
        System.out.println("Connection failed ...");
      }

      //clean up the vector if needed
      for (int i=0; i<convec.size(); i++)
        if (!((RecvHandler)(convec.elementAt(i))).isAlive())
          convec.removeElementAt(i);
    }

    for (int i=0; i<convec.size(); i++) {
      RecvHandler handler = (RecvHandler)convec.elementAt(i);
      try {
        handler.closeConnection();
      } catch (Exception e) {
        System.out.println("Channel closing exception...");
      }
    }
    convec.removeAllElements();

    try { sock.close(); } catch (Exception e) { }
  }

  /**
   * close the connection
   */
  public void closeConnection() {
    listening = false;

    // broadcast that server is now closing
  }

} // end of class
