package sv.server;

import java.io.*;
import java.net.*;

public class SV2Server {

  // cache directory
  File cache_dir = null;

  // receive channel
  RecvChannel receiver;

  // send channel
  SendChannel sender;

  /**
   * SV2 server
   * @param p port number
   * @param dir cache directory
   */
  public SV2Server(int p, String dir) {
    initChacheDir(dir);

    // setup socket for this server
    receiver = new RecvChannel(p, cache_dir, this);
    receiver.start();

    // setup asynchronous sender
    sender = new SendChannel(cache_dir);
  }

  /**
   * kill the server
   */
  public void killServer() {
    /////////////////////////////////////////////////////////////
    // IT SHOULD SEND THE SERVER KILL INFO TO EACH CLIENT...
    /////////////////////////////////////////////////////////////

    // clean up cache directory
    if (cache_dir != null && cache_dir.exists()) {
      String[] files = cache_dir.list();
      for (int i=0; i<files.length; i++) {
        File elem = new File(cache_dir, files[i]);
        elem.delete();
      }
      cache_dir.delete();
    }
    
    receiver.closeConnection();
    System.out.println("The SV2 server is killed.");
    System.exit(0);
  }

  /**
   * return cooked cache dierctory
   */
  public File getChacheDir() {
    return cache_dir;
  }

  /**
   * return sender
   */
  public SendChannel getSender() {
    return sender;
  }

  // initialize the cache directory
  private void initChacheDir(String dir) {
    cache_dir = new File(dir);

    if (cache_dir.exists()) {
      System.out.println("Cache directory is used by someone, choose the other!");
      System.exit(1);
    } else {
      if (!cache_dir.mkdir()) {
        System.out.println("Cache directory creation failed.");
        System.exit(1);
      }
    }

    if (!cache_dir.canRead() || !cache_dir.canWrite()) {
      System.out.println("Can't read or write the cache directory.");
      System.exit(1);
    }
  }


} // end of class
