package sv.util;

import java.util.*;
import java.io.*;
import sv.data.*;

public class FilterMenuHandler {

  static Vector menuVec;          // filter menu information vector
   
  String filter_menu = null;

  public FilterMenuHandler(String f) {
    menuVec = new Vector();
 
    if (f != null) {
      filter_menu = f;
      parseMenuInfo();
    } 
  }

  // return the 1d menu
  public Vector get1DMenus() {
    Vector v = new Vector();

    for (Enumeration e=menuVec.elements(); e.hasMoreElements();) {
      FilterMenuInfo info = (FilterMenuInfo)e.nextElement();
      if (info.rank == TimeData.ONED)
        v.addElement(info);
    }
    return v;
  }

  // return the 2d menu
  public Vector get2DMenus() {
    Vector v = new Vector();

    for (Enumeration e=menuVec.elements(); e.hasMoreElements();) {
      FilterMenuInfo info = (FilterMenuInfo)e.nextElement();
      if (info.rank == TimeData.TWOD)
        v.addElement(info);
    }
    return v;
  }

  // return the 3d menu
  public Vector get3DMenus() {
    Vector v = new Vector();

    for (Enumeration e=menuVec.elements(); e.hasMoreElements();) {
      FilterMenuInfo info = (FilterMenuInfo)e.nextElement();
      if (info.rank == TimeData.THREED)
        v.addElement(info);
    }
    return v;
  }


  // Each line should start with begin, <menu>, <item>, </item>, or end. 
  // It allow only 3 level total.
  private void parseMenuInfo() {
     
    BufferedReader in = null;

    String line, tmp, menuName, token;
    int level = 0, index = 0, root0 = 0, root1 = 0, root2 = 0;

    String mname = null, mclass = null, mpackage = null;
    int mkey = -1;

    try {
      in = new BufferedReader(new FileReader(filter_menu));
    } catch (Exception e) {
      System.out.println("Can't find menu information file.");
    }

    ////////////////////////////////////////////////
    // parse 1d menu information
    ////////////////////////////////////////////////
    try {
      while (true) {
        String readed = in.readLine().toLowerCase();
        if ((readed.indexOf("begin") >= 0) && (readed.indexOf("1d") >= 0))
           break;
      }
 
      while (true) {
        line = in.readLine().trim();
                    
        if (line.equalsIgnoreCase("end")) 
           break;
 
        tmp = line.toLowerCase();
        if (tmp.indexOf("<menu>") >= 0) {
           menuName = line.substring(6);
           menuName = menuName.trim();
           index++;

           if (level == 0) {
             root0 = index;
             menuVec.addElement(new 
                   FilterMenuInfo(menuName, level, index, -1, TimeData.ONED));
           } else if (level == 1) {
             root1 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root0, TimeData.ONED));
           } else {
             root2 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root1, TimeData.ONED));
           }
           level++;
        } else if (tmp.indexOf("</menu>") >= 0) {
           level--;
        } else if (tmp.indexOf("<item>") >= 0) {
           index++;
           menuName = line.substring(6);

           mkey = -1;
           StringTokenizer st = new StringTokenizer(menuName, ",");
           while(st.hasMoreTokens()) {
             tmp = st.nextToken().trim();
                                
             int pos = tmp.indexOf("=");
             if (tmp.indexOf("NAME") >= 0) {
                mname = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("CLASS") >= 0) {
                mclass = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("PACKAGE") >= 0) {
                mpackage = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("SHORTKEY") >= 0) {
                mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
             }
           }

           if (level == 0) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  -1, level, index, -1, TimeData.ONED));
           } else if (level == 1) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  mkey, level, index, root0, TimeData.ONED));
           } else if (level == 2) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root1, TimeData.ONED));
           } else {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root2, TimeData.ONED));
           }
	}
      }
     } catch (Exception e) {
        System.out.println("Menu information(1D) error.");
        e.printStackTrace();
     }


    ////////////////////////////////////////////////
    // parse 2d menu information
    ////////////////////////////////////////////////
    try {
      while (true) {
        String readed = in.readLine().toLowerCase();
        if ((readed.indexOf("begin") >= 0) && (readed.indexOf("2d") >= 0))
           break;
      }
 
      while (true) {
        line = in.readLine().trim();
                    
        if (line.equalsIgnoreCase("end")) 
           break;
 
        tmp = line.toLowerCase();
        if (tmp.indexOf("<menu>") >= 0) {
           menuName = line.substring(6);
           menuName = menuName.trim();
           index++;

           if (level == 0) {
             root0 = index;
             menuVec.addElement(new 
                   FilterMenuInfo(menuName, level, index, -1, TimeData.TWOD));
           } else if (level == 1) {
             root1 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root0, TimeData.TWOD));
           } else {
             root2 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root1, TimeData.TWOD));
           }
           level++;
        } else if (tmp.indexOf("</menu>") >= 0) {
           level--;
        } else if (tmp.indexOf("<item>") >= 0) {
           index++;
           menuName = line.substring(6);

           mkey = -1;
           StringTokenizer st = new StringTokenizer(menuName, ",");
           while(st.hasMoreTokens()) {
             tmp = st.nextToken().trim();
                                
             int pos = tmp.indexOf("=");
             if (tmp.indexOf("NAME") >= 0) {
                mname = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("CLASS") >= 0) {
                mclass = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("PACKAGE") >= 0) {
                mpackage = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("SHORTKEY") >= 0) {
                mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
             }
           }

           if (level == 0) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  -1, level, index, -1, TimeData.TWOD));
           } else if (level == 1) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  mkey, level, index, root0, TimeData.TWOD));
           } else if (level == 2) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root1, TimeData.TWOD));
           } else {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root2, TimeData.TWOD));
           }
	}
      }
     } catch (Exception e) {
        System.out.println("Menu information(2D) error.");
        e.printStackTrace();
     }

    ////////////////////////////////////////////////
    // parse 3d menu information
    ////////////////////////////////////////////////
    try {
      while (true) {
        String readed = in.readLine().toLowerCase();
        if ((readed.indexOf("begin") >= 0) && (readed.indexOf("3d") >= 0))
           break;
      }
 
      while (true) {
        line = in.readLine().trim();
                    
        if (line.equalsIgnoreCase("end")) 
           break;
 
        tmp = line.toLowerCase();
        if (tmp.indexOf("<menu>") >= 0) {
           menuName = line.substring(6);
           menuName = menuName.trim();
           index++;

           if (level == 0) {
             root0 = index;
             menuVec.addElement(new 
                   FilterMenuInfo(menuName, level, index, -1, TimeData.THREED));
           } else if (level == 1) {
             root1 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root0, TimeData.THREED));
           } else {
             root2 = index;
             menuVec.addElement(new FilterMenuInfo(menuName, level, index, root1, TimeData.THREED));
           }
           level++;
        } else if (tmp.indexOf("</menu>") >= 0) {
           level--;
        } else if (tmp.indexOf("<item>") >= 0) {
           index++;
           menuName = line.substring(6);

           mkey = -1;
           StringTokenizer st = new StringTokenizer(menuName, ",");
           while(st.hasMoreTokens()) {
             tmp = st.nextToken().trim();
                                
             int pos = tmp.indexOf("=");
             if (tmp.indexOf("NAME") >= 0) {
                mname = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("CLASS") >= 0) {
                mclass = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("PACKAGE") >= 0) {
                mpackage = tmp.substring(pos+2, tmp.length()-1);
             } else if (tmp.indexOf("SHORTKEY") >= 0) {
                mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
             }
           }

           if (level == 0) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  -1, level, index, -1, TimeData.THREED));
           } else if (level == 1) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage,
                                  mkey, level, index, root0, TimeData.THREED));
           } else if (level == 2) {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root1, TimeData.THREED));
           } else {
             menuVec.addElement(new FilterMenuInfo(mname, mclass, mpackage, 
                                  mkey, level, index, root2, TimeData.THREED));
           }
	}
      }
     } catch (Exception e) {
        System.out.println("Menu information(3D) error.");
        e.printStackTrace();
     }
  }

} // enf od class
