package sv.util;

import java.awt.*;

public class NumberFormat {

  // format mode
  public static final int ALGEBRAIC_FORMAT  = 0;
  public static final int SCIENTIFIC_FORMAT = 1;
   
  /** 
   * format float number
   * @param d formatted number
   * @param n  number of significant figures
   * @param p  precision of the number
   * @param sformat scientific format or not
   */
  public static final String formatFloat(float d, int n, int p, int sformat) {

    if (sformat == 1) {                 // scientific format
       float x = d;
       int left = n - p;
       float right = 0;
       int power;
       int exponent;
       int i;
       StringBuffer s = new StringBuffer(n+4);
       
       if(left < 0 ) {
          System.out.println("Warning: Significant < Precision");          
          String val = String.valueOf(d);
          return val.toLowerCase();
       }

       if(d < 0.0f) {
          x = -d;
          s.append("-");
       }

      if(d == 0.0f) exponent = 0;
      else         exponent = (int)(Math.floor(log10(x)));

      power = exponent - (left - 1);

      if( power < 0 ) {
         for(i=power; i<0; i++) { x *= 10.0f; }
      } else {
         for(i=0; i<power; i++) { x /= 10.0f; }
      }

      left = (int)x;
      s.append(left);

      if (p > 0) {
         s.append('.');
         right = x-left;
         for(i=0; i<p; i++) {
            right *= 10;
            if(i==p-1) right += 0.5;
            s.append((int)(right)); 
            right -= (int)right;
         }
       }

       if(power != 0) {
          s.append('e');
          if(power < 0) s.append('-');
          else          s.append('+');
          power = Math.abs(power);
          if(power > 9) {
             s.append(power);
          } else {
             s.append('0');
             s.append(power);
          }
       } else 
         s.append("e+00");


       return s.toString();


    } else {                       // algebraic format
       int ival = (int)d;

//System.out.println(xPrec + " " + yPrec + " " + timePrec + " " + p);

       if (p < 1) {
//           System.out.println("Warning: decimal < 0");
          return String.valueOf(ival);
       }

       float mult = 1.0f;
       for (int i=0; i<p; i++)
          mult *= 10.0f;
     
       int intd = (int)(d * mult);
       String val = new String(String.valueOf(intd));
       int len = val.length();

//System.out.println(d + " " + mult + " " + val + " " + len + " " + p);

       if (val.indexOf("e") > 0 || val.indexOf("E") > 0)
          return String.valueOf(ival);
     
       if (d > 0) {
     
          if (len == p) 
             val = "0." + val;
          else if (len < p) {
             int gap = p - len - 1;
             val = "0.0" + val;
             for (int i=0; i<gap; i++)
                val = val + "0";
          } else
             val = val.substring(0, len-p) + "." + val.substring(len-p);  

       } else if (d <= 0) {
          
          if (len == p) 
             val = "-0.0" + val.substring(1);
          else if (len < p) {
             int gap = p - len + 1;
             val = val + ".";
             for (int i=0; i<gap; i++)
                val = val + "0";
          } else if (len == p+1)
            val = val.substring(0, len-p) + "0." + val.substring(len-p); 
          else
             val = val.substring(0, len-p) + "." + val.substring(len-p);  
       }

       return val;
    } 
  }

  // return the log_10
  private static double log10(double x) throws ArithmeticException {
    if( x <= 0.0 ) throw new ArithmeticException("range exception");
    return Math.log(x)/2.30258509299404568401;
  }

} // end of class
