package sv.viewer.builtin;

import javax.media.j3d.*;
import javax.vecmath.*;
import sv.config.*;

public class G2DObject extends Object {

   private Shape3D shape3D;
   private Shape3D boundingBox;

   int size;
   Point3f[] verts;
  //   Vector3f[] normals;
   Color3f[] colors;


  //   public G2DObject(int option, int size, 
  //          Point3f[] verts, Vector3f[] normals, Color3f[] colors) {
    
   public G2DObject(int option, int size, 
                                    Point3f[] verts, Color3f[] colors) {
  
      this.size = size;
      this.verts = verts;
      //     this.normals = normals;
      this.colors = colors;
  
      changeOption(option);
   }

  /**
   * change display option 
   */    
   public void changeOption(int option) {
     
      Appearance app = new Appearance();

      if (option == Configs.WIREFRAME) {  // WIREFRAME
         QuadArray quadArray = new QuadArray(size, QuadArray.COORDINATES);
         quadArray.setCoordinates(0, verts);

         // Set up the coloring properties
         Color3f objColor = new Color3f(0.0f, 0.0f, 1.0f);
         ColoringAttributes ca = new ColoringAttributes();
         ca.setColor(objColor);
         app.setColoringAttributes(ca);

         // Set up the polygon attributes
         PolygonAttributes pa = new PolygonAttributes();
         pa.setPolygonMode(pa.POLYGON_LINE);
         pa.setCullFace(pa.CULL_NONE);
         app.setPolygonAttributes(pa);

         shape3D = new Shape3D(quadArray);
         shape3D.setAppearance(app);

      } else {
	/*
         QuadArray quadArray = new QuadArray(size, QuadArray.COORDINATES | 
                                    QuadArray.NORMALS | QuadArray.COLOR_3);
         
         quadArray.setCoordinates(0, verts);
         quadArray.setNormals(0, normals);
	 */

         QuadArray quadArray = new QuadArray(size, QuadArray.COORDINATES | 
                                                      QuadArray.COLOR_3);
         
         quadArray.setCoordinates(0, verts);
         quadArray.setColors(0, colors);
         shape3D = new Shape3D(quadArray);

         // Set up the polygon attributes
         PolygonAttributes pa = new PolygonAttributes();
         pa.setCullFace(pa.CULL_NONE);

         if (option == Configs.POINTPLOT)         // POINT PLOT
            pa.setPolygonMode(pa.POLYGON_POINT);
         else if (option == Configs.COLORWIREFRAME)    // COLORED WIREFRAME
             pa.setPolygonMode(pa.POLYGON_LINE);

         app.setPolygonAttributes(pa);
         shape3D.setAppearance(app);
      }
   }

   public void makeBoundingBox(int dimx, int dimy) {
        float xx = 1.0f;
        float yy = 1.0f;

        if (dimx < dimy) 
           xx = (float)dimx * 2.0f / (float)dimy - 1.0f;
        else if (dimx > dimy) 
           yy = (float)dimy * 2.0f / (float)dimx - 1.0f;

        float[] verts = {
       // front face
	    xx, -1.0f,  1.0f,
   	    xx,  yy,  1.0f,
	   -1.0f,  yy,  1.0f,
   	   -1.0f, -1.0f,  1.0f,
       // back face
	   -1.0f, -1.0f, -1.0f,
  	   -1.0f,  yy, -1.0f,
	    xx,  yy, -1.0f,
	    xx, -1.0f, -1.0f,
       // right face
	    xx, -1.0f, -1.0f,
	    xx,  yy, -1.0f,
	    xx,  yy,  1.0f,
	    xx, -1.0f,  1.0f,
       // left face
	   -1.0f, -1.0f,  1.0f,
	   -1.0f,  yy,  1.0f,
	   -1.0f,  yy, -1.0f,
	   -1.0f, -1.0f, -1.0f,
       // top face
	    xx,  yy,  1.0f,
	    xx,  yy, -1.0f,
	   -1.0f,  yy, -1.0f,
	   -1.0f,  yy,  1.0f,
       // bottom face
	   -1.0f, -1.0f,  1.0f,
	   -1.0f, -1.0f, -1.0f,
	    xx, -1.0f, -1.0f,
	    xx, -1.0f,  1.0f,
       };

       Vector3f[] normals = {
           new Vector3f( 0.0f,  0.0f,  1.0f),	// front face
           new Vector3f( 0.0f,  0.0f, -1.0f),	// back face
           new Vector3f( 1.0f,  0.0f,  0.0f),	// right face
           new Vector3f(-1.0f,  0.0f,  0.0f),	// left face
           new Vector3f( 0.0f,  1.0f,  0.0f),	// top face
           new Vector3f( 0.0f, -1.0f,  0.0f),	// bottom face
       };

       Appearance app = new Appearance();
       
       // Set up the coloring properties
       Color3f objColor = new Color3f(1.0f, 0.4f, 0.0f);
       ColoringAttributes ca = new ColoringAttributes();
       ca.setColor(objColor);
       app.setColoringAttributes(ca);

       // Set up the polygon attributes
       PolygonAttributes pa = new PolygonAttributes();
       pa.setPolygonMode(pa.POLYGON_LINE);
       pa.setCullFace(pa.CULL_NONE);
       app.setPolygonAttributes(pa);

       QuadArray quadArray = new QuadArray(24, QuadArray.COORDINATES | QuadArray.NORMALS);

       quadArray.setCoordinates(0, verts);
       quadArray.setNormals(0, normals);
       boundingBox = new Shape3D(quadArray);

       boundingBox.setAppearance(app);
   }


   /**
    * return data set object
    */
   public Shape3D getChild() {
      return shape3D;
   }

   /**
    * return bounding box
    */
   public Shape3D getBoundChild() {
      return boundingBox;
   }

} // end of class
