package sv.viewer.builtin;

import javax.media.j3d.*;
import javax.vecmath.*;
import sv.config.*;

public class G3DObject extends Object {

  private Shape3D shape3D;
  private Shape3D boundingBox;

  int size;
  Point3f[] verts;
  Vector3f[] normals;
  //   Color3f[] colors;


  boolean wiremode  = true;

  public G3DObject(int option, int size, Point3f[] verts, Vector3f[] normals) {

      this.size = size;
      this.verts = verts;
      this.normals = normals;
   
      changeOption(option, null);
  }

  /**
   * change display option 
   */    
   public void changeOption(int option, Appearance app) {

      if (option == Configs.WIREFRAME) {  // WIREFRAME
         wiremode = true;
         
         TriangleArray triArray = new TriangleArray(size, 
                            TriangleArray.COORDINATES);
         triArray.setCoordinates(0, verts);
	 //         triArray.setNormals(0, normals);

         // overwrite appearance
         app = new Appearance();

         // Set up the coloring properties
         Color3f objColor = new Color3f(0.0f, 0.0f, 1.0f);
         ColoringAttributes ca = new ColoringAttributes();
         ca.setColor(objColor);
         app.setColoringAttributes(ca);

         // Set up the polygon attributes
         PolygonAttributes pa = new PolygonAttributes();
         pa.setPolygonMode(pa.POLYGON_LINE);
         pa.setCullFace(pa.CULL_NONE);
         app.setPolygonAttributes(pa);

         shape3D = new Shape3D(triArray);

      } else {

         wiremode = false;

         TriangleArray triArray = new TriangleArray(size, 
                         TriangleArray.COORDINATES | TriangleArray.NORMALS);
         triArray.setCoordinates(0, verts);
         triArray.setNormals(0, normals);

	 /*
         // Set up the transparency attributes
         TransparencyAttributes ta = new TransparencyAttributes();
         ta.setTransparencyMode(TransparencyAttributes.BLENDED);
         ta.setTransparency(0.6f);
System.out.println("transparency: " + ta.getTransparency());
System.out.println("transparency mode: " + ta.getTransparencyMode());
         app.setTransparencyAttributes(ta);

         PolygonAttributes pa = new PolygonAttributes();
         pa.setCullFace(pa.CULL_NONE);
System.out.println("cull face mode: " + pa.getCullFace());
         app.setPolygonAttributes(pa);

         // Set up the material properties
         Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
         Color3f red   = new Color3f(0.80f, 0.20f, 0.2f);
         Color3f ambientRed = new Color3f(0.2f, 0.05f, 0.0f);
         Color3f specular = new Color3f(0.7f, 0.7f, 0.7f);

         Material redMaterial = 
              new Material(ambientRed, black, red, specular, 75.0f);
         redMaterial.setLightingEnable(true);
         app.setMaterial(redMaterial);   
	 */

         shape3D = new Shape3D(triArray);
      }

      shape3D.setAppearance(app);
   }


   public void makeBoundingBox() {

        float[] verts = {
       // front face
	    1.0f, -1.0f,  1.0f,
   	    1.0f,  1.0f,  1.0f,
	   -1.0f,  1.0f,  1.0f,
   	   -1.0f, -1.0f,  1.0f,
       // back face
	   -1.0f, -1.0f, -1.0f,
  	   -1.0f,  1.0f, -1.0f,
	    1.0f,  1.0f, -1.0f,
	    1.0f, -1.0f, -1.0f,
       // right face
	    1.0f, -1.0f, -1.0f,
	    1.0f,  1.0f, -1.0f,
	    1.0f,  1.0f,  1.0f,
	    1.0f, -1.0f,  1.0f,
       // left face
	   -1.0f, -1.0f,  1.0f,
	   -1.0f,  1.0f,  1.0f,
	   -1.0f,  1.0f, -1.0f,
	   -1.0f, -1.0f, -1.0f,
       // top face
	    1.0f,  1.0f,  1.0f,
	    1.0f,  1.0f, -1.0f,
	   -1.0f,  1.0f, -1.0f,
	   -1.0f,  1.0f,  1.0f,
       // bottom face
	   -1.0f, -1.0f,  1.0f,
	   -1.0f, -1.0f, -1.0f,
	    1.0f, -1.0f, -1.0f,
	    1.0f, -1.0f,  1.0f,
       };

       Vector3f[] normals = {
           new Vector3f( 0.0f,  0.0f,  1.0f),	// front face
           new Vector3f( 0.0f,  0.0f, -1.0f),	// back face
           new Vector3f( 1.0f,  0.0f,  0.0f),	// right face
           new Vector3f(-1.0f,  0.0f,  0.0f),	// left face
           new Vector3f( 0.0f,  1.0f,  0.0f),	// top face
           new Vector3f( 0.0f, -1.0f,  0.0f),	// bottom face
       };

       Appearance app = new Appearance();
       
       // Set up the coloring properties
       Color3f objColor = new Color3f(1.0f, 0.4f, 0.0f);
       ColoringAttributes ca = new ColoringAttributes();
       ca.setColor(objColor);
       app.setColoringAttributes(ca);

       // Set up the polygon attributes
       PolygonAttributes pa = new PolygonAttributes();
       pa.setPolygonMode(pa.POLYGON_LINE);
       pa.setCullFace(pa.CULL_NONE);
       app.setPolygonAttributes(pa);

       QuadArray quadArray = new QuadArray(24, QuadArray.COORDINATES | QuadArray.NORMALS);

       quadArray.setCoordinates(0, verts);
       quadArray.setNormals(0, normals);
       boundingBox = new Shape3D(quadArray);

       boundingBox.setAppearance(app);
   }


   /**
    * return data set object
    */
  public Shape3D getChild() {
     return shape3D;
  }

   /**
    * return bounding box
    */
   public Shape3D getBoundChild() {
      return boundingBox;
   }


} // end of class
