package sv.viewer.builtin;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.AffineTransform;
import java.util.Vector;
import sv.viewer.*;
import sv.data.*;
import sv.config.*;
import sv.gui.*;

public class GCCon extends GCanvas1D 
                              implements MouseListener, MouseMotionListener {

  // for connection list only
  private Image renderedImg;

  public GCCon(GFCon gframe, Config1D cfg) {
    super(gframe, cfg);
  }

  /**
   * destroy rendered image for connection list
   */
  public void destroyImage() {
     renderedImg = null;
  }

  /**
   * regenerate the rendered image for connection list
   */
  public void recalcImage(DataSet data, int j) {
     int pixels[] = new int[320 * 320];

     float cArr[][] = ((TDCon)(data.getTimeData(j))).getColors();
     float cmin = ((TDCon)(data.getTimeData(j))).getcmin();
     float cmax = ((TDCon)(data.getTimeData(j))).getcmax();

     int bgc = config.getBGColor().getRGB();
     Color c;

     int index = 0;
     for (int y = 0; y < 320; y++) {
       for (int x = 0; x < 320; x++) {

         if (cArr[x][y] >= cmin) {
             int val = (int)((cArr[x][y] - cmin) / (cmax-cmin) * 100f);

             if (gframe.colorModel == 1) 
                  c = new Color((int)(gframe.rcolors[val] * 255f), 
                                (int)(gframe.gcolors[val] * 255f), 
                                (int)(gframe.bcolors[val] * 255f));
             else 
                  c = Color.getHSBColor((float)gframe.hcolors[val], 
                                        (float)gframe.scolors[val],
                                        (float)gframe.lcolors[val]);
             pixels[index++] = c.getRGB();
         } else 
             pixels[index++] = bgc;

       }
     }

     renderedImg = createImage(new MemoryImageSource(320, 320, 
                                           ColorModel.getRGBdefault(),
                                           pixels, 0, 320));  
     repaint();
  }
   
  // draw point data
  protected void drawData() {
    int totPoints = 0;
    int f = (int)(2.0f * linewidth);
    int f2 = 2 * f;

    // line width
    pdaG2.setStroke(new BasicStroke(linewidth));

    // erase background
    pdaG2.setColor(config.getBGColor());
    pdaG2.fillRect(0, 0, imgSize.width, imgSize.height);

    float xrange = xmax - xmin;
    float yrange = ymax - ymin;
    if (xrange == 0.0f) xrange = 1.0f;
    if (yrange == 0.0f) yrange = 1.0f;
    
    float xscale = (float)imgSize.width / xrange;
    float yscale = (float)imgSize.height / yrange;

    int xpadout = (int)(xmin * xscale);
    int ypadout = (int)(ymin * yscale);

    Vector polyvec = new Vector();
    int[] xpts, ypts;
    int index = 0;
    
    DataSet[] data = brief.getDataSet();

    for (int i=0; i<data.length; i++) {
      pdaG2.setColor(data[i].getLineColor());
      int sz = data[i].getNoOfTimes();
      for (int j=0; j<sz; j++) {
        if (data[i].getTimeValue(j).getAbsTime() == gframe.getCurrent()) {
          TimeData1D td = (TimeData1D)data[i].getTimeData(j);
        
          int points = td.getNoOfPoints();
          if (totPoints < points)
             totPoints = points;

          float xArr[] = td.getxArr();
          float yArr[] = td.getyArr();

          if (((Config1D)config).getColorMode() == Config1D.COLOR_BY_INFO) {
             ////////////////////////////////////////////////
             // Gouraud shading
             ////////////////////////////////////////////////
             if (renderedImg != null)
               pdaG2.drawImage(renderedImg, 0, 0, dataArea.width, dataArea.height, this);
             else {
              pdaG2.setColor(Color.red);
              FontMetrics fm = pdaG2.getFontMetrics();
              String str = "Calculating colors ...";
              int strx = (dataArea.width-fm.stringWidth(str)) / 2;
              int stry = (dataArea.height-fm.getHeight()+1) / 2 + fm.getAscent(); 
              pdaG2.drawString(str, strx, stry);

              ((TDCon)td).calcColors();
              recalcImage(data[i], gframe.getCurrentIndex());
             }

           } else {
             int xx[] = new int[xArr.length];
             int yy[] = new int[yArr.length];
             int connection[] = ((TDCon)td).getConnectionList();

             //////////////////////////////////////////////////
             //  calculate position of the data point to be displayed
             //////////////////////////////////////////////////
             for (int k=0; k<xArr.length; k++) {
              xx[k] = (int)(xArr[k] * xscale) - xpadout;
              yy[k] = imgSize.height - (int)(yArr[k] * yscale) + ypadout;
             }
             //////////////////////////////////////////////////
             // Make Polygon
             //////////////////////////////////////////////////
             xpts = new int[connection.length];
             ypts = new int[connection.length];
             index = 0;
             for (int k=0; k<connection.length; k++) {
               if (connection[k] < 0) {
                  polyvec.addElement(new Polygon(xpts, ypts, index));
                  index = 0;
               } else {
                  xpts[index] = xx[connection[k]-1];
                  ypts[index] = yy[connection[k]-1];
                  index++; 
               }
             }

             Polygon polys[] = new Polygon[polyvec.size()];
             polyvec.copyInto(polys);
             polyvec.removeAllElements();
             polyvec = null;

             pdaG2.setColor(Color.blue);
             for (int idx=0; idx<polys.length; idx++)
               pdaG2.drawPolygon(polys[idx]);

             xpts = ypts = null;
             polys = null;
	   }
	}
      }
    }

    gframe.message((gframe. getCurrentIndex()+1) + " / " + 
      gframe.getDataSet().getNoOfTime() + "   ( " + totPoints + "  points)", false);
  }

} // end of class


