package sv.viewer.builtin;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.AffineTransform;
import java.util.Vector;
import sv.viewer.*;
import sv.data.*;
import sv.config.*;
import sv.gui.*;

public class GCPoint extends GCanvas1D {

  public GCPoint(GFPoint gframe, Config1D cfg) {
    super(gframe, cfg);
  }
   
  // draw point data
  protected void drawData() {
    int totPoints = 0;
    int f = (int)(2.0f * linewidth);
    int f2 = 2 * f;

    // line width
    pdaG2.setStroke(new BasicStroke(linewidth));

    // erase background
    pdaG2.setColor(config.getBGColor());
    pdaG2.fillRect(0, 0, imgSize.width, imgSize.height);

    float xrange = xmax - xmin;
    float yrange = ymax - ymin;
    if (xrange == 0.0f) xrange = 1.0f;
    if (yrange == 0.0f) yrange = 1.0f;
    
    float xscale = (float)imgSize.width / xrange;
    float yscale = (float)imgSize.height / yrange;

    int xpadout = (int)(xmin * xscale);
    int ypadout = (int)(ymin * yscale);
    
    DataSet[] data = brief.getDataSet();

    for (int i=0; i<data.length; i++) {
      pdaG2.setColor(data[i].getLineColor());
      int sz = data[i].getNoOfTimes();
      for (int j=0; j<sz; j++) {
        if (data[i].getTimeValue(j).getAbsTime() == gframe.getCurrent()) {
          TimeData1D td = (TimeData1D)data[i].getTimeData(j);
        
          int points = td.getNoOfPoints();
          if (totPoints < points)
             totPoints = points;

          float xArr[] = td.getxArr();
          float yArr[] = td.getyArr();
          int xx[] = new int[xArr.length];
          int yy[] = new int[yArr.length];

          // calculate the position of the data point to be displayed
          for (int k=0; k<xArr.length; k++) {
	    xx[k] = (int)(xArr[k] * xscale) - xpadout;
	    yy[k] = imgSize.height - (int)(yArr[k] * yscale) + ypadout;
          }

          for (int k=0; k<xArr.length; k++) {
            switch (data[i].getMarkerType()) {
              case MarkerChooser.CIRCLEMARKER:
                pdaG2.drawOval(xx[k]-f, yy[k]-f, f2, f2);
                break;

              case MarkerChooser.SQUAREMARKER:
                pdaG2.drawRect(xx[k]-f, yy[k]-f, f2, f2);
                break;

              case MarkerChooser.DIAMONDMARKER:
                pdaG2.drawLine(xx[k], yy[k]-f, xx[k]-f, yy[k]);
                pdaG2.drawLine(xx[k]-f, yy[k], xx[k], yy[k]+f);
                pdaG2.drawLine(xx[k], yy[k]+f, xx[k]+f, yy[k]);
                pdaG2.drawLine(xx[k]+f, yy[k], xx[k], yy[k]-f);
                break;

              case MarkerChooser.CIRCLEFILLMARKER:
                pdaG2.fillOval(xx[k]-f, yy[k]-f, f2, f2);
                break;

              case MarkerChooser.SQUAREFILLMARKER:
                pdaG2.fillRect(xx[k]-f, yy[k]-f, f2, f2);
                break;

              case MarkerChooser.DIAMONDFILLMARKER:
	          int[] xpts = { xx[k], xx[k]-f, xx[k], xx[k]+f };
                int[] ypts = { yy[k]-f, yy[k], yy[k]+f, yy[k] };
                pdaG2.fillPolygon(xpts, ypts, 4);
                break;

              case MarkerChooser.CROSSMARKER:
                pdaG2.drawLine(xx[k], yy[k]-f, xx[k], yy[k]+f);
                pdaG2.drawLine(xx[k]-f, yy[k], xx[k]+f, yy[k]);
                break;

              case MarkerChooser.XMARKER:
                pdaG2.drawLine(xx[k]-f, yy[k]-f, xx[k]+f, yy[k]+f);
                pdaG2.drawLine(xx[k]+f, yy[k]-f, xx[k]-f, yy[k]+f);
                break;
	   }

           if (drawClosest && xArr[k] == closest[0] 
                                         && yArr[k] == closest[1]) {
             Color color = pdaG2.getColor();
             pdaG2.setColor(Color.black);
             pdaG2.fillOval(xx[k]-f, yy[k]-f, f2, f2);
             displayStr = displayStr + "), "+(k+1) + "th: (" + 
                closest[0] + ", " + closest[1] + ")";
             gframe.message(displayStr, false);
             pdaG2.setColor(color);
           }

           if (config.isConnectPoints() && k != xArr.length-1) 
             pdaG2.drawLine(xx[k], yy[k], xx[k+1], yy[k+1]);
	  }
	}
      }
    }

    gframe.message((gframe. getCurrentIndex()+1) + " / " + 
      gframe.getDataSet().getNoOfTime() + "   ( " + totPoints + "  points)", false);
  }

} // end of class


