package sv.viewer.builtin;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import sv.viewer.*;

public class GCanvasVol extends Canvas 
                              implements MouseListener, MouseMotionListener {

  // fried class of this 
  private GFrameVol gframe;

  // volume image
  private Image image;

  // MediaTracker
  private MediaTracker tracker;

  // printing flag
  private boolean printing = false;   

  // display area size
  private Dimension canvasSize;

  // mouse related work
  public static final int ROTATE_MODE   = 0;
  public static final int PANNING_MODE  = 1;
  public static final int ZOOMING_MODE  = 2;

  // mouse mode 
  private int mouseMode = 0;
  private boolean dragging = false;

  // mouse click point
  private int clickX = -1, clickY = -1;

  // buffered images
  private Image offImg;
  private Graphics offgc;

  public GCanvasVol(GFrameVol gframe, Image img) {
    this.gframe = gframe;
    image = img;

    canvasSize = new Dimension(-1, -1);
    this.setSize(256, 256);

    tracker = new MediaTracker(this);
    tracker.addImage(image, 0);

    addMouseListener(this);
    addMouseMotionListener(this);
  }

  /**
   * change the volume image
   */
  public void changeImage(Image img) {
    image = img;
    tracker.addImage(image, 0);
    repaint();
  }

  public Dimension getPreferredSize() {
    return getMinimumSize();
  }

  public Dimension getMinimumSize() {
    return new Dimension(256, 256);
  }

  /**
   * overrided paint method
   */
  public void paint(Graphics g) {
    Dimension d = getSize();

    if (offImg == null || 
          d.width != canvasSize.width || d.height != canvasSize.width) {
      offImg = createImage(d.width, d.height);
      offgc = offImg.getGraphics();
      canvasSize = d;
    }

    printing = g instanceof PrintGraphics;

    if (printing) {
      return;
    }
    
    offgc.drawImage(image, 0, 0, d.width, d.height, this);

    if (offImg != null)
      g.drawImage(offImg, 0, 0, this);
  }

  /**
   * overrided update method
   */
  public void update(Graphics g) {
    paint(g);
  }

  /**
   * Invoked when the mouse has been clicked on a component.
   */
  public void mouseClicked(MouseEvent e) {
  }

  /**
   * Invoked when a mouse button has been pressed on a component.
   */
  public void mousePressed(MouseEvent e) {
    clickX = e.getX();
    clickY = e.getY();
  }

  /**
   * Invoked when a mouse button has been released on a component.
   */
  public void mouseReleased(MouseEvent e) {
    int x = e.getX();
    int y = e.getY();

    int dx = clickX - x;
    int dy = clickY - y;

    if (Math.abs(dx) > Math.abs(dy)) {
      if (dx > 0) gframe.RotateY(true);
      else        gframe.RotateY(false);
    } else
      if (dy > 0) gframe.RotateX(true);
      else        gframe.RotateX(false);

    dragging = false;
  }

  /**
   * Invoked when the mouse enters a component.
   */
  public void mouseEntered(MouseEvent e) {
  }

  /**
   * Invoked when the mouse exits a component.
   */
  public void mouseExited(MouseEvent e) {
  }

  /** 
   * Invoked when a mouse button is pressed on a component and then dragged.
   * Mouse drag events will continue to be delivered to the component 
   * where the first originated until the mouse button is released 
   * (regardless of whether the mouse position is within the bounds of the 
   * component).
   */
  public void mouseDragged(MouseEvent e) {
    dragging = true;
  }

  /**
   * Invoked when the mouse button has been moved on a component (with no
   *  buttons no down).
   */
  public void mouseMoved(MouseEvent e) {
  }

} // end of class

