package sv.viewer.builtin;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import sv.engine.*;
import sv.viewer.*;
import sv.config.*;
import sv.data.*;
import sv.gui.*;
import sv.util.*;

/** 
 * Define a frame for 1d data
 */
public class GFCon extends GFrame1D {

  /**
   * constructor
   * @param brief initial data set
   * @param cfg configuration of this window
   */
  public GFCon(DataSetBrief brief, Configs cfg) {
    super(brief, cfg);
  
    // create menu 
    makeMenu();
  
    // layout the components
    setLayout(new BorderLayout());

    add("Center", canvas = new GCCon(this, (Config1D)cfg));
    add("South", status = new StatusBar());
    status.display(brief.getDataName() + " created.", false);
  } 

  /**
   * do action event
   */
  public void doActionEventWork(byte[] datum) {
    super.doActionEventWork(datum);

    StringTokenizer st = new StringTokenizer(new String(datum), ";");

    // The first token is a command
    int comm = Integer.parseInt(st.nextToken());
    st.nextToken();   // trash it

    DataSet[] data;
    boolean flag;
    int sz, r, g, b;
    
    switch (comm) {
      case SV2Action.COLORMODE_CHANGE:
        sz = Integer.parseInt(st.nextToken());
        ((Config1D)config).setColorMode(sz);
        updateDataSet();
        break;
      case SV2Action.COLORMAP_CHANGE:
        setColorMap(st);
        updateDataSet();
        break;
    }
  }

  /**
   * update data set
   */
  public void updateDataSet() {

    DataSet[] data = dataSet.getDataSet();
    for (int i=0; i<data.length; i++)
      for (int j=0; j<data[i].getNoOfTimes(); j++)
        ((TDCon)(data[i].getTimeData(j))).calcColors();
       
    ((GCCon)canvas).destroyImage();
    canvas.repaint();
  }

  /**
   * next
   */
  public void doNext(boolean b) {

    currentTime++;
    if (currentTime == timeValues.length)
      currentTime = 0;
    if (((Config1D)config).getColorMode() == Config1D.COLOR_BY_INFO)
       ((GCCon)canvas).destroyImage();
    if (b) 
      canvas.repaint();
  }

  /**
   * previous 
   */
  public void doPrevious() {

    currentTime--;
    if (currentTime == -1)
       currentTime = timeValues.length-1;
    if (((Config1D)config).getColorMode() == Config1D.COLOR_BY_INFO)
       ((GCCon)canvas).destroyImage();
    canvas.repaint();

  }


  // create the menu
  private void makeMenu() {
    MenuBar menubar = new MenuBar();
    MenuItem mi;

    // File manage
    Menu fileMenu = new Menu("File");

    mi = (MenuItem) fileMenu.add(new MenuItem("Save as..."));
    mi.addActionListener(saveasListener);

    mi = (MenuItem) fileMenu.add(new MenuItem("Print..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        PrintJob printjob =  getToolkit().getPrintJob(GFCon.this,
                                    "sv2 1D Plot", new Properties());
                                                        
        if (printjob != null) {          
           Graphics pg = printjob.getGraphics();
           if (pg != null) {
             canvas.printAll(pg);
             pg.dispose();         // flush page
           }
           printjob.end();
        }
      }
    });

    mi = (MenuItem) fileMenu.add(new MenuItem("GIF Images..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("GIF Images...");
      }
    });

    fileMenu.addSeparator();

    splitMI = (MenuItem) fileMenu.add(new MenuItem("Split Data Set"));
    splitMI.setEnabled(false);
    splitMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Split Data Set");
      }
    });

    fileMenu.addSeparator();

    mi = (MenuItem) fileMenu.add(new MenuItem("Close"));
    mi.addActionListener(closeListener);

    menubar.add(fileMenu);


    // Edit manage
    Menu editMenu = new Menu("Edit");

    mi = (MenuItem) editMenu.add(new MenuItem("Edit Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Edit Mode");
      }
    });

    mi = (MenuItem) editMenu.add(new MenuItem("View in Editor"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("View in Editor");
      }
    });

    menubar.add(editMenu);


    // View manage
    Menu viewMenu = new Menu("View");

    mi = (MenuItem) viewMenu.add(new MenuItem("Pan Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.PANNING_MODE);
        message("Drag the mouse to pan.", false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Zoom Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.ZOOMING_MODE);
        message("Click a mouse to zoom, click with control-key to unzoom.", false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Reset"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        message("Reset.", false);
      }
    });

    viewMenu.addSeparator();

    mi = (MenuItem) viewMenu.add(new MenuItem("Increase Font"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeFont(true);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Decrease Font"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeFont(false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Increase Line Thickness"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeLineThick(true);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Decrease Line Thickness"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeLineThick(false);
      }
    });

    viewMenu.addSeparator();

    Menu colorMenu = new Menu("Color");
    mi = (MenuItem) colorMenu.add(new MenuItem("By Color Info"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.COLORMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Config1D.COLOR_BY_INFO);

        catchActionEvent(comm);
      }
    });

    byDataSetMI = (MenuItem) colorMenu.add(new MenuItem("By Data Set"));
    byDataSetMI.setEnabled(false);
    byDataSetMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.COLORMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Config1D.COLOR_BY_DATA);

        catchActionEvent(comm);
      }
    });

   final Color bgColor = config.getBGColor();
   final Color fgColor = config.getFGColor();

    mi = (MenuItem) colorMenu.add(new MenuItem("Background Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFCon.this, 
                     "Background Color", bgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.BACKGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);
      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("Foreground Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFCon.this, 
                     "Foreground Color", fgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.FOREGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);
      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("Colormap Edit..."));
    mi.addActionListener(new ActionListener() {
     public void actionPerformed(ActionEvent e) {
       ColorMapDialog dialog = new ColorMapDialog(GFCon.this);
       dialog.show();
     }
    });

    viewMenu.add(colorMenu);

    Menu labelMenu = new Menu("Labels");

    displayLabelMI = new CheckboxMenuItem("Display Labels");
    displayLabelMI.setState(config.isLabelDraw());
    displayLabelMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_LABEL) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayLabelMI.getState());
        catchActionEvent(comm);        
      }
    });
    labelMenu.add(displayLabelMI);
 
    final String xlabel = ((Config1D)config).getXLabel();
    final String ylabel = ((Config1D)config).getYLabel();

    changeLabelMI = (MenuItem) labelMenu.add(new MenuItem("Change Labels..."));
    changeLabelMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String[] str = { xlabel, ylabel };

        LabelInputDialog chooser = new LabelInputDialog(GFCon.this, str);
        chooser.show();
        String args[] = chooser.getArgs();
        if (args != null) {
          String comm = String.valueOf(SV2Action.LABEL_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(args.length);
          for (int i=0; i<args.length; i++)
            comm = comm + ";" + args[i];
          catchActionEvent(comm);
        }
      }
    });

    moveLabelMI = (MenuItem) labelMenu.add(new MenuItem("Move Labels"));
    moveLabelMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.LABEL_MOVE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.add(labelMenu);

    Menu legendMenu = new Menu("Legend");
    
    displayLegendMI = new CheckboxMenuItem("Display Legends");
    displayLegendMI.setState(config.isLegendDraw());
    displayLegendMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_LEGEND) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayLegendMI.getState());
        catchActionEvent(comm);   
      }
    });
    legendMenu.add(displayLegendMI);

    changeLegendMI = (MenuItem) legendMenu.add(new MenuItem("Change Legend..."));
    changeLegendMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        DataSetBrief brief = getDataSet();
        DataSet[] data = brief.getDataSet();

        String names[] = new String[data.length];
        Color colors[] = new Color[data.length];
        int markers[] = new int[data.length];

        for (int i=0; i<data.length; i++) {
           names[i] = data[i].getDataName();
           colors[i] = data[i].getLineColor();
           markers[i] = data[i].getMarkerType();
        }
        LegendChange chooser = 
               new LegendChange(GFCon.this, names, colors, markers);
        chooser.show();
      }
    });

    moveLegendMI = (MenuItem) legendMenu.add(new MenuItem("Move Legend"));
    moveLegendMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.LEGEND_MOVE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.add(legendMenu);

    Menu tickMenu = new Menu("Tick Marks");
    drawTickMI = new CheckboxMenuItem("Draw Ticks");
    drawTickMI.setState(config.isTickMarkDraw());
    drawTickMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.TICKMARKS) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(drawTickMI.getState());
        catchActionEvent(comm);        
      }
    });
    tickMenu.add(drawTickMI);

    changeTickMI = (MenuItem) tickMenu.add(new MenuItem("Change Ticks..."));
    changeTickMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Change Ticks...");
      }
    });

    viewMenu.add(tickMenu);

    mi = (MenuItem) viewMenu.add(new MenuItem("Resize Plot Area"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.DATA_AREA_RESIZE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.addSeparator();

    connectMI =  new CheckboxMenuItem("Connect point");
    connectMI.setState(((Config1D)config).isConnectPoints());
    connectMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.CONNECT_POINT) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(connectMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(connectMI);

    displayTimeMI = new CheckboxMenuItem("Display Time");
    displayTimeMI.setState(((Config1D)config).isDisplayTimeMode());
    displayTimeMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_TIME) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayTimeMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(displayTimeMI);

    globalMinMaxMI = new CheckboxMenuItem("Use Global Minmax");
    globalMinMaxMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.GLOBALMINMAX) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(globalMinMaxMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(globalMinMaxMI);

    menubar.add(viewMenu);

     // Time steps
    Menu timeMenu = new Menu("Time");

    animateMI = (MenuItem) timeMenu.add(new MenuItem("Animate"));
    animateMI.addActionListener(animateListener);

    nextMI = (MenuItem) timeMenu.add(new MenuItem("Next"));
    nextMI.addActionListener(nextListener);

    prevMI = (MenuItem) timeMenu.add(new MenuItem("Previous"));
    prevMI.addActionListener(previousListener);

    stopMI = (MenuItem) timeMenu.add(new MenuItem("Stop"));
    stopMI.addActionListener(stopListener);

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Goto..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Goto...");
      }
    });

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Animation Speed..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Animation Speed");
      }
    });

    mi = (MenuItem) timeMenu.add(new MenuItem("Buffer Size..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Buffer Size");
      }
    });

    menubar.add(timeMenu);

    setMenuBar(menubar);
  } 

}  // end of class

