package sv.viewer.builtin;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import sv.engine.*;
import sv.viewer.*;
import sv.config.*;
import sv.data.*;
import sv.gui.*;
import sv.util.*;

/** 
 * Define a frame for 1d data
 */
public class GFPoint extends GFrame1D {

  /**
   * constructor
   * @param brief initial data set
   * @param cfg configuration of this window
   */
  public GFPoint(DataSetBrief brief, Configs cfg) {
    super(brief, cfg);
  
    // create menu 
    makeMenu();
  
    // layout the components
    setLayout(new BorderLayout());

    add("Center", canvas = new GCPoint(this, (Config1D)cfg));
    add("South", status = new StatusBar());
    status.display(brief.getDataName() + " created.", false);
  } 

  /**
   * update data set
   */
  public void updateDataSet() {
  }


  // create the menu
  private void makeMenu() {
    MenuBar menubar = new MenuBar();
    MenuItem mi;

    // File manage
    Menu fileMenu = new Menu("File");

    mi = (MenuItem) fileMenu.add(new MenuItem("Save as..."));
    mi.addActionListener(saveasListener);

    mi = (MenuItem) fileMenu.add(new MenuItem("Print..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        PrintJob printjob =  getToolkit().getPrintJob(GFPoint.this,
                                    "sv2 1D Plot", new Properties());
                                                        
        if (printjob != null) {          
           Graphics pg = printjob.getGraphics();
           if (pg != null) {
             canvas.printAll(pg);
             pg.dispose();         // flush page
           }
           printjob.end();
        }
      }
    });

    mi = (MenuItem) fileMenu.add(new MenuItem("GIF Images..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("GIF Images...");
      }
    });

    fileMenu.addSeparator();

    splitMI = (MenuItem) fileMenu.add(new MenuItem("Split Data Set"));
    splitMI.setEnabled(false);
    splitMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Split Data Set");
      }
    });

    fileMenu.addSeparator();

    mi = (MenuItem) fileMenu.add(new MenuItem("Close"));
    mi.addActionListener(closeListener);

    menubar.add(fileMenu);


    // Edit manage
    Menu editMenu = new Menu("Edit");

    mi = (MenuItem) editMenu.add(new MenuItem("Edit Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Edit Mode");
      }
    });

    mi = (MenuItem) editMenu.add(new MenuItem("View in Editor"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("View in Editor");
      }
    });

    menubar.add(editMenu);


    // View manage
    Menu viewMenu = new Menu("View");

    mi = (MenuItem) viewMenu.add(new MenuItem("Pan Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.PANNING_MODE);
        message("Drag the mouse to pan.", false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Zoom Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.ZOOMING_MODE);
        message("Click a mouse to zoom, click with control-key to unzoom.", false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Reset"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        message("Reset.", false);
      }
    });

    viewMenu.addSeparator();

    mi = (MenuItem) viewMenu.add(new MenuItem("Increase Font"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeFont(true);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Decrease Font"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeFont(false);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Increase Line Thickness"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeLineThick(true);
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Decrease Line Thickness"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.changeLineThick(false);
      }
    });

    viewMenu.addSeparator();

    Menu colorMenu = new Menu("Color");
    mi = (MenuItem) colorMenu.add(new MenuItem("By AMR"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.COLORMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Config1D.COLOR_BY_AMR);

        catchActionEvent(comm);
      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("By Color Info"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.COLORMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Config1D.COLOR_BY_INFO);

        catchActionEvent(comm);
      }
    });

    byDataSetMI = (MenuItem) colorMenu.add(new MenuItem("By Data Set"));
    byDataSetMI.setEnabled(false);
    byDataSetMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.COLORMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Config1D.COLOR_BY_DATA);

        catchActionEvent(comm);
      }
    });

   final Color bgColor = config.getBGColor();
   final Color fgColor = config.getFGColor();

    mi = (MenuItem) colorMenu.add(new MenuItem("Background Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFPoint.this, 
                     "Background Color", bgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.BACKGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);
      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("Foreground Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFPoint.this, 
                     "Foreground Color", fgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.FOREGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);
      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("Colormap Edit..."));
    mi.addActionListener(new ActionListener() {
     public void actionPerformed(ActionEvent e) {
       ColorMapDialog dialog = new ColorMapDialog(GFPoint.this);
       dialog.show();
     }
    });

    viewMenu.add(colorMenu);

    Menu labelMenu = new Menu("Labels");

    displayLabelMI = new CheckboxMenuItem("Display Labels");
    displayLabelMI.setState(config.isLabelDraw());
    displayLabelMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_LABEL) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayLabelMI.getState());
        catchActionEvent(comm);        
      }
    });
    labelMenu.add(displayLabelMI);
 
    final String xlabel = ((Config1D)config).getXLabel();
    final String ylabel = ((Config1D)config).getYLabel();

    changeLabelMI = (MenuItem) labelMenu.add(new MenuItem("Change Labels..."));
    changeLabelMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String[] str = { xlabel, ylabel };

        LabelInputDialog chooser = new LabelInputDialog(GFPoint.this, str);
        chooser.show();
        String args[] = chooser.getArgs();
        if (args != null) {
          String comm = String.valueOf(SV2Action.LABEL_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(args.length);
          for (int i=0; i<args.length; i++)
            comm = comm + ";" + args[i];
          catchActionEvent(comm);
        }
      }
    });

    moveLabelMI = (MenuItem) labelMenu.add(new MenuItem("Move Labels"));
    moveLabelMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.LABEL_MOVE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.add(labelMenu);

    Menu legendMenu = new Menu("Legend");
    
    displayLegendMI = new CheckboxMenuItem("Display Legends");
    displayLegendMI.setState(config.isLegendDraw());
    displayLegendMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_LEGEND) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayLegendMI.getState());
        catchActionEvent(comm);   
      }
    });
    legendMenu.add(displayLegendMI);

    changeLegendMI = (MenuItem) legendMenu.add(new MenuItem("Change Legend..."));
    changeLegendMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        DataSetBrief brief = getDataSet();
        DataSet[] data = brief.getDataSet();

        String names[] = new String[data.length];
        Color colors[] = new Color[data.length];
        int markers[] = new int[data.length];

        for (int i=0; i<data.length; i++) {
           names[i] = data[i].getDataName();
           colors[i] = data[i].getLineColor();
           markers[i] = data[i].getMarkerType();
        }
        LegendChange chooser = 
               new LegendChange(GFPoint.this, names, colors, markers);
        chooser.show();
      }
    });

    moveLegendMI = (MenuItem) legendMenu.add(new MenuItem("Move Legend"));
    moveLegendMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.LEGEND_MOVE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.add(legendMenu);

    Menu tickMenu = new Menu("Tick Marks");
    drawTickMI = new CheckboxMenuItem("Draw Ticks");
    drawTickMI.setState(config.isTickMarkDraw());
    drawTickMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.TICKMARKS) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(drawTickMI.getState());
        catchActionEvent(comm);        
      }
    });
    tickMenu.add(drawTickMI);

    changeTickMI = (MenuItem) tickMenu.add(new MenuItem("Change Ticks..."));
    changeTickMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Change Ticks...");
      }
    });

    viewMenu.add(tickMenu);

    mi = (MenuItem) viewMenu.add(new MenuItem("Resize Plot Area"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        canvas.setMouseMode(GCanvas1D.DATA_AREA_RESIZE_MODE);
        canvas.repaint();
      }
    });

    viewMenu.addSeparator();

    connectMI =  new CheckboxMenuItem("Connect point");
    connectMI.setState(((Config1D)config).isConnectPoints());
    connectMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.CONNECT_POINT) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(connectMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(connectMI);

    displayTimeMI = new CheckboxMenuItem("Display Time");
    displayTimeMI.setState(((Config1D)config).isDisplayTimeMode());
    displayTimeMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.DISPLAY_TIME) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(displayTimeMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(displayTimeMI);

    globalMinMaxMI = new CheckboxMenuItem("Use Global Minmax");
    globalMinMaxMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String comm = String.valueOf(SV2Action.GLOBALMINMAX) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(globalMinMaxMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(globalMinMaxMI);

    menubar.add(viewMenu);

    // Filter menus
    filterMenu = new Menu("Filter");
    createFilterMenu(); 
    menubar.add(filterMenu);

    // Time steps
    Menu timeMenu = new Menu("Time");

    animateMI = (MenuItem) timeMenu.add(new MenuItem("Animate"));
    animateMI.addActionListener(animateListener);

    nextMI = (MenuItem) timeMenu.add(new MenuItem("Next"));
    nextMI.addActionListener(nextListener);

    prevMI = (MenuItem) timeMenu.add(new MenuItem("Previous"));
    prevMI.addActionListener(previousListener);

    stopMI = (MenuItem) timeMenu.add(new MenuItem("Stop"));
    stopMI.addActionListener(stopListener);

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Goto..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Goto...");
      }
    });

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Animation Speed..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Animation Speed");
      }
    });

    mi = (MenuItem) timeMenu.add(new MenuItem("Buffer Size..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Buffer Size");
      }
    });

    menubar.add(timeMenu);

    setMenuBar(menubar);
  } 

  // create the filter menu
  private void createFilterMenu() {
    MenuItem mi;
  
    subFilterMenu();

    filterMenu.addSeparator();

    Vector level0Vec = new Vector();
    Vector level1Vec = new Vector();
    Vector level2Vec = new Vector();
    Vector level3Vec = new Vector();

    for (Enumeration e=config.getEngine().vmanager.getFilterMenu().get1DMenus().elements(); e.hasMoreElements();) {
      FilterMenuInfo info = (FilterMenuInfo)e.nextElement();

      if (info.level == 0) 
        level0Vec.addElement(info);
      else if (info.level == 1) 
        level1Vec.addElement(info);
      else if (info.level == 2) 
        level2Vec.addElement(info);
      else if (info.level == 3) 
        level3Vec.addElement(info);
    }

    for (Enumeration e=level0Vec.elements(); e.hasMoreElements();) {
      FilterMenuInfo lv0item = (FilterMenuInfo)e.nextElement();

      if (!lv0item.isItem) {
         Menu lv0Menu = new Menu(lv0item.name);

         for (Enumeration e1=level1Vec.elements(); e1.hasMoreElements();) {
           FilterMenuInfo lv1item = (FilterMenuInfo)e1.nextElement();
              
           if (lv1item.parent == lv0item.id) {
             if (!lv1item.isItem) {
                Menu lv1Menu = new Menu(lv1item.name);
                    
                for (Enumeration e2=level2Vec.elements(); 
                                              e2.hasMoreElements();) {
                   FilterMenuInfo lv2item = (FilterMenuInfo)e2.nextElement();

                   if (lv2item.parent == lv1item.id) {
                     if (!lv2item.isItem) {
                       Menu lv2Menu = new Menu(lv2item.name);

                       for (Enumeration e3=level3Vec.elements(); 
                                             e3.hasMoreElements();) {
                          FilterMenuInfo lv3item = (FilterMenuInfo)e3.nextElement();
                          if (lv3item.parent == lv2item.id) {
                            if (lv3item.isItem) {
                               mi = new MenuItem(lv3item.name);
                               FilterListener fl = new FilterListener(this, lv3item);
                               mi.addActionListener(fl);
                               lv2Menu.add(mi);
			    }
                          }
		       }
                       lv1Menu.add(lv2Menu);
		     } else {
                       mi = new MenuItem(lv2item.name);
                       FilterListener fl = new FilterListener(this, lv2item);
                       mi.addActionListener(fl);
                       lv1Menu.add(mi);
		     }
		   }
		}
                lv0Menu.add(lv1Menu);
	     } else {
                mi = new MenuItem(lv1item.name);
                FilterListener fl = new FilterListener(this, lv1item);
                mi.addActionListener(fl);
                lv0Menu.add(mi);
	     }
	   }

	 }
         filterMenu.add(lv0Menu);
      } else {
         mi = new MenuItem(lv0item.name);
         FilterListener fl = new FilterListener(this, lv0item);
         mi.addActionListener(fl);
         filterMenu.add(mi);
      }
    }     
  }

}  // end of class

