package sv.viewer.builtin;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import sv.engine.*;
import sv.viewer.*;
import sv.config.*;
import sv.data.*;
import sv.gui.*;
import sv.util.*;

/** 
 * Define a frame for 2d data
 */

public class GFrame2D extends GFrame implements WheelClient, DisplayImpl {

  // menu items
  private MenuItem splitMI, displayTimeMI, 
                   wireMI, cwireMI, spectrumMI, pointMI;
  private CheckboxMenuItem boundMI, globalMinMaxMI, contourMI;

  // global min and max value
  float globalxmin = Float.MAX_VALUE, globalxmax = Float.MIN_VALUE;
  float globalymin = Float.MAX_VALUE, globalymax = Float.MIN_VALUE;
  float globalzmin = Float.MAX_VALUE, globalzmax = Float.MIN_VALUE;
  
  JSlider zoomer;
  Label zoomValue;
  int zfactor = 40;

  Positions positions;
  WheelBehavior wbehavior;
  ZoomBehavior zbehavior;

  private DisplayPanel display;

  /**
   * constructor
   * @param brief initial data set
   * @param cfg configuration of this window
   */
  public GFrame2D(DataSetBrief brief, Configs cfg) {
    super(brief, cfg);

    DataSet[] data = brief.getDataSet();
    for (int i=0; i<data.length; i++) {
      for (int j=0; j<data[i].getNoOfTimes(); j++) {
        TimeData2D timedata = (TimeData2D)data[i].getTimeData(j);

        if (globalxmin > timedata.getxmin()) 
           globalxmin = timedata.getxmin();
        if (globalxmax < timedata.getxmax()) 
           globalxmax = timedata.getxmax();

        if (globalymin > timedata.getymin()) 
           globalymin = timedata.getymin();
        if (globalymax < timedata.getymax()) 
           globalymax = timedata.getymax();

        if (globalzmin > timedata.getzmin()) 
           globalzmin = timedata.getzmin();
        if (globalzmax < timedata.getzmax()) 
           globalzmax = timedata.getzmax();
      }
   }

   data = brief.getDataSet();
   for (int i=0; i<data.length; i++) {
     for (int j=0; j<data[i].getNoOfTimes(); j++) {
       J3DVertex[] vertex = ((TimeData2D)data[i].getTimeData(j)).makeVertex(true, globalzmin, globalzmax);
      ((TimeData2D)data[i].getTimeData(j)).makeFace(vertex, Configs.WIREFRAME, rcolors, gcolors, bcolors);
     }
   }

   init();
   displayInfo();
  } 

   /**
    * Calls the various methids necessary to initialize the 
    * program.
    */
   public void init() {
      // Set the layout manager
      setLayout(new BorderLayout());

      positions = new Positions(dataSet.getDataSet(), TimeData.TWOD);

      // Create the 2D canvas
      display = new DisplayPanel(this);
      add("Center", display);
   
      Image img = (new ImageIcon(getClass().getResource("wheel.gif"))).getImage();
      add("East", new Wheel(Wheel.VERTICAL, 140, this, img));

      Panel p2 = new Panel();
      p2.setLayout(new BorderLayout());

      JPanel p = new JPanel();
      p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
      p.add(Box.createRigidArea(new Dimension(20, 1)));
      p.add(new Wheel(Wheel.HORIZONTAL, 140, this, img));
      p.add(Box.createRigidArea(new Dimension(20, 1)));

      Panel zp = new Panel();
      zp.setLayout(new BorderLayout());
      zp.add("West", new Label("Zoom:"));
      zp.add("Center", zoomer = new JSlider(JSlider.HORIZONTAL, 0, 80, zfactor));   
      zp.add("East", zoomValue = new Label("0.0"));
      p.add(zp); 
      p.add(Box.createRigidArea(new Dimension(20, 1)));
      createSlideListener();

      p2.add("North", p);
      p2.add("Center", status = new StatusBar());
      add("South", p2);
    
      makeMenu();       
  }

  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {
    String comm = String.valueOf(SV2Action.COLORMAP_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID());
    for (int i=0; i<args.length; i++)
      comm = comm + ";" + args[i];
    catchActionEvent(comm);
  }


  /**
   * repaint
   */
  public void notifyRepaint() {
    updateDataSet();
  }

  /**
   * do action event
   */
  public void doActionEventWork(byte[] datum) {
    StringTokenizer st = new StringTokenizer(new String(datum), ";");

    // The first token is a command
    int comm = Integer.parseInt(st.nextToken());
    st.nextToken();   // trash it

    DataSet[] data;
    boolean flag;
    int sz;

    switch (comm) {
      case SV2Action.BOUNDINGBOX:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
        ((Config2D)config).setBoungBox(flag);
        boundMI.setState(flag);
        if (flag)
          positions.set(currentTime, true);
        else
          positions.set(currentTime, false);
        break;
        
      case SV2Action.CONTOUR:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
	contourMI.setState(flag);

        data = dataSet.getDataSet();

        if (flag) {
           if (data.length > 1) {
             status.display("Doesn't support contour for multi data set", true);
             return;
           }
        } else {
           positions.setContour(-1); 
           break;
        }

        sz = data[0].getNoOfTimes();
        for (int i=0; i<sz; i++) 
          ((TimeData2D)data[0].getTimeData(i)).makeContour(false);

        positions.makeContourObject(data[0]);
        displayInfo();
        break;

      case SV2Action.COLORMAP_CHANGE:
        setColorMap(st);
        updateDataSet();
        break;

      case SV2Action.BACKGROUND_COLOR:
        int r = Integer.parseInt(st.nextToken());
        int g = Integer.parseInt(st.nextToken());
        int b = Integer.parseInt(st.nextToken());
        config.setBGColor(new Color(r, g, b));
        Color3f bgColor = 
               new Color3f((float)r/255f, (float)g/255f, (float)b/255f);
        display.changeBGColor(bgColor);
        break;

      case SV2Action.PLOTMODE_CHANGE:
	wireMI.setEnabled(true);
        cwireMI.setEnabled(true);
        spectrumMI.setEnabled(true);
        pointMI.setEnabled(true);

        int plotmode = Integer.parseInt(st.nextToken());
        ((Config2D)config).setPlotMode(plotmode);
        if (plotmode == Configs.WIREFRAME)
          wireMI.setEnabled(false);
        else if (plotmode == Configs.COLORWIREFRAME)
          cwireMI.setEnabled(false);
        else if (plotmode == Configs.SHADING)
          spectrumMI.setEnabled(false);
        else if (plotmode == Configs.POINTPLOT)
          pointMI.setEnabled(false);

        data = dataSet.getDataSet();
        int large_slice = 100 / data.length;
        int progress = 0;

        for (int i=0; i<data.length; i++) {
          for (int j=0; j<data[i].getNoOfTimes(); j++) {
            ((TimeData2D)data[i].getTimeData(j)).getJ3dObject().makeBoundingBox(((TimeData2D)data[i].getTimeData(j)).getDimX(), ((TimeData2D)data[i].getTimeData(j)).getDimY());
            ((TimeData2D)data[i].getTimeData(j)).getJ3dObject().changeOption(plotmode);
            int small_slice = large_slice*progress + large_slice/data[i].getNoOfTimes()*j;
          status.display("Progress: " + small_slice + " %", false); 
	  }
          progress++;
        }

        positions.changeDisplayOption(data, TimeData.TWOD);
        display.changeDisplayOption(positions.getChild());
        displayInfo();
        break;
    }
  }

  /**
   * update data set
   */
  public void updateDataSet() {
    int plotmode = ((Config2D)config).getPlotMode();

    DataSet[] data = dataSet.getDataSet();
    int large_slice = 100 / data.length;
    int progress = 0;

    for (int i=0; i<data.length; i++) {
      for (int j=0; j<data[i].getNoOfTimes(); j++) {
        TimeData2D timedata = (TimeData2D)data[i].getTimeData(j);
        J3DVertex[] vertex = ((TimeData2D)data[i].getTimeData(j)).makeVertex(true, globalzmin, globalzmax);
        ((TimeData2D)data[i].getTimeData(j)).makeFace(vertex, plotmode, rcolors, gcolors, bcolors);
        int small_slice = large_slice*progress + large_slice/data[i].getNoOfTimes()*j;
        status.display("Updating: " + small_slice + " %", false); 
      }
      progress++;
    }

    positions.changeDisplayOption(data, TimeData.TWOD);
    display.changeDisplayOption(positions.getChild());
    displayInfo();
  }

  /**
   * animate 
   */
  public void doAnimate() {
    if (animator == null) {
      animator = new Animator(this, animationSpeed);
      animator.start();
    } else 
      System.out.println("Animation thread was corrupted ...");

    animateMI.setEnabled(false);
    nextMI.setEnabled(false);
    prevMI.setEnabled(false);
    stopMI.setEnabled(true);
  }

  /**
   * next
   */
  public void doNext(boolean b) {
    currentTime++;
    if (currentTime == timeValues.length)
      currentTime = 0;
    displayInfo();
  }

  /**
   * previous 
   */
  public void doPrevious() {
    currentTime--;
    if (currentTime == -1)
       currentTime = timeValues.length-1;
    displayInfo();
  }

  /**
   * stop 
   */
  public void doStop(int pos) {
    if (animator != null) {
      animator.stop();
      animator = null;
    }
    animateMI.setEnabled(true);
    stopMI.setEnabled(false);
    nextMI.setEnabled(true);
    prevMI.setEnabled(true);
  }

  /**
   * close this visualization window
   */
  public void closeFrame() {
    //////////////////////////////////////////////////////
    // DO SOME MISC WORKS

    this.dispose();    
  }

  /**
   * write misc information into the file
   */
  public void writeAux(DataOutputStream out) throws IOException {
    out.writeByte(0);
  }

  /**
   * wheel motion 
   */
  public void onWheelMotion(Wheel wheel, int delta) {
     wbehavior.wakeup(wheel, delta);
  }

  /**
   * when clicked reset interface
   */
  public void onWheelReset(Wheel wheel) {
    System.out.println("onWheelReset");
  }

  /**
   * create scene graph
   */
  public BranchGroup createSceneGraph(Canvas3D canvas) {
 
      // Define colors
      Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f bgColor = new Color3f(0.05f, 0.05f, 0.2f);

      // Create the root of branch group
      BranchGroup branchGroup = new BranchGroup();
      branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
      branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
      branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
      //      branchGroup.setCapability(BranchGroup.ALLOW_DETACH);

      // First 5 nodes are reserved to control this branch group
      for (int i=0; i<5; i++) {
         Group dummy = new Group();
         branchGroup.addChild(dummy);
      }

      // Create the bounding leaf node
      BoundingSphere bounds = 
           new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
                            
      BoundingLeaf boundingLeaf = new BoundingLeaf(bounds);
      branchGroup.addChild(boundingLeaf);   

      // Set up the background 
      // (background node is assigned at index 1)
      Background bg = new Background(bgColor);    // default black
      bg.setApplicationBounds(bounds);
      BranchGroup branch = new BranchGroup();
      branch.setCapability(BranchGroup.ALLOW_DETACH);
      branch.addChild(bg);
      branchGroup.setChild(branch, 1);

      // Create the ambient light
      // (ambient light node is assigned at index 2)
      AmbientLight ambLight = new AmbientLight(white);
      ambLight.setInfluencingBounds(bounds);
      branchGroup.setChild(ambLight, 2);

      // Create the directional light
      // (directional light is assgined at index 3)
      Vector3f dir = new Vector3f(-1.0f, -1.0f, -1.0f);
      DirectionalLight dirLight = new DirectionalLight(white, dir);
      dirLight.setInfluencingBounds(bounds);
      branchGroup.setChild(dirLight, 3);

      // create the transform group node to scale all objects so they
      // appear in the scene.
      // (transform group is assgined at index 4)
      Transform3D t3d = new Transform3D();
      t3d.setScale(0.4);
      TransformGroup transformGroup = new TransformGroup(t3d);
      transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
      transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
      transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
      transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND); 
      branchGroup.setChild(transformGroup, 4);

      // create the data set objects 
      // and insert it into transform group at index 0
      transformGroup.insertChild(positions.getChild(), 0);

      // Let the positions object know about the transform group
      positions.setTransformGroup(transformGroup);

      // Create the drag behavior node
      MouseRotate behavior = new MouseRotate(transformGroup);
      behavior.setTransformGroup(transformGroup);
      transformGroup.addChild(behavior);
      behavior.setSchedulingBounds(bounds);
	
      // Create the zoom behavior node
      MouseZoom behavior2 = new MouseZoom(transformGroup);
      behavior2.setTransformGroup(transformGroup);
      transformGroup.addChild(behavior2);
      behavior2.setSchedulingBounds(bounds);
	
      // Create the translation behavior node
      MouseTranslate behavior3 = new MouseTranslate(transformGroup);
      behavior3.setTransformGroup(transformGroup);
      transformGroup.addChild(behavior3);
      behavior3.setSchedulingBounds(bounds);

      // Create the wheel behavior node
      wbehavior = new WheelBehavior(transformGroup);
      transformGroup.addChild(wbehavior);
      wbehavior.setSchedulingBounds(bounds);

       // Create the zoom behavior node
      zbehavior = new ZoomBehavior(transformGroup);
      transformGroup.addChild(zbehavior);
      zbehavior.setSchedulingBounds(bounds);
    
      // let Java 3D perform optimizations on this scene graph
      branchGroup.compile();

      return branchGroup;    
  }


  // create the menu
  private void makeMenu() {
    MenuBar menubar = new MenuBar();
    MenuItem mi;

    // File manage
    Menu fileMenu = new Menu("File");

    mi = (MenuItem) fileMenu.add(new MenuItem("Save as..."));
    mi.addActionListener(saveasListener);

    fileMenu.addSeparator();

    splitMI = (MenuItem) fileMenu.add(new MenuItem("Split Data Set"));
    splitMI.setEnabled(false);
    splitMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Split Data Set");
      }
    });

    fileMenu.addSeparator();

    mi = (MenuItem) fileMenu.add(new MenuItem("Close"));
    mi.addActionListener(closeListener);

    menubar.add(fileMenu);

    // Edit manage
    Menu editMenu = new Menu("Edit");

    mi = (MenuItem) editMenu.add(new MenuItem("View in Editor"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("View in Editor");
      }
    });

    menubar.add(editMenu);

    // View manage
    Menu viewMenu = new Menu("View");

    wireMI = (MenuItem) viewMenu.add(new MenuItem("Wire Frame"));
    wireMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.WIREFRAME);
        catchActionEvent(comm);
      }
    });

    cwireMI = (MenuItem) viewMenu.add(new MenuItem("Colored Wire Frame"));
    cwireMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.COLORWIREFRAME);
        catchActionEvent(comm);
      }
    });

    spectrumMI = (MenuItem) viewMenu.add(new MenuItem("Color Mapped"));
    spectrumMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.SHADING);
        catchActionEvent(comm);
      }
    });

    pointMI = (MenuItem) viewMenu.add(new MenuItem("Point Plot"));
    pointMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.POINTPLOT);
        catchActionEvent(comm);
      }
    });

    int plotmode = ((Config2D)config).getPlotMode();
    if (plotmode == Configs.WIREFRAME)
      wireMI.setEnabled(false);
    else if (plotmode == Configs.COLORWIREFRAME)
      cwireMI.setEnabled(false);
    else if (plotmode == Configs.SHADING)
      spectrumMI.setEnabled(false);
    else if (plotmode == Configs.POINTPLOT)
      pointMI.setEnabled(false);

    Menu colorMenu = new Menu("Colors");
    final Color bgColor = config.getBGColor();

    mi = (MenuItem) colorMenu.add(new MenuItem("Background Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFrame2D.this, 
                     "Background Color", bgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.BACKGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);

      }
    });

    mi = (MenuItem) colorMenu.add(new MenuItem("Colormap Edit..."));
    mi.addActionListener(new ActionListener() {
     public void actionPerformed(ActionEvent e) {
       ColorMapDialog dialog = new ColorMapDialog(GFrame2D.this);
       dialog.show();
     }
    });

    viewMenu.add(colorMenu);

    viewMenu.addSeparator();

    boundMI = new CheckboxMenuItem("Bounding box");
    boundMI.setState(((Config2D)config).isBoundingBox());
    boundMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.BOUNDINGBOX) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(boundMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(boundMI);

    contourMI = new CheckboxMenuItem("Contour");
    contourMI.setState(false);
    contourMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.CONTOUR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(contourMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(contourMI);

    viewMenu.addSeparator();

    displayTimeMI = (MenuItem) viewMenu.add(new MenuItem("Display Time"));
    displayTimeMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Display Time");
      }
    });


    globalMinMaxMI = new CheckboxMenuItem("Use global Min & Max");
    globalMinMaxMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        System.out.println("Use global Min & Max");
      }
    });
    viewMenu.add(globalMinMaxMI);

    viewMenu.addSeparator();

    mi = (MenuItem) viewMenu.add(new MenuItem("Reset"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Reset");
      }
    });

    menubar.add(viewMenu);

    // Filter menus
    filterMenu = new Menu("Filter");
    createFilterMenu(); 
    menubar.add(filterMenu);

    // Time steps
    Menu timeMenu = new Menu("Time");

    animateMI = (MenuItem) timeMenu.add(new MenuItem("Animate"));
    animateMI.addActionListener(animateListener);

    nextMI = (MenuItem) timeMenu.add(new MenuItem("Next"));
    nextMI.addActionListener(nextListener);

    prevMI = (MenuItem) timeMenu.add(new MenuItem("Previous"));
    prevMI.addActionListener(previousListener);

    stopMI = (MenuItem) timeMenu.add(new MenuItem("Stop"));
    stopMI.addActionListener(stopListener);

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Goto..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Goto...");
      }
    });

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Animation Speed..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Animation Speed");
      }
    });

    mi = (MenuItem) timeMenu.add(new MenuItem("Buffer Size..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Buffer Size");
      }
    });

    menubar.add(timeMenu);

    setMenuBar(menubar);
  } 


  // create the filter menu
  private void createFilterMenu() {
    MenuItem mi;
  
    subFilterMenu();

    filterMenu.addSeparator();

    Vector level0Vec = new Vector();
    Vector level1Vec = new Vector();
    Vector level2Vec = new Vector();
    Vector level3Vec = new Vector();

    for (Enumeration e=config.getEngine().vmanager.getFilterMenu().get2DMenus().elements(); e.hasMoreElements();) {
      FilterMenuInfo info = (FilterMenuInfo)e.nextElement();

      if (info.level == 0) 
        level0Vec.addElement(info);
      else if (info.level == 1) 
        level1Vec.addElement(info);
      else if (info.level == 2) 
        level2Vec.addElement(info);
      else if (info.level == 3) 
        level3Vec.addElement(info);
    }

    for (Enumeration e=level0Vec.elements(); e.hasMoreElements();) {
      FilterMenuInfo lv0item = (FilterMenuInfo)e.nextElement();

      if (!lv0item.isItem) {
         Menu lv0Menu = new Menu(lv0item.name);

         for (Enumeration e1=level1Vec.elements(); e1.hasMoreElements();) {
           FilterMenuInfo lv1item = (FilterMenuInfo)e1.nextElement();
              
           if (lv1item.parent == lv0item.id) {
             if (!lv1item.isItem) {
                Menu lv1Menu = new Menu(lv1item.name);
                    
                for (Enumeration e2=level2Vec.elements(); 
                                              e2.hasMoreElements();) {
                   FilterMenuInfo lv2item = (FilterMenuInfo)e2.nextElement();

                   if (lv2item.parent == lv1item.id) {
                     if (!lv2item.isItem) {
                       Menu lv2Menu = new Menu(lv2item.name);

                       for (Enumeration e3=level3Vec.elements(); 
                                             e3.hasMoreElements();) {
                          FilterMenuInfo lv3item = (FilterMenuInfo)e3.nextElement();
                          if (lv3item.parent == lv2item.id) {
                            if (lv3item.isItem) {
                               mi = new MenuItem(lv3item.name);
                               FilterListener fl = new FilterListener(this, lv3item);
                               mi.addActionListener(fl);
                               lv2Menu.add(mi);
			    }
                          }
		       }
                       lv1Menu.add(lv2Menu);
		     } else {
                       mi = new MenuItem(lv2item.name);
                       FilterListener fl = new FilterListener(this, lv2item);
                       mi.addActionListener(fl);
                       lv1Menu.add(mi);
		     }
		   }
		}
                lv0Menu.add(lv1Menu);
	     } else {
                mi = new MenuItem(lv1item.name);
                FilterListener fl = new FilterListener(this, lv1item);
                mi.addActionListener(fl);
                lv0Menu.add(mi);
	     }
	   }

	 }
         filterMenu.add(lv0Menu);
      } else {
         mi = new MenuItem(lv0item.name);
         FilterListener fl = new FilterListener(this, lv0item);
         mi.addActionListener(fl);
         filterMenu.add(mi);
      }
    }     
  }

  // slide value changes
  private void createSlideListener() {
     ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
           int zfactor = zoomer.getValue();
	   float scalefudge = (float)zfactor / 10.0f - 4.0f;
           
           String val = String.valueOf(scalefudge);
           if (val.length() > 4)
              val = val.substring(0, 4);
           
           zoomValue.setText(val);
	   zbehavior.wakeup(scalefudge);
        }
     };
     zoomer.addChangeListener(changeListener);
   }

  // display data information
  private void displayInfo() {
    if (contourMI.getState())
      positions.setContour(currentTime);

    if (boundMI.getState())
      positions.set(currentTime, true);
    else
      positions.set(currentTime, false);

    DataSet[] data = dataSet.getDataSet(); 
    int dimx = ((TimeData2D)data[0].getTimeData(currentTime)).getDimX() - 1;
    int dimy = ((TimeData2D)data[0].getTimeData(currentTime)).getDimY() - 1;

    status.display((currentTime+1) + " / " +  data[0].getNoOfTimes() + "   ( " + (dimx+1) +"x"+ (dimy+1) + " )", false);
  }

}  // end of class

