package sv.viewer.builtin;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import sv.engine.*;
import sv.viewer.*;
import sv.config.*;
import sv.data.*;
import sv.gui.*;
import sv.util.*;

/** 
 * Define a frame for 3d data
 */

public class GFrame3D extends GFrame implements WheelClient, DisplayImpl {

  public static int XSLIDERCONTROL = 0;
  public static int YSLIDERCONTROL = 1;
  public static int ZSLIDERCONTROL = 2;
  public static int ZOOMCONTROL    = 3;

  JSlider zoomer;
  Label zoomLabel;
  Label zoomValue;
  int sliderMode = ZOOMCONTROL;
  int sliderCacheVal = 50;

  Positions positions;
  WheelBehavior wbehavior;
  ZoomBehavior zbehavior;

  private DisplayPanel display;

  // menu items
  private MenuItem splitMI, displayTimeMI, wireMI, shadingMI,
          xsliderMI, ysliderMI, zsliderMI, zoomMI;
  private CheckboxMenuItem boundMI, globalMinMaxMI,
          xgridMI, ygridMI, zgridMI;

  // global min and max value
  float globalvalmin = Float.MAX_VALUE, globalvalmax = Float.MIN_VALUE;

  // threshold value
  float threshold;

  /**
   * constructor
   * @param brief initial data set
   * @param cfg configuration of this window
   */
  public GFrame3D(DataSetBrief brief, Configs cfg) {
    super(brief, cfg);

    DataSet[] data = brief.getDataSet();

    for (int i=0; i<data.length; i++) {
      for (int j=0; j<data[i].getNoOfTimes(); j++) {
        TimeData3D timedata = (TimeData3D)data[i].getTimeData(j);
        timedata.makeJ3DObject(((Config3D)config).getPlotMode());

        if (globalvalmin > timedata.getzmin()) 
         globalvalmin = timedata.getzmin();
        if (globalvalmax < timedata.getzmax()) 
         globalvalmax = timedata.getzmax();
      }
   }

   init();
   displayInfo();
  } 

  /**
   * Calls the various methids necessary to initialize the 
   * program.
   */
  public void init() {
     // Set the layout manager
     setLayout(new BorderLayout());

     positions = new Positions(dataSet.getDataSet(), TimeData.THREED);

     // Create the 3D canvas
     display = new DisplayPanel(this);
     add("Center", display);
  
     Image img = (new ImageIcon(getClass().getResource("wheel.gif"))).getImage();
     add("East", new Wheel(Wheel.VERTICAL, 140, this, img));

     Panel p2 = new Panel();
     p2.setLayout(new BorderLayout());

     JPanel p = new JPanel();
     p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
     p.add(Box.createRigidArea(new Dimension(20, 1)));
     p.add(new Wheel(Wheel.HORIZONTAL, 140, this, img));
     p.add(Box.createRigidArea(new Dimension(20, 1)));

     Panel zp = new Panel();
     zp.setLayout(new BorderLayout());
     zp.add("West", zoomLabel = new Label("Zoom:"));
     zp.add("Center", zoomer = new JSlider(JSlider.HORIZONTAL, 0, 100, 50));   
     zp.add("East", zoomValue = new Label("0.0"));
     p.add(zp); 
     p.add(Box.createRigidArea(new Dimension(20, 1)));
     createSlideListener();

     p2.add("North", p);
     p2.add("Center", status = new StatusBar());
     add("South", p2);
   
     makeMenu();       
  }


  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {
  }

  /**
   * repaint
   */
  public void notifyRepaint() {
    updateDataSet();
  }

  /**
   * do action event
   */
  public void doActionEventWork(byte[] datum) {
    StringTokenizer st = new StringTokenizer(new String(datum), ";");

    // The first token is a command
    int comm = Integer.parseInt(st.nextToken());
    st.nextToken();   // trash it

    DataSet[] data;
    boolean flag;
    int sz;
    int large_slice;
    int progress;

    switch (comm) {
      case SV2Action.XGRID_VIEW:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
        xgridMI.setState(flag);
        if (flag) 
          xsliderMI.setEnabled(true);
        else {
          positions.setSlice(1, -1);    // first parameter means x grid
          xsliderMI.setEnabled(false);
          break;
        }

        data = dataSet.getDataSet();
        sz = data[0].getNoOfTimes();
        for (int i=0; i<sz; i++) 
          extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.XSKIPED, (int)(((TimeData3D)(data[0].getTimeData(0))).getDimX()/2), rcolors, gcolors, bcolors);

        positions.makeSliceData(TimeData3D.XSKIPED, data[0]);
        displayInfo();
        break;

      case SV2Action.YGRID_VIEW:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
        ygridMI.setState(flag);
        if (flag) 
          ysliderMI.setEnabled(true);
        else {
          positions.setSlice(2, -1);    // first parameter means x grid
          ysliderMI.setEnabled(false);
          break;
        }

        data = dataSet.getDataSet();
        sz = data[0].getNoOfTimes();
        for (int i=0; i<sz; i++) 
          extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.YSKIPED, (int)(((TimeData3D)(data[0].getTimeData(0))).getDimY()/2), rcolors, gcolors, bcolors);

        positions.makeSliceData(TimeData3D.YSKIPED, data[0]);
        displayInfo();
        break;

      case SV2Action.ZGRID_VIEW:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
        zgridMI.setState(flag);
        if (flag) 
          zsliderMI.setEnabled(true);
        else {
          positions.setSlice(3, -1);    // first parameter means x grid
          zsliderMI.setEnabled(false);
          break;
        }

        data = dataSet.getDataSet();
        sz = data[0].getNoOfTimes();
        for (int i=0; i<sz; i++) 
          extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.ZSKIPED, (int)(((TimeData3D)(data[0].getTimeData(0))).getDimZ()/2), rcolors, gcolors, bcolors);

        positions.makeSliceData(TimeData3D.ZSKIPED, data[0]);
        displayInfo();
        break;

      case SV2Action.GRIDUPDATE:
        data = dataSet.getDataSet();
        int extraction = 0;

        if (sliderMode == XSLIDERCONTROL) {
           extraction = (int)((((TimeData3D)(data[0].getTimeData(0))).getDimX()-1) * sliderCacheVal / 100);

	   //System.out.println("extraction: " + extraction);
           for (int i=0; i<data[0].getNoOfTimes(); i++) 
             extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.XSKIPED, extraction, rcolors, gcolors, bcolors);
           positions.makeSliceData(TimeData3D.XSKIPED, data[0]);
           positions.setSlice(1, currentTime);
	   ///////////////////////////////////////////////////////////////
           // NEED COLLABORATION
           ((Config3D)config).setXSliderValue(sliderCacheVal);

        } else if (sliderMode == YSLIDERCONTROL) {
           extraction = (int)((((TimeData3D)(data[0].getTimeData(0))).getDimY()-1) * sliderCacheVal / 100);
           for (int i=0; i<data[0].getNoOfTimes(); i++) 
             extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.YSKIPED, extraction, rcolors, gcolors, bcolors);
           positions.makeSliceData(TimeData3D.YSKIPED, data[0]);
           positions.setSlice(2, currentTime);
           ((Config3D)config).setYSliderValue(sliderCacheVal);

        } else if (sliderMode == ZSLIDERCONTROL) {
           extraction = (int)((((TimeData3D)(data[0].getTimeData(0))).getDimZ()-1) * sliderCacheVal / 100);
           for (int i=0; i<data[0].getNoOfTimes(); i++) 
             extractSliceData(dataSet.getID(), (TimeData3D)data[0].getTimeData(i), TimeData3D.ZSKIPED, extraction, rcolors, gcolors, bcolors);
           positions.makeSliceData(TimeData3D.ZSKIPED, data[0]);
           positions.setSlice(3, currentTime);
           ((Config3D)config).setZSliderValue(sliderCacheVal);
	}
        break;

      case SV2Action.BOUNDINGBOX:
        flag = Boolean.valueOf(st.nextToken()).booleanValue();
        ((Config3D)config).setBoungBox(flag);
        boundMI.setState(flag);
        if (flag)
          positions.set(currentTime, true);
        else
          positions.set(currentTime, false);
        break;
        
      case SV2Action.MATERIAL_CHANGE:
        float[] color = new float[3];

        color[0] = Float.valueOf(st.nextToken()).floatValue();
        color[1] = Float.valueOf(st.nextToken()).floatValue();
        color[2] = Float.valueOf(st.nextToken()).floatValue();
        ((Config3D)config).setAmbientColor(color);

        color = new float[3];
        color[0] = Float.valueOf(st.nextToken()).floatValue();
        color[1] = Float.valueOf(st.nextToken()).floatValue();
        color[2] = Float.valueOf(st.nextToken()).floatValue();
        ((Config3D)config).setEmissiveColor(color);

        color = new float[3];
        color[0] = Float.valueOf(st.nextToken()).floatValue();
        color[1] = Float.valueOf(st.nextToken()).floatValue();
        color[2] = Float.valueOf(st.nextToken()).floatValue();
        ((Config3D)config).setDiffuseColor(color);

        color = new float[3];
        color[0] = Float.valueOf(st.nextToken()).floatValue();
        color[1] = Float.valueOf(st.nextToken()).floatValue();
        color[2] = Float.valueOf(st.nextToken()).floatValue();
        ((Config3D)config).setSpecularColor(color);

        ((Config3D)config).setShininess(Float.valueOf(st.nextToken()).floatValue());
        ((Config3D)config).setCull(Integer.parseInt(st.nextToken()));

        data = dataSet.getDataSet();
        large_slice = 100 / data.length;
        progress = 0;

        for (int i=0; i<data.length; i++) {
          for (int j=0; j<data[i].getNoOfTimes(); j++) {
            ((TimeData3D)data[i].getTimeData(j)).getJ3dObject().makeBoundingBox();
            ((TimeData3D)data[i].getTimeData(j)).getJ3dObject().changeOption(((Config3D)config).getPlotMode(), makeAppearance());
            int small_slice = large_slice*progress + large_slice/data[i].getNoOfTimes()*j;
          status.display("Progress: " + small_slice + " %", false); 
	  }
          progress++;
        }

        positions.changeDisplayOption(data, TimeData.THREED);
        display.changeDisplayOption(positions.getChild());
        displayInfo();
        break;

      case SV2Action.PLOTMODE_CHANGE:
	wireMI.setEnabled(true);
        shadingMI.setEnabled(true);

        int plotmode = Integer.parseInt(st.nextToken());
        ((Config3D)config).setPlotMode(plotmode);
        if (plotmode == Configs.WIREFRAME)
          wireMI.setEnabled(false);
        else if (plotmode == Configs.SHADING)
          shadingMI.setEnabled(false);

        data = dataSet.getDataSet();
        large_slice = 100 / data.length;
        progress = 0;

        for (int i=0; i<data.length; i++) {
          for (int j=0; j<data[i].getNoOfTimes(); j++) {
            ((TimeData3D)data[i].getTimeData(j)).getJ3dObject().makeBoundingBox();
            ((TimeData3D)data[i].getTimeData(j)).getJ3dObject().changeOption(plotmode, makeAppearance());
            int small_slice = large_slice*progress + large_slice/data[i].getNoOfTimes()*j;
          status.display("Progress: " + small_slice + " %", false); 
	  }
          progress++;
        }

        positions.changeDisplayOption(data, TimeData.THREED);
        display.changeDisplayOption(positions.getChild());
        displayInfo();
        break;

      case SV2Action.BACKGROUND_COLOR:
        int r = Integer.parseInt(st.nextToken());
        int g = Integer.parseInt(st.nextToken());
        int b = Integer.parseInt(st.nextToken());
        config.setBGColor(new Color(r, g, b));
        Color3f bgColor = 
               new Color3f((float)r/255f, (float)g/255f, (float)b/255f);
        display.changeBGColor(bgColor);
        break;
    }
  }

  /**
   * update data set
   */
  public void updateDataSet() {
    DataSet[] data = dataSet.getDataSet();
    int large_slice = 100 / data.length;
    int progress = 0;

    for (int i=0; i<data.length; i++) {
      for (int j=0; j<data[i].getNoOfTimes(); j++) {
        TimeData3D timedata = (TimeData3D)data[i].getTimeData(j);
        timedata.makeJ3DObject(((Config3D)config).getPlotMode());
        int small_slice = large_slice*progress + large_slice/data[i].getNoOfTimes()*j;
        status.display("Updating: " + small_slice + " %", false); 
      }
      progress++;
    }

    positions.changeDisplayOption(data, TimeData.THREED);
    display.changeDisplayOption(positions.getChild());
    displayInfo();
  }

  /**
   * animate 
   */
  public void doAnimate() {
    if (animator == null) {
      animator = new Animator(this, animationSpeed);
      animator.start();
    } else 
      System.out.println("Animation thread was corrupted ...");

    animateMI.setEnabled(false);
    nextMI.setEnabled(false);
    prevMI.setEnabled(false);
    stopMI.setEnabled(true);
  }

  /**
   * next
   */
  public void doNext(boolean b) {
    currentTime++;
    if (currentTime == timeValues.length)
      currentTime = 0;
    displayInfo();
  }

  /**
   * previous 
   */
  public void doPrevious() {
    currentTime--;
    if (currentTime == -1)
       currentTime = timeValues.length-1;
    displayInfo();
  }

  /**
   * stop 
   */
  public void doStop(int pos) {
    if (animator != null) {
      animator.stop();
      animator = null;
    }
    animateMI.setEnabled(true);
    stopMI.setEnabled(false);
    nextMI.setEnabled(true);
    prevMI.setEnabled(true);
  }

  /**
   * close this visualization window
   */
  public void closeFrame() {
    this.dispose();
  }

  /**
   * write misc information into the file
   */
  public void writeAux(DataOutputStream out) throws IOException {
    out.writeByte(0);
  }

  /**
   * wheel motion 
   */
  public void onWheelMotion(Wheel wheel, int delta) {
     wbehavior.wakeup(wheel, delta);
  }

  /**
   * when clicked reset interface
   */
  public void onWheelReset(Wheel wheel) {
    System.out.println("onWheelReset");
  }

  /**
   * create scene graph
   */
  public BranchGroup createSceneGraph(Canvas3D canvas) {
 
    // Define colors
    Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
    Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
    Color3f red   = new Color3f(0.80f, 0.20f, 0.2f);
    Color3f bgColor = new Color3f(0.05f, 0.05f, 0.2f);

    // Create the root of branch group
    BranchGroup branchGroup = new BranchGroup();
    branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    // First 5 nodes are reserved to control this branch group
    for (int i=0; i<5; i++) {
       Group dummy = new Group();
       branchGroup.addChild(dummy);
    }

    // Create the bounding leaf node
    BoundingSphere bounds = 
         new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
                          
    BoundingLeaf boundingLeaf = new BoundingLeaf(bounds);
    branchGroup.addChild(boundingLeaf);   

    // Set up the background
    Background bg = new Background(bgColor);    // default black
    bg.setApplicationBounds(bounds);
    BranchGroup branch = new BranchGroup();
    branch.setCapability(BranchGroup.ALLOW_DETACH);
    branch.addChild(bg);
    branchGroup.setChild(branch, 1);

    // Create the ambient light
    AmbientLight ambLight = new AmbientLight(white);
    ambLight.setInfluencingBounds(bounds);
    branchGroup.setChild(ambLight, 2);

    // Create the directional light
    Vector3f dir = new Vector3f(-1.0f, -1.0f, -1.0f);
    DirectionalLight dirLight = new DirectionalLight(white, dir);
    dirLight.setInfluencingBounds(bounds);
    branchGroup.setChild(dirLight, 3);

    //////////////////////////////////////////////////////////
    // Create the directional light
    Vector3f dir2 = new Vector3f(1.0f, 1.0f, 1.0f);
    DirectionalLight dirLight2 = new DirectionalLight(white, dir2);
    dirLight2.setInfluencingBounds(bounds);
    branchGroup.addChild(dirLight2);
    //////////////////////////////////////////////////////////

    // create the transform group node to scale all objects so they
    // appear in the scene.
    Transform3D t3d = new Transform3D();
    t3d.setScale(0.4);
    TransformGroup transformGroup = new TransformGroup(t3d);
    transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
    transformGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND); 
    branchGroup.setChild(transformGroup, 4);

    // create the data set objects 
    // and insert it into transform group at index 0
    transformGroup.insertChild(positions.getChild(), 0);

    // Let the positions object know about the transform group
    positions.setTransformGroup(transformGroup);

    // Create the drag behavior node
    MouseRotate behavior = new MouseRotate(transformGroup);
    behavior.setTransformGroup(transformGroup);
    transformGroup.addChild(behavior);
    behavior.setSchedulingBounds(bounds);
	
    // Create the zoom behavior node
    MouseZoom behavior2 = new MouseZoom(transformGroup);
    behavior2.setTransformGroup(transformGroup);
    transformGroup.addChild(behavior2);
    behavior2.setSchedulingBounds(bounds);
	
    // Create the translation behavior node
    MouseTranslate behavior3 = new MouseTranslate(transformGroup);
    behavior3.setTransformGroup(transformGroup);
    transformGroup.addChild(behavior3);
    behavior3.setSchedulingBounds(bounds);

    // Create the wheel behavior node
    wbehavior = new WheelBehavior(transformGroup);
    transformGroup.addChild(wbehavior);
    wbehavior.setSchedulingBounds(bounds);

     // Create the zoom behavior node
    zbehavior = new ZoomBehavior(transformGroup);
    transformGroup.addChild(zbehavior);
    zbehavior.setSchedulingBounds(bounds);
    
    // let Java 3D perform optimizations on this scene graph
    branchGroup.compile();

    return branchGroup;    
  }


  // send update threshold command to the server
  private void sendUpdateThreshold() {
    if (!dataSet.isPrivate()) {
      String comm = String.valueOf(SV2Action.ISOVALUE_CHANGE) + ";" +
                                String.valueOf(dataSet.getID()) + ";" +
                                String.valueOf(threshold) + ";" +
				String.valueOf(getID());

      byte[] datum = comm.getBytes();
       
      try {
        config.getEngine().sender.serverCommand(datum);
      } catch (Exception E) {
        System.err.println("Error: connection to server");
      }

      if (!config.getEngine().sender.getIsMaster()) 
        config.getEngine().getGUI().message("(Slave)Event action to shared data was disabled.", true);
    } else
      System.out.println("Not support yet ...");
  }

  // create the menu
  private void makeMenu() {
    MenuBar menubar = new MenuBar();
    MenuItem mi;

    // File manage
    Menu fileMenu = new Menu("File");

    mi = (MenuItem) fileMenu.add(new MenuItem("Save as..."));
    mi.addActionListener(saveasListener);

    fileMenu.addSeparator();

    splitMI = (MenuItem) fileMenu.add(new MenuItem("Split Data Set"));
    splitMI.setEnabled(false);
    splitMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Split Data Set");
      }
    });

    fileMenu.addSeparator();

    mi = (MenuItem) fileMenu.add(new MenuItem("Close"));
    mi.addActionListener(closeListener);

    menubar.add(fileMenu);


    // Edit manage
    Menu editMenu = new Menu("Edit");

    mi = (MenuItem) editMenu.add(new MenuItem("View in Editor"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("View in Editor");
      }
    });

    menubar.add(editMenu);
    

    // View manage
    Menu viewMenu = new Menu("View");

    wireMI = (MenuItem) viewMenu.add(new MenuItem("Wire Frame"));
    wireMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.WIREFRAME);
        catchActionEvent(comm);
      }
    });

    shadingMI = (MenuItem) viewMenu.add(new MenuItem("Shading"));
    shadingMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.PLOTMODE_CHANGE) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(Configs.SHADING);
        catchActionEvent(comm);
      }
    });

    int plotmode = ((Config3D)config).getPlotMode();
    if (plotmode == Configs.WIREFRAME)
      wireMI.setEnabled(false);
    else if (plotmode == Configs.SHADING)
      shadingMI.setEnabled(false);


    mi = (MenuItem) viewMenu.add(new MenuItem("Material Edit..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        materialChange();
      }
    });

    final Color bgColor = config.getBGColor();

    mi = (MenuItem) viewMenu.add(new MenuItem("Background Color..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = 
              JColorChooser.showDialog(GFrame3D.this, 
                     "Background Color", bgColor);
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.BACKGROUND_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);

      }
    });

    viewMenu.addSeparator();

    boundMI = new CheckboxMenuItem("Bounding box");
    boundMI.setState(((Config3D)config).isBoundingBox());
    boundMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.BOUNDINGBOX) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(boundMI.getState());
        catchActionEvent(comm);
      }
    });
    viewMenu.add(boundMI);


    displayTimeMI = (MenuItem) viewMenu.add(new MenuItem("Display Time"));
    displayTimeMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Display Time");
      }
    });

    globalMinMaxMI = new CheckboxMenuItem("Use global Min & Max");
    globalMinMaxMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        System.out.println("Use global Min & Max");
      }
    });
    viewMenu.add(globalMinMaxMI);

    viewMenu.addSeparator();

    mi = (MenuItem) viewMenu.add(new MenuItem("Reset"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Reset");
      }
    });

    menubar.add(viewMenu);

    // control menus
    Menu controlMenu = new Menu("Control");

    mi = (MenuItem) controlMenu.add(new MenuItem("Change Threshold"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {

        String lbl =  "Threshold range " + String.valueOf(globalvalmin) + 
                      " to " + String.valueOf(globalvalmax);
        UserInputDialog dialog = 
             new UserInputDialog(GFrame3D.this, "Change Threshold",
                "Choose new threshold:", lbl,
                UserInputDialog.FLOAT_CHOOSER,
                new Float(globalvalmin), new Float(globalvalmax), 
                new Float(threshold));
        dialog.show();

        if (!dialog.clickedCancel()) {
          threshold = dialog.getChoosedValue().floatValue();
          sendUpdateThreshold();
        }
      }
    });

    controlMenu.addSeparator();

    // grid menus
    Menu gridMenu = new Menu("Grids");

    xgridMI = new CheckboxMenuItem("X Grid");
    xgridMI.setState(false); 
    xgridMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.XGRID_VIEW) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(xgridMI.getState());
        catchActionEvent(comm);
      }
    });
    gridMenu.add(xgridMI);

    ygridMI = new CheckboxMenuItem("Y Grid");
    ygridMI.setState(false); 
    ygridMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.YGRID_VIEW) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(ygridMI.getState());
        catchActionEvent(comm);
      }
    });
    gridMenu.add(ygridMI);

    zgridMI = new CheckboxMenuItem("Z Grid");
    zgridMI.setState(false); 
    zgridMI.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        
        String comm = String.valueOf(SV2Action.ZGRID_VIEW) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(zgridMI.getState());
        catchActionEvent(comm);
      }
    });
    gridMenu.add(zgridMI);

    mi = (MenuItem) gridMenu.add(new MenuItem("Update grids"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String comm = String.valueOf(SV2Action.GRIDUPDATE) + ";" +
                      String.valueOf(getDataSet().getID());
        catchActionEvent(comm);
      }
    });

    controlMenu.add(gridMenu);    

    // slide choose menus
    Menu slideMenu = new Menu("Slides");

    xsliderMI = (MenuItem) slideMenu.add(new MenuItem("X Grid Control"));
    xsliderMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sliderMode = XSLIDERCONTROL;
        changeSliderMode();
      }
    });

    ysliderMI = (MenuItem) slideMenu.add(new MenuItem("Y Grid Control"));
    ysliderMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sliderMode = YSLIDERCONTROL;
        changeSliderMode();
      }
    });

    zsliderMI = (MenuItem) slideMenu.add(new MenuItem("Z Grid Control"));
    zsliderMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sliderMode = ZSLIDERCONTROL;
        changeSliderMode();
      }
    });

    zoomMI = (MenuItem) slideMenu.add(new MenuItem("Zoom Control"));
    zoomMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sliderMode = ZOOMCONTROL;
        changeSliderMode();
      }
    });

    controlMenu.add(slideMenu);    

    menubar.add(controlMenu);

    // Filter menus
    Menu filterMenu = new Menu("Filter");
    menubar.add(filterMenu);

    // Time steps
    Menu timeMenu = new Menu("Time");

    animateMI = (MenuItem) timeMenu.add(new MenuItem("Animate"));
    animateMI.addActionListener(animateListener);

    nextMI = (MenuItem) timeMenu.add(new MenuItem("Next"));
    nextMI.addActionListener(nextListener);

    prevMI = (MenuItem) timeMenu.add(new MenuItem("Previous"));
    prevMI.addActionListener(previousListener);

    stopMI = (MenuItem) timeMenu.add(new MenuItem("Stop"));
    stopMI.addActionListener(stopListener);

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Goto..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Goto...");
      }
    });

    timeMenu.addSeparator();
    mi = (MenuItem) timeMenu.add(new MenuItem("Animation Speed..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Animation Speed");
      }
    });

    mi = (MenuItem) timeMenu.add(new MenuItem("Buffer Size..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Buffer Size");
      }
    });

    menubar.add(timeMenu);

    setMenuBar(menubar);
  } 

  // material change
  private void materialChange() {
    float[] ac = ((Config3D)config).getAmbientColor();
    float[] ec = ((Config3D)config).getEmissiveColor();
    float[] dc = ((Config3D)config).getDiffuseColor();
    float[] sc = ((Config3D)config).getSpecularColor();
    MaterialEditor editor = new MaterialEditor(this, 
              new Color3f(ac[0], ac[1], ac[2]),
              new Color3f(ec[0], ec[1], ec[2]),
              new Color3f(dc[0], dc[1], dc[2]),
              new Color3f(sc[0], sc[1], sc[2]),
              ((Config3D)config).getShininess(), 
              ((Config3D)config).getCull());
    editor.setVisible(true);
    String[] args = editor.getResults();
    String comm = String.valueOf(SV2Action.MATERIAL_CHANGE) + ";" +
                  String.valueOf(getDataSet().getID());

    if (args == null) {
      System.out.println("Material editor canceled.");
      return;
    }
    
    for (int i=0; i<14; i++)
      comm = comm + ";" + args[i];

    catchActionEvent(comm);
  }


  // make appearance 
  // INITAL VERSION (transparency, material should be supported ...)
  private Appearance makeAppearance() {
     Appearance app = new Appearance();

     // Set up the transparency attributes
	 /*
     TransparencyAttributes ta = new TransparencyAttributes();
     ta.setTransparencyMode(TransparencyAttributes.BLENDED);
     ta.setTransparency(0.6f);
     app.setTransparencyAttributes(ta);
	 */

     PolygonAttributes pa = new PolygonAttributes();
     pa.setCullFace(((Config3D)config).getCull());
     app.setPolygonAttributes(pa);

     // Set up the material properties
     float[] ac = ((Config3D)config).getAmbientColor();
     float[] ec = ((Config3D)config).getEmissiveColor();
     float[] dc = ((Config3D)config).getDiffuseColor();
     float[] sc = ((Config3D)config).getSpecularColor();

System.out.println(ac[0] + " " + ac[1]  + " " + ac[2]);     
     Material material = new Material(
                  new Color3f(ac[0], ac[1], ac[2]),
                  new Color3f(ec[0], ec[1], ec[2]),
                  new Color3f(ec[0], dc[1], dc[2]),
                  new Color3f(sc[0], sc[1], sc[2]),
                  ((Config3D)config).getShininess());

     material.setLightingEnable(true);
     app.setMaterial(material);   
     return app;
  }

  // extract surface data info 
  private void extractSliceData(long id, TimeData3D td, int skipmode, 
           int extraction, float[] rcolors, float[] gcolors, float[] bcolors) {

      int timesteps, dimx = 0, dimy = 0, dimz = 0, tot;
      float timeval;
      GridInfo3D grids = null;

      long pos = 0;

      try {
         RandomAccessFile in = new RandomAccessFile(new File(config.getEngine().getCacheDir(), String.valueOf(id)), "r");
 
         timesteps = in.readInt();  
         pos = 4;

         for (int i=0; i<timesteps; i++) {
            // set the file position
            pos = pos + (dimx * dimy * dimz * 4);
            in.seek(pos);
            pos = pos + 16;

            // read time value and dimension
	    timeval = in.readFloat(); 
            dimx = in.readInt(); 
            dimy = in.readInt();
            dimz = in.readInt();

            if (td.getTime().getAbsTime() == timeval) {

               if (skipmode == TimeData3D.XSKIPED) {
                  tot = dimy * dimz;
                  grids = new GridInfo3D(tot, TimeData3D.XSKIPED, extraction);

                  in.skipBytes(extraction * tot * 4);
		  //System.out.println("current: " + in.getFilePointer());

                  for (int j=0; j<tot; j++) 
                     grids.setValue(j, in.readFloat());

               } else if (skipmode == TimeData3D.YSKIPED) {
                  tot = dimx * dimz;
                  int idx = 0;
                  grids = new GridInfo3D(tot, TimeData3D.YSKIPED, extraction);

                  for (int j=0; j<dimx; j++) {
                     in.skipBytes((dimz * extraction) * 4);
		     //System.out.println("current: " + in.getFilePointer());
 
                     for (int k=0; k<dimz; k++) {
                        grids.setValue(idx, in.readFloat());
                        idx++;
                     }
                     in.skipBytes((dimz * (dimy-extraction-1)) * 4);
                  }

               } else if (skipmode == TimeData3D.ZSKIPED) {
                  tot = dimx * dimy;
                  int idx = 0;
                  grids = new GridInfo3D(tot, TimeData3D.ZSKIPED, extraction);

                  for (int j=0; j<dimx; j++) {
                     for (int k=0; k<dimy; k++) {
                        in.skipBytes(extraction * 4);
	   	     //System.out.println("current: " + in.getFilePointer());
                        grids.setValue(idx, in.readFloat());
                        in.skipBytes((dimz-extraction-1) * 4);
		        //System.out.println(grids.getValue(idx));
                        idx++;
                     }
                  }
               }
               td.makeSliceObject(grids, skipmode, rcolors, gcolors, bcolors);
               break;
	    }
          }

          in.close();

       } catch (Exception e) {
         e.printStackTrace();
       }
  }

  // slide value changes
  private void createSlideListener() {
     ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
	  if (sliderMode == ZOOMCONTROL) {
            int zfactor = zoomer.getValue();
	    float scalefudge = (float)zfactor / 10.0f - 4.0f;
           
            String val = String.valueOf(scalefudge);
            if (val.length() > 4)
              val = val.substring(0, 4);
           
            zoomValue.setText(val);
	    zbehavior.wakeup(scalefudge);
	  } else {
	    sliderCacheVal = zoomer.getValue();
            updateSliderValue(); 
          }	    
        }
     };
     zoomer.addChangeListener(changeListener);
  }

  // change slider mode
  private void changeSliderMode() {
    switch (sliderMode) {
      case 3:
	zoomLabel.setText("Zoom :");
        zoomer.setValue(((Config3D)config).getZoomValue());
        String sval = String.valueOf(((float)((Config3D)config).getZoomValue() / 10.0f - 5.0f));
        if (sval.length() > 4)
           sval = sval.substring(0, 4);
        zoomValue.setText(sval);
        break;

      case 0:
	zoomLabel.setText("X Grid:");
        zoomer.setValue(((Config3D)config).getXSliderValue());
        updateSliderValue();
        break;

      case 1:
	zoomLabel.setText("Y Grid:");
	zoomer.setValue(((Config3D)config).getYSliderValue());
        updateSliderValue();
        break;

      case 2:
	zoomLabel.setText("Z Grid:");
        zoomer.setValue(((Config3D)config).getZSliderValue());
        updateSliderValue();
        break;
    }
  }

  // update slider value
  private void updateSliderValue() {
     if (sliderMode == ZOOMCONTROL) return;

     float val = 0.0f;
     DataSet[] data = dataSet.getDataSet();
 
     if (sliderMode == XSLIDERCONTROL) 
         val = (((TimeData3D)(data[0].getTimeData(0))).getDimX()-1)* sliderCacheVal / 100.0f;
     else if (sliderMode == YSLIDERCONTROL) 
         val = (((TimeData3D)(data[0].getTimeData(0))).getDimY()-1)* sliderCacheVal / 100.0f;
     else if (sliderMode == ZSLIDERCONTROL) 
         val = (((TimeData3D)(data[0].getTimeData(0))).getDimZ()-1)* sliderCacheVal / 100.0f;

     String sval = String.valueOf(val);
     if (sval.length() > 4)
        sval = sval.substring(0, 4);
     zoomValue.setText(sval);
  }  

  // display data information
  private void displayInfo() {
    if (xgridMI.getState())
      positions.setSlice(1, currentTime);
    if (ygridMI.getState())
      positions.setSlice(2, currentTime);
    if (zgridMI.getState())
      positions.setSlice(3, currentTime);

    if (boundMI.getState())
      positions.set(currentTime, true);
    else
      positions.set(currentTime, false);

    DataSet[] data = dataSet.getDataSet();
    int tri = ((TimeData3D)data[0].getTimeData(currentTime)).triangles.length;
    threshold = ((TimeData3D)data[0].getTimeData(currentTime)).getThreshold();
     //     updateSliderValue();

     status.display((currentTime+1) + " / " + data[0].getNoOfTimes() +
        " threshold: " + threshold +
        " (" + tri + " triangles)", false);
  }

}  // end of class


