package sv.viewer.builtin;

import java.awt.*;
import java.awt.image.ImageProducer;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import sv.engine.*;
import sv.viewer.*;
import sv.config.*;
import sv.util.*;
import sv.data.*;
import sv.gui.*;

/** 
 * Define a frame for 1d data
 */
public class GFrameVol extends GFrame {
  static int XROTATE = 0;
  static int YROTATE = 1;

  // canvas to draw the volume
  private GCanvasVol canvas;

  // image information storage
  private Vector imageStorage;
 
  // menu items will be here
  MenuItem xanimMI, yanimMI, stopMI;  

  // rotate animation axis
  private int rotate_mode = XROTATE;

  // image file prefix
  String file_prefix;

  // cache images for animation
  private Image anim_cache_img[];
  private double rotx_cache[];
  private double roty_cache[];
  private double requested_value;
  private int anim_pos = 0;

  /**
   * constructor
   * @param brief initial data set
   */
  public GFrameVol(DataSetBrief brief, File path, byte[] arr, 
                                         double[] vars, Configs cfg) {
    setTitle(brief.getDataName());
    dataSet = brief;
    file_prefix = String.valueOf(System.currentTimeMillis());
    animationSpeed = 1000;

    //    timeValues = new float[1];
    //    timeValues[0] = 0.0f;
    config = cfg;

    imageStorage = new Vector();
    saveImage(path, arr, vars);

    // create menu 
    makeMenu();

    // layout the components
    setLayout(new BorderLayout());

    ImageInfo info = (ImageInfo)imageStorage.firstElement();
    Image img = createImage(info.filename);
    
    add("Center", canvas = new GCanvasVol(this, img));
    add("South", status = new StatusBar());
    status.display(brief.getDataName() + " created.", false);

    WindowListener l = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
      }
    };
    this.addWindowListener(l);
  } 

  /**
   * display message on the staus bar
   */
  public void message(String msg, boolean critical) {
    status.display(msg, critical);
  }

  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {

  }

  /**
   * repaint
   */
  public void notifyRepaint() {
    canvas.repaint();
  }

  /**
   * do action event
   */
  public void doActionEventWork(byte[] datum) {
    StringTokenizer st = new StringTokenizer(new String(datum), ";");

    // The first token is a command
    int comm = Integer.parseInt(st.nextToken());
    st.nextToken();   // trash it

    DataSet[] data;
    boolean flag;
    int sz;

    switch (comm) {
      case SV2Action.VOLUME_RAY_COLOR:
        double r = Integer.parseInt(st.nextToken()) / 255;
        double g = Integer.parseInt(st.nextToken()) / 255;
        double b = Integer.parseInt(st.nextToken()) / 255;
        double[] cvals = { r, g, b };
        ((Config3D)config).setRayColor(cvals);

        double rotx = ((Config3D)config).getRotationX();
        double roty = ((Config3D)config).getRotationY();
        int pos = requestImage(r, g, b, rotx, roty);

        if (pos >= 0) {
          ImageInfo info = (ImageInfo)imageStorage.elementAt(pos);
          Image img = createImage(info.filename);
          canvas.changeImage(img);
        }
        break;
    }
  }

  /**
   * update data set
   */
  public void updateDataSet() {
    System.out.println("Update data set: Not implemented.");
  }

  /**
   * update image
   */
  public void updateImage(File path, byte[] arr, double[] vars, boolean b) {
    int pos = saveImage(path, arr, vars);
    if (b) {
      ImageInfo info = (ImageInfo)imageStorage.elementAt(pos);
      Image img = createImage(info.filename);
      canvas.changeImage(img);
      message("Rotation x = " + ((Config3D)config).getRotationX() + ", y = " + 
                                   ((Config3D)config).getRotationX(), false);
    }
  }

  /**
   * rotate aound x axis
   */
  public void RotateX(boolean increase) {
    double[] rayColor = ((Config3D)config).getRayColor();
    double r = rayColor[0];
    double g = rayColor[1];
    double b = rayColor[2];

    double rotx = ((Config3D)config).getRotationX();
    double roty = ((Config3D)config).getRotationY();
    int pos = -1;

    if (increase) {
      if (rotx+30 > 170.0) rotx = -180.0; 
      ((Config3D)config).setRotation(rotx+30, roty);
      pos = requestImage(r, g, b, rotx+30, roty);
    } else {
      if (rotx-30 < -180.0) rotx = 150.0; 
      ((Config3D)config).setRotation(rotx-30, roty);
      pos = requestImage(r, g, b, rotx-30, roty);
    }

    if (pos >= 0) {
      ImageInfo info = (ImageInfo)imageStorage.elementAt(pos);
      Image img = createImage(info.filename);
      canvas.changeImage(img);
      message("Rotation x = " + ((Config3D)config).getRotationX() + ", y = " + 
                                   ((Config3D)config).getRotationX(), false);
    }
  }

  /**
   * rotae around y axis
   */
  public void RotateY(boolean increase) {
    double[] rayColor = ((Config3D)config).getRayColor();
    double r = rayColor[0];
    double g = rayColor[1];
    double b = rayColor[2];

    double rotx = ((Config3D)config).getRotationX();
    double roty = ((Config3D)config).getRotationY();
    int pos = -1;

    if (increase) {
      if (roty+30 > 170.0) roty = -180.0; 
      ((Config3D)config).setRotation(rotx, roty+30);
      pos  = requestImage(r, g, b, rotx, roty+30);
    } else {
      if (roty-30 < -180.0) roty = 150.0; 
      ((Config3D)config).setRotation(rotx, roty-30);
      pos  = requestImage(r, g, b, rotx, roty-30);
    }

    if (pos >= 0) {
      ImageInfo info = (ImageInfo)imageStorage.elementAt(pos);
      Image img = createImage(info.filename);
      canvas.changeImage(img);
      message("Rotation x = " + ((Config3D)config).getRotationX() + ", y = " + 
                                   ((Config3D)config).getRotationX(), false);
    }
  }

  /**
   * animate 
   */
  public void doAnimate() {
    if (animator == null) {
      animator = new Animator(this, animationSpeed);
      animator.start();
    } else 
      System.out.println("Animation thread was corrupted ...");
  }

  /**
   * next
   */
  public void doNext(boolean b) {
    anim_pos++;
    if (anim_pos == 12)
      anim_pos = 0;
    canvas.changeImage(anim_cache_img[anim_pos]);
   ((Config3D)config).setRotation(rotx_cache[anim_pos], roty_cache[anim_pos]);
    message("Rotation x = " + rotx_cache[anim_pos] + ", y = " + 
                                   roty_cache[anim_pos], false);
  }

  /**
   * previous 
   */
  public void doPrevious() {
  }

  /**
   * stop 
   */
  public void doStop(int pos) {
    if (animator != null) {
      animator.stop();
      animator = null;
    }

    xanimMI.setEnabled(true);
    yanimMI.setEnabled(true);
    stopMI.setEnabled(false);
  }


  /**
   * close this visualization window
   */
  public void closeFrame() {

    //////////////////////////////////////////////////////
    // KILL ATTACHED THREADS

    for (int i=0; i<imageStorage.size(); i++) {
      ImageInfo info = (ImageInfo)imageStorage.elementAt(i);
      info.filename.delete();
    }
    imageStorage.removeAllElements();
    imageStorage = null;

    anim_cache_img = null;
    rotx_cache = null;
    roty_cache = null;
    
    canvas = null;
    this.dispose();    
  }

  /**
   * write misc information into the file
   */
  public void writeAux(DataOutputStream out) throws IOException {

  }

  /**
   * received bulk images from the server to animate the images
   */
  public void receivedBulkImages() {

    message("Preparing the images for animation.", false);

    double[] rayColor = ((Config3D)config).getRayColor();
    double r = rayColor[0];
    double g = rayColor[1];
    double b = rayColor[2];

    double rotx, roty;

    int sz = imageStorage.size();
    Vector vec = new Vector();
    boolean flag = false;
    ImageInfo info = null;

    if (rotate_mode == XROTATE) {
      roty = requested_value;

      for (rotx=-180.0; rotx<180.0; rotx=rotx+30.0) {
        for (int i=0; i<sz; i++) {
          info = (ImageInfo)imageStorage.elementAt(i);
          if (info.contains(r, g, b, rotx, roty)) {
            flag = true;
            break;
          } 
	}
        if (flag) {
          vec.addElement(info);
          flag = false;
	} else {
System.out.println("at(y): " + rotx + ", " + roty);
          message("Missing image frames.", true);
          return;
        }
      }
    } else if (rotate_mode == YROTATE) {
      rotx = requested_value;

      for (roty=-180.0; roty<180; roty=roty+30.0) {
        for (int i=0; i<sz; i++) {
          info = (ImageInfo)imageStorage.elementAt(i);
          if (info.contains(r, g, b, rotx, roty)) {
            flag = true;
            break;
	  }
	}
        if (flag) {
          vec.addElement(info);
          flag = false;
	} else {
System.out.println("at(x): " + rotx + ", " + roty);
          message("Missing image frames.", true);
          return;
        }
      }
    }
    
    // double check
    if (vec.size() != 12) {
      message("Missing image frames(12).", true);
      return;
    }

    anim_cache_img = new Image[12];
    rotx_cache = new double[12];
    roty_cache = new double[12];

    for (int i=0; i<12; i++) {
      info = (ImageInfo)vec.elementAt(i);
      anim_cache_img[i] = createImage(info.filename);
      rotx_cache[i] = info.rotx;
      roty_cache[i] = info.roty;
    }
    animateListen();     
  }

  // create the menu
  private void makeMenu() {
    MenuBar menubar = new MenuBar();
    MenuItem mi;

    // File manage
    Menu fileMenu = new Menu("File");

    mi = (MenuItem) fileMenu.add(new MenuItem("Save as..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Save as...");
      }
    });

    mi = (MenuItem) fileMenu.add(new MenuItem("Print..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Print...");
      }
    });

    mi = (MenuItem) fileMenu.add(new MenuItem("GIF Images..."));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("GIF Images...");
      }
    });

    fileMenu.addSeparator();

    mi = (MenuItem) fileMenu.add(new MenuItem("Close"));
    mi.addActionListener(closeListener);

    menubar.add(fileMenu);


    // View manage
    Menu viewMenu = new Menu("View");

    mi = (MenuItem) viewMenu.add(new MenuItem("Pan Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Pan Mode");
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Zoom Mode"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Zoom Mode");
      }
    });

    mi = (MenuItem) viewMenu.add(new MenuItem("Reset"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("Reset");
      }
    });

    viewMenu.addSeparator();

    final double[] rayColor = ((Config3D)config).getRayColor();
    mi = (MenuItem) viewMenu.add(new MenuItem("Ray Color"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Color c = JColorChooser.showDialog(GFrameVol.this, "Ray Color", 
     new Color((float)rayColor[0], (float)rayColor[0], (float)rayColor[0]));
        if (c == null)
          return;

        String comm = String.valueOf(SV2Action.VOLUME_RAY_COLOR) + ";" +
                      String.valueOf(getDataSet().getID()) + ";" +
                      String.valueOf(c.getRed()) + ";" +
                      String.valueOf(c.getGreen()) + ";" +
                      String.valueOf(c.getBlue());
        catchActionEvent(comm);
      }
    });

    menubar.add(viewMenu);

    // Filter menus
    filterMenu = new Menu("Filter");
    mi = (MenuItem) filterMenu.add(new MenuItem("Downsizing"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String arg = JOptionPane.showInputDialog(GFrameVol.this, "Enter the downsize factor:");
        basicFilter(SV2Action.DOWNSIZE, arg);
      }
    });

    mi = (MenuItem) filterMenu.add(new MenuItem("Smoothing"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        basicFilter(SV2Action.SMOOTHE, null);
      }
    });

    mi = (MenuItem) filterMenu.add(new MenuItem("Downsizing & Smoothing"));
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         String arg = JOptionPane.showInputDialog(GFrameVol.this, "Enter the downsize factor:");
         basicFilter(SV2Action.DOWNSIZE_SMOOTHE, arg);
      }
    });

    menubar.add(filterMenu);

    // Animation manage
    Menu animationMenu = new Menu("Animation");

    xanimMI = (MenuItem) animationMenu.add(new MenuItem("Rotation X"));
    xanimMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rotate_mode = YROTATE;
        prepareImages();
        xanimMI.setEnabled(false);
        yanimMI.setEnabled(false);
        stopMI.setEnabled(true);
      }
    });

    yanimMI = (MenuItem) animationMenu.add(new MenuItem("Rotation Y"));
    yanimMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rotate_mode = XROTATE;
        prepareImages();
        xanimMI.setEnabled(false);
        yanimMI.setEnabled(false);
        stopMI.setEnabled(true);
      }
    });

    stopMI = (MenuItem) animationMenu.add(new MenuItem("Stop"));
    stopMI.setEnabled(false);
    stopMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stopListen();
      }
    });

    menubar.add(animationMenu);

    setMenuBar(menubar);
  } 

  // basic filter for 3d
  private void basicFilter(int opt, String arg) {
    if (!dataSet.isPrivate()) {
      if (!config.getEngine().sender.getIsMaster()) {
        config.getEngine().getGUI().message("(Slave)Event action to shared data was disabled.", true);
        return;
      }

      String comm = String.valueOf(opt) + ";" +
                         String.valueOf(dataSet.getID());
      if (arg != null)
        comm = comm + ";" + arg;

      byte[] datum = comm.getBytes();
     
      try {
        config.getEngine().sender.serverCommand(datum);
      } catch (Exception E) {
        System.err.println("Error: connection to server");
      }

    } else 
      config.getEngine().getGUI().message("Only works in the server.", true);
  }


  // animation listener
  private void animateListen() {
    if (!dataSet.isPrivate()) {
      String comm = String.valueOf(SV2Action.ANIMATE) + ";" +
                                     String.valueOf(dataSet.getID());

      byte[] datum = comm.getBytes();
     
      try {
        config.getEngine().sender.broadcastCommand(datum);
      } catch (Exception E) {
        System.err.println("Error: connection to server");
      }

      if (config.getEngine().sender.getIsMaster()) 
        config.getEngine().vmanager.animateAction(identification);
      else
        config.getEngine().getGUI().message("(Slave)Event action to shared data was disabled.", true);
    } else
      config.getEngine().vmanager.animateAction(identification);
  }

  // stop listener
  private void stopListen() {
    if (!dataSet.isPrivate()) {
      String comm = String.valueOf(SV2Action.STOP) + ";" +
               String.valueOf(dataSet.getID()) + ";" + 
               String.valueOf(0);

      byte[] datum = comm.getBytes();
     
      try {
        config.getEngine().sender.broadcastCommand(datum);
      } catch (Exception E) {
        System.err.println("Error: connection to server");
      }

      if (config.getEngine().sender.getIsMaster()) 
        config.getEngine().vmanager.stopAction(identification, 0);
      else
        config.getEngine().getGUI().message("(Slave)Event action to shared data was disabled.", true);
    } else
      config.getEngine().vmanager.stopAction(identification, 0);

  }

  // prepare image for the rotation
  private void prepareImages() {
    message("Wait for receiving images.", false);

    double[] rayColor = ((Config3D)config).getRayColor();
    double r = rayColor[0];
    double g = rayColor[1];
    double b = rayColor[2];

    double rotx, roty;

    int sz = imageStorage.size();
    Vector vec = new Vector();
    boolean flag = false;

    if (rotate_mode == XROTATE) {
      roty = ((Config3D)config).getRotationY();
      requested_value = roty;
System.out.println("requesting value(y) : " + roty);

      for (rotx=-180.0; rotx<180.0; rotx=rotx+30.0) {
        for (int i=0; i<sz; i++) {
          ImageInfo info = (ImageInfo)imageStorage.elementAt(i);
          if (info.contains(r, g, b, rotx, roty)) {
            flag = true;
            break;
          }
	}

        if (!flag) 
          vec.addElement(new Double(rotx));
        else
          flag = false;
      }

    } else if (rotate_mode == YROTATE) {
      rotx = ((Config3D)config).getRotationX();
      requested_value = rotx; 
System.out.println("requesting value(x) : " + rotx);
      for (roty=-180.0; roty<180; roty=roty+30.0) {
        for (int i=0; i<sz; i++) {
          ImageInfo info = (ImageInfo)imageStorage.elementAt(i);
          if (info.contains(r, g, b, rotx, roty)) {
            flag = true;
            break;
          }
	}
 
        if (!flag) 
          vec.addElement(new Double(roty));
        else 
          flag = false;
      }
    }

    if (vec.size() < 1) animateListen();
    else {
      double[] vals = new double[vec.size()];
      for (int i=0; i<vals.length; i++)
        vals[i] = ((Double)vec.elementAt(i)).doubleValue();
      
      String comm = String.valueOf(SV2Action.VOLBULK_REQUEST) + ";" +
           String.valueOf(dataSet.getID()) + ";" + 
           String.valueOf(getID()) + ";" +
           String.valueOf(rotate_mode) + ";" +
	   String.valueOf(requested_value) + ";" +
           String.valueOf(vals.length);
      for (int i=0; i<vals.length; i++) 
        comm += ";" +  String.valueOf(vals[i]);

      byte[] datum = comm.getBytes();

      try {
        config.getEngine().sender.serverCommand(datum);
      } catch (Exception E) {
        System.err.println("Error: connection to server");
      }
    }
  }

  // request image to the server
  private int requestImage(double r, double g, double b, double rotx, double roty) {

    int sz = imageStorage.size();
    for (int i=0; i<sz; i++) {
      ImageInfo info = (ImageInfo)imageStorage.elementAt(i);
      if (info.contains(r, g, b, rotx, roty)) {
System.out.println("Found at the cache ...");
        return i;
      }
    }

    String comm = String.valueOf(SV2Action.VOLVALUE_CHANGE) + ";" +
            String.valueOf(dataSet.getID()) + ";" + 
            String.valueOf(getID()) + ";" +
            String.valueOf(r) + ";" +
            String.valueOf(g) + ";" +
            String.valueOf(b) + ";" +
            String.valueOf(rotx) + ";" +
            String.valueOf(roty);

    byte[] datum = comm.getBytes();

    try {
      config.getEngine().sender.serverCommand(datum);
    } catch (Exception E) {
      System.err.println("Error: connection to server");
    }

    return -1;
  }

  // create the image
  private Image createImage(File file) {
    BufferedInputStream in = null;
    try {
      in = new BufferedInputStream(new FileInputStream(file));
      
    } catch (Exception e) {
      System.err.println("No image..");
      return null;
    }
    ImageProducer producer = new PPMImgProducer(in);
    return getToolkit().createImage(producer);
  }

  // save image
  private int saveImage(File path, byte[] arr, double[] vars) {
    int sz = 0;

    try {
      sz = imageStorage.size();
      File file = new File(path, 
                file_prefix + "_" + String.valueOf(sz) + ".ppm");
      imageStorage.addElement(new ImageInfo(vars, file));      

      BufferedOutputStream out = new BufferedOutputStream(new 
                                          FileOutputStream(file));

      String str = "P6\n";
      out.write(str.getBytes());

      str = "256 256\n";
      out.write(str.getBytes());
      
      str = "255\n";
      out.write(str.getBytes());
      
      out.write(arr, 0, arr.length);

      out.close();
    } catch (Exception e) {
      System.out.println("Writing ppm file error.");
    }

    return sz;
  }

  class ImageInfo {
    double red;
    double green;
    double blue;
    double rotx;
    double roty;
    File filename;

    ImageInfo(double[] vars, File file) {
      filename = file;
      red = vars[0];
      green = vars[1];
      blue = vars[2];
      rotx = vars[3];
      roty = vars[4];
System.out.println("ImageInfo: " + rotx + " " + roty);
    }

    boolean contains(double r, double g, double b, double x, double y) {
      if (r == red && g == green && b == blue && x == rotx && y == roty) 
        return true;
      else 
        return false;
    }
  }

}  // end of class

