package sv.viewer.builtin;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.Text2D;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

import sv.data.*;
import sv.util.*;

// for java 3d version

public class Positions {
  private BranchGroup group;
  private BranchGroup xsliceGroup;  // 3D ONLY 
  private BranchGroup ysliceGroup;  // 3D ONLY 
  private BranchGroup zsliceGroup;  // 3D ONLY 
  private BranchGroup contourGroup; // 2D ONLY 

  private Switch objSwitch;
  private Switch boundSwitch;
  private Switch xsliceSwitch;      // 3D ONLY
  private Switch ysliceSwitch;      // 3D ONLY
  private Switch zsliceSwitch;      // 3D ONLY
  private Switch contourSwitch;     // 3D ONLY

  private BitSet objMask;
  private BitSet boundMask;
  private BitSet xsliceMask;        // 3D ONLY
  private BitSet ysliceMask;        // 3D ONLY
  private BitSet zsliceMask;        // 3D ONLY
  private BitSet contourMask;       // 3D ONLY

  private int prevpos = 0;
  private TransformGroup tgroup;

  public Positions(DataSet[] dataSet, int dim) {
    initialize(dataSet, dim);
  }

  /**
   * regenerate the data set display 
   */
  public void changeDisplayOption(DataSet[] dataSet, int dim) {
      initialize(dataSet, dim);

      if (dim == TimeData.TWOD) {
         if (contourGroup != null) {
            contourGroup.detach();
            group.setChild(contourGroup, 2);
         }

      } else if (dim == TimeData.THREED) {
         if (xsliceGroup != null) {
            xsliceGroup.detach();
            group.setChild(xsliceGroup, 2);
         }
         if (ysliceGroup != null) {
            ysliceGroup.detach();
            group.setChild(ysliceGroup, 3);
         }
         if (zsliceGroup != null) {
            zsliceGroup.detach();
            group.setChild(zsliceGroup, 4);
         } 
      }
  }


  // create the switch 
  private void initialize(DataSet[] dataSet, int dim) {
      // Define colors
      Color3f bgColor = new Color3f(0.05f, 0.05f, 0.2f);

      // create the Switch
      objSwitch = new Switch(Switch.CHILD_MASK);
      objSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
      objSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
      boundSwitch = new Switch(Switch.CHILD_MASK);
      boundSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
      boundSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);

      objMask = new BitSet();
      boundMask = new BitSet();

      // create the time data object
      // BUT ASSUME THAT ONLY ONE DATASET IS ALLOWED
      // AND THE DATASET HAS THE SORTED TIME DATA
      if (dim == TimeData.TWOD) {

         for (int j=0; j<dataSet[0].getNoOfTimes(); j++) {
         
            TimeData2D timeData = (TimeData2D)dataSet[0].getTimeData(j);
 
            Transform3D transform3D = new Transform3D();
            TransformGroup transformGroup = new TransformGroup(transform3D);

            if (timeData.getJ3dObject() != null) {
               Shape3D s3d = timeData.getJ3dObject().getChild();
               transformGroup.addChild(s3d);
            }

            objSwitch.addChild(transformGroup);
            objMask.clear(j);
         }

         for (int j=0; j<dataSet[0].getNoOfTimes(); j++) {

            TimeData2D timeData = (TimeData2D)dataSet[0].getTimeData(j);
 
            Transform3D transform3D = new Transform3D();
            TransformGroup transformGroup = new TransformGroup(transform3D);

            if (timeData.getJ3dObject() != null) {
               Shape3D s3d = timeData.getJ3dObject().getBoundChild();
               transformGroup.addChild(s3d);
            }

            boundSwitch.addChild(transformGroup);
            boundMask.clear(j);
         }

      } else if (dim == TimeData.THREED) {

         for (int j=0; j<dataSet[0].getNoOfTimes(); j++) {
         
            TimeData3D timeData = (TimeData3D)dataSet[0].getTimeData(j);
 
            Transform3D transform3D = new Transform3D();
            TransformGroup transformGroup = new TransformGroup(transform3D);

            if (timeData.getJ3dObject() != null) {
               Shape3D s3d = timeData.getJ3dObject().getChild();
	       // timeData.j3dObject.setAppearance(redAppearance);
               transformGroup.addChild(s3d);
            }

            objSwitch.addChild(transformGroup);
            objMask.clear(j);
         }

         for (int j=0; j<dataSet[0].getNoOfTimes(); j++) {

            TimeData3D timeData = (TimeData3D)dataSet[0].getTimeData(j);
   
            Transform3D transform3D = new Transform3D();
            TransformGroup transformGroup = new TransformGroup(transform3D);

            if (timeData.getJ3dObject() != null) {
               Shape3D s3d = timeData.getJ3dObject().getBoundChild();
               transformGroup.addChild(s3d);
            }

            boundSwitch.addChild(transformGroup);
            boundMask.clear(j);
         }

      } 

      objSwitch.setChildMask(objMask);
      boundSwitch.setChildMask(boundMask);

      group = new BranchGroup();
      group.setCapability(BranchGroup.ALLOW_DETACH);
      group.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
      group.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
      group.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

      // First 5 nodes are reserved to control this branch group
      for (int i=0; i<5; i++) {
         BranchGroup dummy = new BranchGroup();
         dummy.setCapability(BranchGroup.ALLOW_DETACH);
         group.addChild(dummy);
      }

      group.setChild(objSwitch, 0);
      group.setChild(boundSwitch, 1);
  }

  /**
   * make slice data for 3d (3D ONLY)
   */
  public void makeSliceData(int mode, DataSet dataSet) {
    if (mode == TimeData3D.XSKIPED) {
       xsliceGroup = new BranchGroup();
       xsliceGroup.setCapability(BranchGroup.ALLOW_DETACH);
  
       // create the Switch
       xsliceSwitch = new Switch(Switch.CHILD_MASK);
       xsliceSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
       xsliceSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
  
       xsliceMask = new BitSet();
  
       // create the slice data object
       // BUT ASSUME THAT ONLY ONE DATASET IS ALLOWED
       // AND THE DATASET HAS THE SORTED TIME DATA
  
       for (int j=0; j<dataSet.getNoOfTimes(); j++) {
          TimeData3D timeData = (TimeData3D)dataSet.getTimeData(j);
  
          Transform3D transform3D = new Transform3D();
          TransformGroup transformGroup = new TransformGroup(transform3D);
  
          if (timeData.getXSliceObjects() != null) 
             transformGroup.addChild(timeData.getXSliceObjects().getChild());
          else
  
          xsliceSwitch.addChild(transformGroup);
          xsliceMask.clear(j);
       }
  
       xsliceSwitch.setChildMask(xsliceMask);
       xsliceGroup.addChild(xsliceSwitch);
       group.setChild(xsliceGroup, 2);
  
    ////////////////////////////////////////////////////////////////////
    } else if (mode == TimeData3D.YSKIPED) {
  
       ysliceGroup = new BranchGroup();
       ysliceGroup.setCapability(BranchGroup.ALLOW_DETACH);
  
       // create the Switch
       ysliceSwitch = new Switch(Switch.CHILD_MASK);
       ysliceSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
       ysliceSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
  
       ysliceMask = new BitSet();
  
       // create the slice data object
       // BUT ASSUME THAT ONLY ONE DATASET IS ALLOWED
       // AND THE DATASET HAS THE SORTED TIME DATA
  
       for (int j=0; j<dataSet.getNoOfTimes(); j++) {
    
          TimeData3D timeData = (TimeData3D)dataSet.getTimeData(j);
  
          Transform3D transform3D = new Transform3D();
          TransformGroup transformGroup = new TransformGroup(transform3D);
  
          if (timeData.getYSliceObjects() != null) 
             transformGroup.addChild(timeData.getYSliceObjects().getChild());
  
          ysliceSwitch.addChild(transformGroup);
          ysliceMask.clear(j);
       }
  
       ysliceSwitch.setChildMask(ysliceMask);
       ysliceGroup.addChild(ysliceSwitch);
       group.setChild(ysliceGroup, 3);
  
    ////////////////////////////////////////////////////////////////////
    } else if (mode == TimeData3D.ZSKIPED) {
  
       zsliceGroup = new BranchGroup();
       zsliceGroup.setCapability(BranchGroup.ALLOW_DETACH);
  
       // create the Switch
       zsliceSwitch = new Switch(Switch.CHILD_MASK);
       zsliceSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
       zsliceSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
  
       zsliceMask = new BitSet();
  
       // create the slice data object
       // BUT ASSUME THAT ONLY ONE DATASET IS ALLOWED
       // AND THE DATASET HAS THE SORTED TIME DATA
  
       for (int j=0; j<dataSet.getNoOfTimes(); j++) {
    
          TimeData3D timeData = (TimeData3D)dataSet.getTimeData(j);
  
          Transform3D transform3D = new Transform3D();
          TransformGroup transformGroup = new TransformGroup(transform3D);
  
          if (timeData.getZSliceObjects() != null) 
             transformGroup.addChild(timeData.getZSliceObjects().getChild());
  
          zsliceSwitch.addChild(transformGroup);
          zsliceMask.clear(j);
       }
  
       zsliceSwitch.setChildMask(zsliceMask);
       zsliceGroup.addChild(zsliceSwitch);
       group.setChild(zsliceGroup, 4);
    }
  }


  /**
   * make contour for 2d (2D ONLY)
   */
  public void makeContourObject(DataSet dataSet) {
      contourGroup = new BranchGroup();
      contourGroup.setCapability(BranchGroup.ALLOW_DETACH);

      // create the Switch
      contourSwitch = new Switch(Switch.CHILD_MASK);
      contourSwitch.setCapability(Switch.ALLOW_SWITCH_READ);
      contourSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);

      contourMask = new BitSet();

      // create the slice data object
      // BUT ASSUME THAT ONLY ONE DATASET IS ALLOWED
      // AND THE DATASET HAS THE SORTED TIME DATA

      int sz = dataSet.getNoOfTimes();
      for (int j=0; j<sz; j++) {
   
         TimeData2D timeData = (TimeData2D)dataSet.getTimeData(j);
         ContourObject[] contourObjects = timeData.getContourObject();

         Transform3D transform3D = new Transform3D();
         TransformGroup transformGroup = new TransformGroup(transform3D);

         if (contourObjects != null) { 
	   for (int i=0; i<contourObjects.length; i++) {
             transformGroup.addChild(contourObjects[i].getChild());
	   }
         }

         contourSwitch.addChild(transformGroup);
         contourMask.clear(j);
      }

      contourSwitch.setChildMask(contourMask);
      contourGroup.addChild(contourSwitch);
      group.setChild(contourGroup, 2);
  }


  public void setTransformGroup(TransformGroup transformGroup) {
     tgroup = transformGroup;
  }

  public BranchGroup getChild() {
     return group;
  }
 
  /**
   * show current time step
   * @param pos cuttunt time position
   * @param b if true, show the bounding box, too.
   */
  public void set(int pos, boolean bound) {
     objMask.clear(prevpos);     
     objMask.set(pos);
     objSwitch.setChildMask(objMask);

     boundMask.clear(prevpos);   
     if (bound) 
        boundMask.set(pos);

     boundSwitch.setChildMask(boundMask);

     prevpos = pos;

     // The following three lines fix a bug in J3D
     Transform3D t = new Transform3D();
     tgroup.getTransform(t);
     tgroup.setTransform(t);
  }

  /**
   * show current time step contour 
   * (2D ONLY, SHOULD BE CALLED BEFORE set(int, boolean))
   * @param pos cuttunt time position
   */
  public void setContour(int pos) {
     contourMask.clear(prevpos);     
     if (pos >= 0)
        contourMask.set(pos);
  
     contourSwitch.setChildMask(contourMask);
  }


  /**
   * show current time step slice 
   * (3D ONLY, SHOULD BE CALLED BEFORE set(int, boolean))
   * @param mode 1-xgrid, 2-ygrid, 3-zgrid
   * @param pos cuttunt time position
   */
  public void setSlice(int mode, int pos) {
    //System.out.println("setslice " + mode + ", " + pos);
     if (mode == 1) {
        xsliceMask.clear(prevpos);     
        if (pos >= 0)
           xsliceMask.set(pos);
  
        xsliceSwitch.setChildMask(xsliceMask);

     } else if (mode == 2) {
        ysliceMask.clear(prevpos);     
        if (pos >= 0)
           ysliceMask.set(pos);
  
        ysliceSwitch.setChildMask(ysliceMask);

     } else if (mode == 3) {
        zsliceMask.clear(prevpos);     
        if (pos >= 0)
           zsliceMask.set(pos);
  
        zsliceSwitch.setChildMask(zsliceMask);
     }
  }

  /**
   * detach branch group
   */
  public void detach() {
     group.detach();
  }

} // end of class
