package sv.viewer.builtin;

import javax.media.j3d.*;
import javax.vecmath.*;
import sv.util.GridInfo3D;
import sv.data.TimeData3D;

// for java3d version

public class SliceObject {

  private Shape3D shape3D;

  public SliceObject(GridInfo3D grids, int dimx, int dimy, int dimz, 
                     float zmin, float zmax, 
                     float[] rcolors, float[] gcolors, float[] bcolors) {
     int gridsize = grids.getSize();
     int extraction = grids.getExtraction();
     int index = 0, idx = 0;
     int cval;

     float xval, yval, zval;

     float xinc = (2.0f) / (float)dimx;
     float yinc = (2.0f) / (float)dimy;
     float zinc = (2.0f) / (float)dimz;


     Point3f[] verts = null;
     Color3f[] vcolors = null;
     Vector3f[] normals = null;
     Vector3f norm;


     /////////////////////////////////////////////////////////////////////////
     if (grids.getSkipmode() == TimeData3D.XSKIPED) {
        xinc = (2.0f) / (float)(dimx-1);

        verts = new Point3f[dimy * dimz * 4];
        vcolors = new Color3f[verts.length];
	normals = new Vector3f[dimy * dimz];
        norm = new Vector3f(-1.0f, 0.0f, 0.0f);

        for (int iy=0; iy<dimy; iy++) {
	  for (int iz=0; iz<dimz; iz++) {
             cval = (int)((grids.getValue(idx)-zmin) / (zmax-zmin) * 100f);
             Color3f color = new Color3f(rcolors[cval], gcolors[cval], bcolors[cval]);
             xval = xinc * (float)extraction - 1.0f;  
             yval = yinc * (float)iy - 1.0f;
             zval = zinc * (float)iz - 1.0f;

             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;
            
             yval = yinc * (float)(iy+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             zval = zinc * (float)(iz+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             yval = yinc * (float)iy - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             normals[idx] = norm;
             index++;
             idx++;
	  }
	}
        

     /////////////////////////////////////////////////////////////////////////
     } else if (grids.getSkipmode() == TimeData3D.YSKIPED) {
        yinc = (2.0f) / (float)(dimy-1);

        verts = new Point3f[dimx * dimz * 4];
        vcolors = new Color3f[verts.length];
	normals = new Vector3f[dimx * dimz];
        norm = new Vector3f(0.0f, -1.0f, 0.0f);

        for (int ix=0; ix<dimx; ix++) {
	  for (int iz=0; iz<dimz; iz++) {
             cval = (int)((grids.getValue(idx)-zmin) / (zmax-zmin) * 100f);
             Color3f color = new Color3f(rcolors[cval], gcolors[cval], bcolors[cval]);
             xval = xinc * (float)ix - 1.0f;  
             yval = yinc * (float)extraction - 1.0f;
             zval = zinc * (float)iz - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;
            
             xval = xinc * (float)(ix+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             zval = zinc * (float)(iz+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             xval = xinc * (float)ix - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             normals[idx] = norm;
             index++;
             idx++;
	  }
	}
        

     /////////////////////////////////////////////////////////////////////////
     } else if (grids.getSkipmode() == TimeData3D.ZSKIPED) {
        zinc = (2.0f) / (float)(dimz-1);

        verts = new Point3f[dimx * dimy * 4];
        vcolors = new Color3f[verts.length];
	normals = new Vector3f[dimx * dimy];
        norm = new Vector3f(0.0f, 0.0f, -1.0f);

        for (int ix=0; ix<dimx; ix++) {
	  for (int iy=0; iy<dimy; iy++) {
             cval = (int)((grids.getValue(idx)-zmin) / (zmax-zmin) * 100f);
             Color3f color = new Color3f(rcolors[cval], gcolors[cval], bcolors[cval]);
             xval = xinc * (float)ix - 1.0f;  
             yval = yinc * (float)iy - 1.0f;
             zval = zinc * (float)extraction -1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;
            
             xval = xinc * (float)(ix+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             yval = yinc * (float)(iy+1) - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             index++;

             xval = xinc * (float)ix - 1.0f;
             verts[index] = new Point3f(xval, yval, zval);
             vcolors[index] = color;
             normals[idx] = norm;
             index++;
             idx++;
	  }
	}
        
     ////////////////////////////////////////////////////////////////////////
     } else {
        System.out.println("Unknown axis ...");
        return;
     }

     QuadArray  quadArray = new QuadArray(verts.length, QuadArray.COORDINATES | QuadArray.COLOR_3 | QuadArray.NORMALS);

     quadArray.setCoordinates(0, verts);
     quadArray.setColors(0, vcolors);
     quadArray.setNormals(0, normals);

     Appearance app = new Appearance();

        // Set up the rendering attributes
	//        RenderingAttributes ra = new RenderingAttributes(true, true, 0.5f, RenderingAttributes.GREATER);
	//System.out.println(ra.getDepthBufferEnable());
	//System.out.println(ra.getDepthBufferWriteEnable());
	//System.out.println(ra.getAlphaTestValue());
	//System.out.println(ra.getAlphaTestFunction());
	//        app.setRenderingAttributes(ra);

     PolygonAttributes pa = new PolygonAttributes();
     pa.setCullFace(pa.CULL_NONE);
     app.setPolygonAttributes(pa);

     shape3D = new Shape3D(quadArray);
     shape3D.setAppearance(app);
  }

  public Shape3D getChild() {
     return shape3D;
  }

} // end of class


