package sv.viewer.builtin;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import sv.gui.*;

/**
 * for Wheel manipulators.
 */

public class WheelBehavior extends Behavior {
  int x, y, x_last, y_last;
  double x_angle, y_angle;
  double x_factor, y_factor;

  TransformGroup transformGroup;
  Transform3D transformX;
  Transform3D transformY;
  boolean reset = false;
  boolean debug = true;

  /**
   * Initializes standard fields.
   * @param transformGroup The transform group to be manipulated.
   */
  public WheelBehavior(TransformGroup transformGroup) {
    super();

    this.transformGroup = transformGroup;
    transformX = new Transform3D();
    transformY = new Transform3D();
    reset = true;
  }
 
  /** 
   * Swap a new transformGroup replacing the old one. This allows 
   * manipulators to operate on different nodes.
   * 
   * @param transformGroup The *new* transform group to be manipulated.
   */
  public void setTransformGroup(TransformGroup transformGroup){
    // need to remove old behavior from group 
    this.transformGroup = transformGroup;
    transformX = new Transform3D();
    transformY = new Transform3D();
    reset = true;
  }

  /** Initializes the behavior.
   */
  public void initialize() {
    x = y = 0;
    x_last = y_last = 0;

    x_angle = 0;
    y_angle = 0;
    x_factor = 0.03;
    y_factor = 0.03;
  }
  
  /** 
   * Manually wake up the behavior. 
   */
  public void wakeup(Wheel wheel, int delta) {

      if (wheel.getOrientation () == Wheel.HORIZONTAL) {
         y = y_last + delta;
         y_angle = delta * y_factor;         
 
         transformY.rotY(y_angle);

         Transform3D currXform = new Transform3D();
         transformGroup.getTransform(currXform);

         Matrix4d mat = new Matrix4d();

         // remember old matrix
         currXform.get(mat);

         // translate to origin
         currXform.setTranslation(new Vector3d(0.0, 0.0, 0.0));
         currXform.mul(transformY, currXform);
         
         // set old translation back
         Vector3d translation = new Vector3d(mat.m03, mat.m13, mat.m23);
         currXform.setTranslation(translation);

         transformGroup.setTransform(currXform);
         y_last = y;

      } else {

         x = x_last + delta;
         x_angle = delta * x_factor;         
 
         transformX.rotX(x_angle);

         Transform3D currXform = new Transform3D();
         transformGroup.getTransform(currXform);

         Matrix4d mat = new Matrix4d();

         // remember old matrix
         currXform.get(mat);

         // translate to origin
         currXform.setTranslation(new Vector3d(0.0, 0.0, 0.0));
         currXform.mul(transformX, currXform);
         
         // set old translation back
         Vector3d translation = new Vector3d(mat.m03, mat.m13, mat.m23);
         currXform.setTranslation(translation);

         transformGroup.setTransform(currXform);
         x_last = x;

      }
  }

  /**
   * All mouse manipulators must implement this.
   */
  public void processStimulus (Enumeration criteria) {
     System.out.println("processStimulus");
  }
}



